package com.xxh.cloud.dw.healthy.engine.modules.data.convert;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Dict;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.*;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.utils.StrSqlUtils;
import com.xxh.cloud.healthy.scale.framework.modules.client.api.bean.bos.ScaleResultBO;
import com.xxh.cloud.healthy.serve.label.client.api.bean.bos.PatientLabelBO;
import com.xxh.cloud.healthy.serve.label.client.api.bean.bos.PatientLabelHistoryBO;
import com.xxh.cloud.metadata.framework.modules.common.constant.enums.MetadataStandardResultTypeEnum;
import com.xxh.cloud.metadata.framework.modules.common.constant.enums.RuleCondtionDataTypeEnum;
import com.xxh.cloud.rule.basic.context.XXRuleRoamHealthItem;
import com.xxh.cloud.rule.basic.context.XXRuleRoamHealthItemHelper;
import com.xxh.cloud.rule.basic.context.bean.*;
import io.swagger.annotations.ApiModelProperty;

import java.util.*;

public class RuleRoamHealthDataConvert {



    /**
     * 就诊记录数据 时间(yyyy-mm-dd)：
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getVisitRecord(List<DwdVisitRecordInfo> itemList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();

        for(DwdVisitRecordInfo item:itemList){
            if(item.getVisitDate()!=null){
                String dataKey= DateUtil.format(item.getVisitDate(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                list.add(getVisitRecord(item));
            }
        }
        return data;
    }

    /**
     * 就诊记录数据
     * */
    public static XXRuleRoamHealthItem getVisitRecord(DwdVisitRecordInfo item){
        RoamItemDwdVisitRecord val=new RoamItemDwdVisitRecord();
        BeanUtil.copyProperties(item,val);
        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.VisitRecord.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.VisitRecord.getCode(), item.getId()),
                val,
                item.getDepartmentCodeStandard(),
                item.getVisitDate()
        );
        return data;
    }





    /**
     * 挂号记录
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getRegisterRecord(List<DwdRegisterRecordInfo> itemList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(DwdRegisterRecordInfo item:itemList){
            if(item.getRegDate()!=null){
                String dataKey= DateUtil.format(item.getRegDate(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                list.add(getRegisterRecord(item));
            }
        }
        return data;
    }

    /**
     * 挂号记录
     * */
    public static XXRuleRoamHealthItem getRegisterRecord(DwdRegisterRecordInfo item){
        RoamItemDwdRegisterRecord val=new RoamItemDwdRegisterRecord();
        BeanUtil.copyProperties(item,val);
        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.RegisterRecord.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.RegisterRecord.getCode(), item.getId()),
                val,
                item.getDepartmentCodeStandard(),
                item.getRegDate());
        return data;
    }


    /**
     * 诊断数据
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getDiagnosisReport(List<DwdDiagnosisReportInfo> itemList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(DwdDiagnosisReportInfo item:itemList){
            if(item.getDiagnosisTime()!=null){
                String dataKey= DateUtil.format(item.getDiagnosisTime(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                list.add(getDiagnosisReport(item));
            }
        }
        return data;
    }


    /**
     * 诊断数据
     * */
    public static XXRuleRoamHealthItem getDiagnosisReport(DwdDiagnosisReportInfo item){
        RoamItemDwdDiagnosisReport val=new RoamItemDwdDiagnosisReport();
        BeanUtil.copyProperties(item,val);

        if(item.getSortIdx()!=null){
            if(item.getSortIdx().intValue()==1){
                val.setMainFlag("1");
            }else {
                val.setMainFlag("0");
            }
        }else {
            val.setMainFlag(item.getMainFlag());
        }

        val.setSceneTypeRaw(item.getSceneTypeRaw());
        val.setSceneTypeStandard(item.getSceneTypeStandard());


        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Diagnosis.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Diagnosis.getCode(), item.getDiagnosisCodeStandard(),item.getSceneTypeStandard()),
                val,
                item.getDepartmentCodeStandard(),
                item.getDiagnosisTime()
        );
        return data;
    }


    /**
     *  预约住院信息
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getInpatientReserve(List<DwdInpatientReserveInfo>  itemList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(DwdInpatientReserveInfo item:itemList){
            if(item.getApplyDate()!=null){
                String dataKey= DateUtil.format(item.getApplyDate(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                list.add(getInpatientReserve(item));
            }
        }
        return data;
    }

    /**
     *  预约住院信息
     * */
    public static XXRuleRoamHealthItem getInpatientReserve(DwdInpatientReserveInfo item){
        RoamItemDwdInpatientReserve val=new RoamItemDwdInpatientReserve();
        BeanUtil.copyProperties(item,val,"diagnosisCodeStandard");
        val.setDiagnosisCodeStandard(StrJoinSplitUtils.split2List(item.getDiagnosisCodeStandard()));

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Reserve_Inpatient_Not.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Reserve_Inpatient_Not.getCode(), item.getId()),
                val,
                item.getApplyDepartmentCodeStandard(),
                item.getBookDate()
        );
        return data;
    }


    /**
     * 住院记录
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getInpatientRecord(List<DwdInpatientRecordInfo>  itemList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(DwdInpatientRecordInfo item:itemList){
            if(item.getInpatientDate()!=null){
                String dataKey= DateUtil.format(item.getInpatientDate(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                list.add(getInpatientRecord(item));
            }
        }
        return data;
    }

    /**
     * 住院记录
     * */
    public static XXRuleRoamHealthItem getInpatientRecord(DwdInpatientRecordInfo item){
        RoamItemDwdInpatientRecord val=new RoamItemDwdInpatientRecord();
        BeanUtil.copyProperties(item,val,"diagnosisCodeStandard");
        val.setDiagnosisCodeStandard(StrJoinSplitUtils.split2List(item.getDiagnosisCodeStandard()));

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Inpatient_Not.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Inpatient_Not.getCode(), item.getId()),
                val,
                item.getAdmitDepartmentCodeStandard(),
                item.getInpatientDate()
        );
        return data;
    }

    /**
     * 住院变更记录
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getInpatientChange(List<DwdInpatientChangeInfo>  itemList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(DwdInpatientChangeInfo item:itemList){
            if(item.getChangeTime()!=null){
                Date _changeTime= Convert.toDate(item.getChangeTime());
                String dataKey= DateUtil.format(_changeTime, DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                list.add(getInpatientChange(item));
            }
        }
        return data;
    }

    /**
     * 住院变更记录
     * */
    public static  XXRuleRoamHealthItem getInpatientChange(DwdInpatientChangeInfo item){

        RoamItemDwdInChangeRecord val=new RoamItemDwdInChangeRecord();
        val.setPatientId(item.getPatientId());
        val.setId(item.getChangeId());
        val.setInId(item.getInpatientId());
        val.setChangeTime(item.getChangeTime());
        val.setChangeType(item.getChangeType());
        val.setChangeCode(item.getCode());
        val.setChangeName(item.getName());
        val.setSourceCode(item.getSourceCode());
        val.setTargetCode(item.getTargetCode());


        if("ZKZCKS".equals(item.getChangeType())){
            val.setDataType(MetadataStandardResultTypeEnum.MSRT_Out_Dept.getCode());
        }
        if("ZKZRKS".equals(item.getChangeType())){
            val.setDataType(MetadataStandardResultTypeEnum.MSRT_In_Dept.getCode());
        }



        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Change_Dept.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Change_Dept.getCode(), item.getChangeId()),
                val,
                item.getCode(),
                Convert.toDate(item.getChangeTime())
        );

        return data;
    }

    //



    /**
     * 出院记录
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getDischargeRecord(List<DwdDischargeRecordInfo> itemList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(DwdDischargeRecordInfo item:itemList){
            if(item.getLeaveDate()!=null){
                String dataKey= DateUtil.format(item.getLeaveDate(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                list.add(getDischargeRecord(item));
            }
        }
        return data;
    }

    /**
     * 出院记录
     * */
    public static XXRuleRoamHealthItem getDischargeRecord(DwdDischargeRecordInfo item){
        RoamItemDwdDischargeRecord val=new RoamItemDwdDischargeRecord();
        BeanUtil.copyProperties(item,val,"contentStandard","diagnosisCodeStandard");
//        val.setContentStandard(StrJoinSplitUtils.split2List(item.getContentStandard()));
        val.setDiagnosisCodeStandard(StrJoinSplitUtils.split2List(item.getDiagnosisCodeStandard()));
        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Discharge.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Discharge.getCode(), item.getId()),
                val,
                item.getAdmitDepartmentCodeStandard(),
                item.getLeaveDate());

        return data;
    }


    /**
     * 检查预约记录
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getExamineReserve(List<DwdExamineReserveInfo> itemList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(DwdExamineReserveInfo item:itemList){
            if(item.getBookDate()!=null){
                String dataKey= DateUtil.format(item.getBookDate(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                list.add(getExamineReserve(item));
            }
        }
        return data;
    }

    /**
     * 检查预约记录
     * */
    public static XXRuleRoamHealthItem getExamineReserve(DwdExamineReserveInfo item){
        RoamItemDwdExamineReserve val=new RoamItemDwdExamineReserve();
        BeanUtil.copyProperties(item,val,"itemsStandard");
        val.setItemsStandard(StrJoinSplitUtils.split2List(item.getItemsStandard()));

        //todo:

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Reserve_Examine.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Reserve_Examine.getCode(), item.getId()),
                val,
                item.getApplyDepartmentCodeStandard(),
                item.getBookDate());
        return data;
    }


    /**
     * 检查申请
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getExamineApply(List<DwdExamineApplyInfo> itemList,List<DwdExamineApplyItemInfo> dwdExamineApplyItemList){
        Map<String,List<String>> projectMap=new HashMap<>();
        dwdExamineApplyItemList.forEach(item->{
            List<String> list=projectMap.get(item.getApplyId());
            if(list==null){
                list=new ArrayList<>();
                projectMap.put(item.getApplyId(),list);
            }
            list.add(item.getCodeStandard());
        });



        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(DwdExamineApplyInfo item:itemList){
            if(item.getApplyDate()!=null){
                String dataKey= DateUtil.format(item.getApplyDate(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                list.add(getExamineApply(item,projectMap));
            }
        }
        return data;
    }


    /**
     * 检查申请
     * */
    public static XXRuleRoamHealthItem getExamineApply(DwdExamineApplyInfo item,Map<String,List<String>> projectMap){
        RoamItemDwdExamineApply val=new RoamItemDwdExamineApply();
        BeanUtil.copyProperties(item,val,"itemsStandard","diagnosisCodeStandard");
        val.setDiagnosisCodeStandard(StrJoinSplitUtils.split2List(item.getDiagnosisCodeStandard()));
        if(projectMap!=null){
            val.setItemsStandard(projectMap.get(item.getId()));
        }
        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Apply_Examine.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Apply_Examine.getCode(), item.getId()),
                val,
                item.getDepartmentCodeStandard(),
                item.getApplyDate());
        return data;
    }


    /**
     * 检查报告单
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getExamineReport(List<DwdExamineReportInfo> itemList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(DwdExamineReportInfo item:itemList){
            if(item.getReportsTime()!=null){
                String dataKey= DateUtil.format(item.getReportsTime(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                list.add(getExamineReport(item));
            }
        }
        return data;
    }

    /**
     * 检查报告单
     * */
    public static XXRuleRoamHealthItem getExamineReport(DwdExamineReportInfo item){
        RoamItemDwdExamineReport val=new RoamItemDwdExamineReport();
        BeanUtil.copyProperties(item,val,"reportDescriptionStandard","reportResultStandard");
        val.setReportDescriptionStandard(StrJoinSplitUtils.split2List(item.getReportDescriptionStandard()));
        val.setReportResultStandard(StrJoinSplitUtils.split2List(item.getReportResultStandard()));
        val.setActNo(item.getActivityNo());

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Report_Examine.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Report_Examine.getCode(), item.getId()),
                val,
                item.getDepartmentCodeStandard(),
                item.getReportsTime());
        return data;
    }


//    /**
//     * 病理申请单
//     * */
//    public static Map<String, List<XXRuleRoamHealthItem>> getExamineApply_Pathology(List<DwdExamineApplyInfo> itemList){
//        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
//        for(DwdExamineApplyInfo item:itemList){
//            if(item.getApplyDate()!=null){
//                String dataKey= DateUtil.format(item.getApplyDate(), DatePattern.PURE_DATE_FORMAT);
//                List<XXRuleRoamHealthItem> list=data.get(dataKey);
//                if(list==null){
//                    list=new ArrayList<>();
//                    data.put(dataKey,list);
//                }
//                list.add(getExamineApply_Pathology(item));
//            }
//        }
//        return data;
//    }
//
//    /**
//     * 病理申请单
//     * */
//    public static XXRuleRoamHealthItem getExamineApply_Pathology(DwdExamineApplyInfo item){
//        RoamItemDwdExamineApply val=new RoamItemDwdExamineApply();
//        BeanUtil.copyProperties(item,val,"itemsStandard","diagnosisCodeStandard");
//        val.setItemsStandard(StrJoinSplitUtils.split2List(item.getItemsStandard()));
//        val.setDiagnosisCodeStandard(StrJoinSplitUtils.split2List(item.getDiagnosisCodeStandard()));
//        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
//                RuleCondtionDataTypeEnum.Apply_Pathology.getCode(),
//                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Apply_Pathology.getCode(), item.getDataUpdateDate()),
//                val,
//                item.getDepartmentCodeStandard(),
//                item.getApplyDate());
//        return data;
//    }
//
//
//
//    /**
//     * 病理报告单
//     * */
//    public static Map<String, List<XXRuleRoamHealthItem>> getExamineReport_Pathology(List<DwdExamineReportInfo> itemList){
//        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
//        for(DwdExamineReportInfo item:itemList){
//            if(item.getReportsTime()!=null){
//                String dataKey= DateUtil.format(item.getReportsTime(), DatePattern.PURE_DATE_FORMAT);
//                List<XXRuleRoamHealthItem> list=data.get(dataKey);
//                if(list==null){
//                    list=new ArrayList<>();
//                    data.put(dataKey,list);
//                }
//                list.add(getExamineReport_Pathology(item));
//            }
//        }
//        return data;
//    }
//
//    /**
//     * 病理报告单
//     * */
//    public static XXRuleRoamHealthItem getExamineReport_Pathology(DwdExamineReportInfo item){
//        RoamItemDwdExamineReport val=new RoamItemDwdExamineReport();
//        BeanUtil.copyProperties(item,val,"reportDescriptionStandard","reportResultStandard");
//        val.setReportDescriptionStandard(StrJoinSplitUtils.split2List(item.getReportDescriptionStandard()));
//        val.setReportResultStandard(StrJoinSplitUtils.split2List(item.getReportResultStandard()));
//
//        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
//                RuleCondtionDataTypeEnum.Report_Pathology.getCode(),
//                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Report_Pathology.getCode(), item.getDataUpdateDate()),
//                val,
//                item.getDepartmentCodeStandard(),
//                item.getReportsTime());
//        return data;
//    }




//    /**
//     * 病理申请单
//     * */
//    public static Map<String, List<XXRuleRoamHealthItem>> getRecipePathologyApply(List<DwdRecipePathologyApplyInfo> itemList ,List<DwdRecipePathologyApplyProjectInfo> projectList){
//        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
//
//        Map<String,List<DwdRecipePathologyApplyProjectInfo>> projectMap=new HashMap<>();
//        projectList.forEach(item->{
//            List<DwdRecipePathologyApplyProjectInfo> list=projectMap.get(item.getRecipePathologyApplyId());
//            if(list==null){
//                list=new ArrayList<>();
//                projectMap.put(item.getRecipePathologyApplyId(),list);
//            }
//            list.add(item);
//        });
//
//
//
//        for(DwdRecipePathologyApplyInfo item:itemList){
//            if(item.getApplyDate()!=null){
//                String dataKey= DateUtil.format(item.getApplyDate(), DatePattern.PURE_DATE_FORMAT);
//                List<XXRuleRoamHealthItem> list=data.get(dataKey);
//                if(list==null){
//                    list=new ArrayList<>();
//                    data.put(dataKey,list);
//                }
//                List<DwdRecipePathologyApplyProjectInfo> childProjectList = projectMap.get(item.getId());
//                list.add(getRecipePathologyApply(item,childProjectList));
//            }
//        }
//
//        return data;
//    }

//    /**
//     * 病理申请单
//     * */
//    public static XXRuleRoamHealthItem getRecipePathologyApply(DwdRecipePathologyApplyInfo item,List<DwdRecipePathologyApplyProjectInfo> itemList){
//        RoamItemDwdRecipePathologyApply val=new RoamItemDwdRecipePathologyApply();
//        BeanUtil.copyProperties(item,val,"itemMap");
//
//        itemList.forEach(itemEntity->{
//            RoamItemDwdRecipePathologyApplyItem itemModel=new RoamItemDwdRecipePathologyApplyItem();
//            BeanUtil.copyProperties(itemEntity,itemModel);
//            val.addItem(itemModel);
//        });
//
//        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
//                RuleCondtionDataTypeEnum.Apply_Pathology.getCode(),
//                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Apply_Pathology.getCode(), item.getOpDate()),
//                val,
//                item.getDepartmentCodeStandard(),
//                item.getApplyDate());
//
//        return data;
//    }


//    /**
//     * 病理报告单
//     * */
//    public static Map<String, List<XXRuleRoamHealthItem>> getRecipePathologyReport(List<DwdRecipePathologyReportInfo> itemList){
//        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
//        for(DwdRecipePathologyReportInfo item:itemList){
//            if(item.getReportsTime()!=null){
//                String dataKey= DateUtil.format(item.getReportsTime(), DatePattern.PURE_DATE_FORMAT);
//                List<XXRuleRoamHealthItem> list=data.get(dataKey);
//                if(list==null){
//                    list=new ArrayList<>();
//                    data.put(dataKey,list);
//                }
//                list.add(getRecipePathologyReport(item));
//            }
//        }
//        return data;
//    }
//
//    /**
//     * 病理报告单
//     * */
//    public static XXRuleRoamHealthItem getRecipePathologyReport(DwdRecipePathologyReportInfo item){
//        RoamItemDwdRecipePathologyReport val=new RoamItemDwdRecipePathologyReport();
//        BeanUtil.copyProperties(item,val);
//
//        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
//                RuleCondtionDataTypeEnum.Report_Pathology.getCode(),
//                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Report_Pathology.getCode(), item.getOpDate()),
//                null,
//                item.getDepartmentCodeStandard(),
//                item.getReportsTime());
//        return data;
//    }




    /**
     *  检验申请单
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getLaboratoryApply(List<DwdLaboratoryApplyInfo> itemList ,List<DwdLaboratoryApplyProjectInfo> projectList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();

        Map<String,List<DwdLaboratoryApplyProjectInfo>> projectMap=new HashMap<>();
        projectList.forEach(item->{
            List<DwdLaboratoryApplyProjectInfo> list=projectMap.get(item.getLaboratoryApplyId());
            if(list==null){
                list=new ArrayList<>();
                projectMap.put(item.getLaboratoryApplyId(),list);
            }
            list.add(item);
        });


        for(DwdLaboratoryApplyInfo item:itemList){
            if(item.getApplyDate()!=null){
                String dataKey= DateUtil.format(item.getApplyDate(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                List<DwdLaboratoryApplyProjectInfo> childProjectList = projectMap.get(item.getId());
                list.add(getLaboratoryApply(item,childProjectList));
            }
        }
        return data;
    }


    /**
     *  检验申请单
     * */
    public static XXRuleRoamHealthItem getLaboratoryApply(DwdLaboratoryApplyInfo item,List<DwdLaboratoryApplyProjectInfo> itemList){
        RoamItemDwdLaboratoryApply val=new RoamItemDwdLaboratoryApply();
        BeanUtil.copyProperties(item,val,"diagnosisCodeStandard","itemMap");
        val.setDiagnosisCodeStandard(StrJoinSplitUtils.split2List(item.getDiagnosisCodeStandard()));

        if(CollUtil.isNotEmpty(itemList)){
            itemList.forEach(itemEntity->{
                RoamItemDwdLaboratoryApplyItem itemModel=new RoamItemDwdLaboratoryApplyItem();
                itemModel.setItemCodeRaw(itemEntity.getItemGroupCodeRaw());
                itemModel.setItemCodeStandard(itemEntity.getItemGroupCodeStandard());
                itemModel.setItemNameRaw(itemEntity.getItemGroupNameRaw());
                itemModel.setItemNameStandard(itemEntity.getItemGroupNameStandard());
                val.addItem(itemModel);
            });
        }

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Apply_Laboratory_Project.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Apply_Laboratory_Project.getCode(), item.getId()),
                val,
                item.getDepartmentCodeStandard(),
                item.getApplyDate());
        return data;
    }


    /**
     * 检验报告单
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getLaboratoryReport(List<DwdLaboratoryReportInfo> itemList ,List<DwdLaboratoryReportItemInfo> projectList) {
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();

        Map<String,List<DwdLaboratoryReportItemInfo>> projectMap=new HashMap<>();
        projectList.forEach(item->{
            List<DwdLaboratoryReportItemInfo> list=projectMap.get(item.getLaboratoryReportId());
            if(list==null){
                list=new ArrayList<>();
                projectMap.put(item.getLaboratoryReportId(),list);
            }
            list.add(item);
        });


        for(DwdLaboratoryReportInfo item:itemList){
            if(item.getReportsTime()!=null){
                String dataKey= DateUtil.format(item.getReportsTime(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                List<DwdLaboratoryReportItemInfo> childProjectList = projectMap.get(item.getId());
                list.add(getLaboratoryReport(item,childProjectList));
            }
        }
        return data;
    }

    /**
     *  检验报告单
     * */
    public static XXRuleRoamHealthItem getLaboratoryReport(DwdLaboratoryReportInfo item,List<DwdLaboratoryReportItemInfo> itemList){
        RoamItemDwdLaboratoryReport val=new RoamItemDwdLaboratoryReport();
        BeanUtil.copyProperties(item,val,"descriptionStandard","conclusionStandard","itemMap");
        val.setDescriptionStandard(StrJoinSplitUtils.split2List(item.getDescriptionStandard()));
        val.setConclusionStandard(StrJoinSplitUtils.split2List(item.getConclusionStandard()));
        val.setActNo(item.getActivityNo());

        if(itemList != null){
            itemList.forEach(itemEntity->{
                RoamItemDwdLaboratoryReportItem itemModel=new RoamItemDwdLaboratoryReportItem();
                BeanUtil.copyProperties(itemEntity,itemModel,"resultNarrate");
                if(StrUtil.isNotEmpty(itemEntity.getResultNarrate())){
                    itemModel.setResultNarrate(StrJoinSplitUtils.split2List(itemEntity.getResultNarrate()));
                }
                val.addItem(itemModel);
            });
        }

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Report_Laboratory.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Report_Laboratory.getCode(), item.getId()),
                val,
                item.getDepartmentCodeStandard(),
                item.getReportsTime());

        return data;
    }


    /**
     * 医嘱用药
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getRecipeMedicine(List<DwdRecipeMedicineInfo> itemList ,List<DwdRecipeMedicineItemInfo> projectList) {
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();

        Map<String,List<DwdRecipeMedicineItemInfo>> projectMap=new HashMap<>();
        projectList.forEach(item->{
            List<DwdRecipeMedicineItemInfo> list=projectMap.get(item.getRecipeMedicineId());
            if(list==null){
                list=new ArrayList<>();
                projectMap.put(item.getRecipeMedicineId(),list);
            }
            list.add(item);
        });

        for(DwdRecipeMedicineInfo item:itemList){
            if(item.getRecipeTime()!=null){
                String dataKey= DateUtil.format(item.getRecipeTime(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                List<DwdRecipeMedicineItemInfo> childProjectList = projectMap.get(item.getId());
                list.add(getRecipeMedicine(item,childProjectList));
            }
        }
        return data;
    }

    /**
     * 医嘱用药
     * */
    public static XXRuleRoamHealthItem getRecipeMedicine(DwdRecipeMedicineInfo item ,List<DwdRecipeMedicineItemInfo> itemList) {
        RoamItemDwdRecipeMedicine val=new RoamItemDwdRecipeMedicine();
        BeanUtil.copyProperties(item,val,"itemMap","diagnosisCodeStandard");
        val.setDiagnosisCodeStandard(StrJoinSplitUtils.split2List(item.getDiagnosisCodeStandard()));

        if(CollUtil.isNotEmpty(itemList)){
            itemList.forEach(itemEntity->{
                RoamItemDwdRecipeMedicineItem itemModel=new RoamItemDwdRecipeMedicineItem();
                BeanUtil.copyProperties(itemEntity,itemModel);
                val.addItem(itemModel);
            });
        }

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Medicine_MZ.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Medicine_MZ.getCode(), item.getId()),
                val,
                item.getDepartmentCodeStandard(),
                item.getRecipeTime());

        return data;
    }


    /**
     * 治疗
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getRecipeTherapy(List<DwdRecipeTherapyInfo> itemList,List<DwdRecipeTherapyItemInfo> therapyItemList){

        Map<String,List<DwdRecipeTherapyItemInfo>> therapyMap=new HashMap<>();
        therapyItemList.forEach(item->{
            List<DwdRecipeTherapyItemInfo> list=therapyMap.get(item.getTherapyId());
            if(list==null){
                list=new ArrayList<>();
                therapyMap.put(item.getTherapyId(),list);
            }
            list.add(item);
        });

        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(DwdRecipeTherapyInfo item:itemList){
            if(item.getPayTime()!=null){
                String dataKey= DateUtil.format(item.getPayTime(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                List<DwdRecipeTherapyItemInfo> childList = therapyMap.get(item.getId());
                list.add(getRecipeTherapy(item,childList));
            }
        }
        return data;
    }

    /**
     * 治疗
     * */
    public static XXRuleRoamHealthItem getRecipeTherapy(DwdRecipeTherapyInfo item,List<DwdRecipeTherapyItemInfo> childList){
        RoamItemDwdRecipeTherapy val=new RoamItemDwdRecipeTherapy();
        BeanUtil.copyProperties(item,val,"diagnosisCodeStandard","itemMap");
        val.setDiagnosisCodeStandard(StrJoinSplitUtils.split2List(item.getDiagnosisCodeStandard()));

        if(CollUtil.isNotEmpty(childList)){
            childList.forEach(itemEntity->{
                RoamItemDwdRecipeTherapyItem itemModel=new RoamItemDwdRecipeTherapyItem();
                BeanUtil.copyProperties(itemEntity,itemModel);
                val.addItem(itemModel);
            });
        }

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Therapy.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Therapy.getCode(), item.getId()),
                val,
                item.getDepartmentCodeStandard(),
                item.getPayTime());

        return data;
    }



    /**
     * 手术申请
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getOperationApply(List<DwdOperationApplyInfo> itemList ,List<DwdOperationApplyItemInfo> projectList) {
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();

        Map<String,List<DwdOperationApplyItemInfo>> projectMap=new HashMap<>();
        projectList.forEach(item->{
            List<DwdOperationApplyItemInfo> list=projectMap.get(item.getOperationApplyId());
            if(list==null){
                list=new ArrayList<>();
                projectMap.put(item.getOperationApplyId(),list);
            }
            list.add(item);
        });

        for(DwdOperationApplyInfo item:itemList){
            if(item.getApplyTime()!=null){
                String dataKey= DateUtil.format(item.getApplyTime(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                List<DwdOperationApplyItemInfo> childProjectList = projectMap.get(item.getId());
                list.add(getOperationApply(item,childProjectList));
            }
        }
        return data;
    }

    /**
     * 手术申请
     * */
    public static XXRuleRoamHealthItem getOperationApply(DwdOperationApplyInfo item,List<DwdOperationApplyItemInfo> itemList) {
        RoamItemDwdOperationApply val=new RoamItemDwdOperationApply();
        BeanUtil.copyProperties(item,val,"diagnosisCodeStandard","itemMap");
        val.setDiagnosisCodeStandard(StrJoinSplitUtils.split2List(item.getDiagnosisCodeStandard()));

        if(CollUtil.isNotEmpty(itemList)){
            itemList.forEach(itemEntity->{
                RoamItemDwdOperationApplyItem itemModel=new RoamItemDwdOperationApplyItem();
                BeanUtil.copyProperties(itemEntity,itemModel);
                val.addItem(itemModel);
            });
        }

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Apply_Operation.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Apply_Operation.getCode(), item.getId()),
                val,
                item.getDepartmentCodeStandard(),
                item.getPayTime());

        return data;
    }


    /**
     * 手术记录
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getOperationRecord(List<DwdOperationRecordInfo> itemList ,List<DwdOperationRecordItemInfo> projectList) {
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();

        Map<String,List<DwdOperationRecordItemInfo>> projectMap=new HashMap<>();
        projectList.forEach(item->{
            List<DwdOperationRecordItemInfo> list=projectMap.get(item.getOperationRecordId());
            if(list==null){
                list=new ArrayList<>();
                projectMap.put(item.getOperationRecordId(),list);
            }
            list.add(item);
        });

        for(DwdOperationRecordInfo item:itemList){
            if(item.getPayTime()!=null){
                String dataKey= DateUtil.format(item.getPayTime(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                List<DwdOperationRecordItemInfo> childProjectList = projectMap.get(item.getId());
                list.add(getOperationRecord(item,childProjectList));
            }
        }
        return data;
    }

    /**
     * 手术记录
     * */
    public static XXRuleRoamHealthItem getOperationRecord(DwdOperationRecordInfo item ,List<DwdOperationRecordItemInfo> itemList) {
        RoamItemDwdOperationRecord val=new RoamItemDwdOperationRecord();
        BeanUtil.copyProperties(item,val,"itemMap","preDiagnosisCodeStandard","finalDiagnosisCodeStandard","contentStandard");
        val.setPreDiagnosisCodeStandard(StrJoinSplitUtils.split2List(item.getPreDiagnosisCodeStandard()));
        val.setFinalDiagnosisCodeStandard(StrJoinSplitUtils.split2List(item.getFinalDiagnosisCodeStandard()));
        val.setContentStandard(StrJoinSplitUtils.split2List(item.getContentStandard()));

        if(CollUtil.isNotEmpty(itemList)){
            itemList.forEach(itemEntity->{
                RoamItemDwdOperationRecordItem itemModel=new RoamItemDwdOperationRecordItem();
                BeanUtil.copyProperties(itemEntity,itemModel);
                val.addItem(itemModel);
            });
        }

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.OperationRecord.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.OperationRecord.getCode(), item.getId()),
                val,
                item.getDepartmentCodeStandard(),
                item.getPayTime());
        return data;
    }


    /**
     * 电子病例-体格检查（基础）
     * */
    public static Map<String,List<XXRuleRoamHealthItem>> getPhysicalSign(Map<String,DwdPatientMedicalRecordInfo> dwdEmrMap,List<DwdPatientEmrBodyCheckBasicInfo> dwdEmrCheckBasicList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(DwdPatientEmrBodyCheckBasicInfo item:dwdEmrCheckBasicList){
            DwdPatientMedicalRecordInfo patientMedicalRecord= dwdEmrMap.get(item.getEmrId());
            if(patientMedicalRecord!=null){
                String dataKey= DateUtil.format(item.getDataUpdateDate(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                list.add(getPhysicalSignRecord(patientMedicalRecord,item));
            }
        }
        return data;
    }
    public static XXRuleRoamHealthItem getPhysicalSignRecord(DwdPatientMedicalRecordInfo patientMedical,DwdPatientEmrBodyCheckBasicInfo bodyCheckBasic) {
        RoamItemDwdEmrCheckBasic val=new RoamItemDwdEmrCheckBasic();
        val.setHeight(EmrValueUtils.getBodyCheckHeight(bodyCheckBasic.getHeight()));
        val.setWeight(EmrValueUtils.getBodyCheckWeight(bodyCheckBasic.getWeight()));
        val.setBloodPressure(EmrValueUtils.getBodyCheckBloodPressure(bodyCheckBasic.getBloodPressure()));
        val.setHeartRate(EmrValueUtils.getBodyCheckHeartRat(bodyCheckBasic.getHeartRate()));
        val.setTemperature(EmrValueUtils.getBodyCheckTemperature(bodyCheckBasic.getTemperature()));
        val.setBmi(EmrValueUtils.getBodyCheckBmi(bodyCheckBasic.getBmi()));
        val.setDeptCode(patientMedical.getDepartmentCodeRaw());
        val.setDeptName(patientMedical.getDepartmentNameStandard());
        val.setEmrData(patientMedical.getDataUpdateDate());

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Physical_Sign.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Physical_Sign.getCode(), bodyCheckBasic.getId()),
                val,
                patientMedical.getDepartmentCodeStandard(),
                patientMedical.getDataUpdateDate()
        );

        return data;
    }


    /**
     *  电子病例-症状
     * */
    public static Map<String,List<XXRuleRoamHealthItem>> getEmrSymptom(RuleCondtionDataTypeEnum type, Map<String,DwdPatientMedicalRecordInfo> dwdEmrMap,List<DwdPatientEmrSymptomInfo>  dwdEmrSymptomList){
        //todo:
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(DwdPatientEmrSymptomInfo item:dwdEmrSymptomList){
            boolean flag=false;
            if(RuleCondtionDataTypeEnum.EMR_SymptomZS==type){
                if("zz-zs".equals(item.getType())){
                    flag=true;
                }
            }else if(RuleCondtionDataTypeEnum.EMR_SymptomJWS==type){
                if("zz-jws".equals(item.getType())){
                    flag=true;
                }
            }else if(RuleCondtionDataTypeEnum.EMR_SymptomXBS==type){
                if("zz-xbs".equals(item.getType())){
                    flag=true;
                }
            }
            if(flag){
                DwdPatientMedicalRecordInfo patientMedicalRecord= dwdEmrMap.get(item.getEmrId());
                if(patientMedicalRecord.getDataUpdateDate()!=null){
                    String dataKey= DateUtil.format(item.getDataUpdateDate(), DatePattern.PURE_DATE_FORMAT);
                    List<XXRuleRoamHealthItem> list=data.get(dataKey);
                    if(list==null){
                        list=new ArrayList<>();
                        data.put(dataKey,list);
                    }
                    list.add(getEmrSymptomRecord(type,patientMedicalRecord,item));
                }
            }
        }

        return data;
    }


    public static XXRuleRoamHealthItem getEmrSymptomRecord(RuleCondtionDataTypeEnum type, DwdPatientMedicalRecordInfo patientMedical,DwdPatientEmrSymptomInfo symptom) {

        RoamItemDwdEmrSymptom val=new RoamItemDwdEmrSymptom();
        val.setCode(symptom.getId());
        val.setName(symptom.getName());
        val.setPart(EmrValueUtils.getSymptom_BodyPart(symptom.getBodyPart()));

        Dict duration= EmrValueUtils.getSymptom_Duration(symptom.getDurationTime());
        if(duration!=null){
            val.setDuration(duration.getInt("val"));
            val.setDurationUnit(duration.getStr("unit"));
        }

        Dict oneTime= EmrValueUtils.getSymptom_OneTime(symptom.getOneTime());
        if(oneTime!=null){
            val.setOneTime(duration.getInt("val"));
            val.setOneTimeUnit(duration.getStr("unit"));
        }

        val.setDeptCode(patientMedical.getDepartmentCodeRaw());
        val.setDeptName(patientMedical.getDepartmentNameStandard());
        val.setEmrData(patientMedical.getDataUpdateDate());

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                type.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(type.getCode(), symptom.getId()),
                val,
                patientMedical.getDepartmentCodeStandard(),
                patientMedical.getDataUpdateDate()
        );

        return data;
    }



    /**
     * 标签
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getLabel(List<PatientLabelBO> itemList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(PatientLabelBO item:itemList){
            if(item.getCreateDate()!=null){
                String dataKey= DateUtil.format(item.getCreateDate(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                list.add(getLabel(item));
            }
        }
        return data;
    }

    /**
     * 标签
     * */
    public static XXRuleRoamHealthItem getLabel(PatientLabelBO item){

        RoamItemDwdLabel val=new RoamItemDwdLabel();
        val.setId(item.getId());
        val.setPatientId(item.getPatientId());
        val.setLabelId(item.getLabelId());
        val.setLabelName(item.getLabelName());
        val.setLabelType(item.getBizType());
        val.setSourceType(item.getSourceType());
        val.setSourceOneId(item.getSourceRelId());
        val.setSourceTwoId(item.getSourceRelTwoId());
        val.setDataUpdateDate(item.getDataDt());
        val.setDataType(MetadataStandardResultTypeEnum.MSRT_ResultNarrate_Current.getCode());

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Label.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Label.getCode(), item.getCreateDate(),item.getLabelId(),"R"),
                val,
                //item.getLabelId(),
                null,
                item.getCreateDate());
        return data;
    }



    /**
     * 历史标签
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getLabelHis(List<PatientLabelHistoryBO> itemList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(PatientLabelHistoryBO item:itemList){
            if(item.getDataDt()!=null){
                String dataKey= DateUtil.format(item.getDataDt(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                list.add(getLabelHis(item));
            }
        }
        return data;
    }



    /**
     * 标签
     * */
    public static XXRuleRoamHealthItem getLabelHis(PatientLabelHistoryBO item){

        RoamItemDwdLabel val=new RoamItemDwdLabel();
        val.setId(item.getId());
        val.setPatientId(item.getPatientId());
        val.setLabelId(item.getLabelId());
        val.setSourceType(item.getSourceType());
        val.setSourceOneId(item.getSourceRelId());
        val.setSourceTwoId(item.getSourceRelTwoId());
        val.setDataUpdateDate(item.getDataDt());
        val.setDataType(MetadataStandardResultTypeEnum.MSRT_ResultNarrate_History.getCode());

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Label.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Label.getCode(), item.getDataDt(),item.getLabelId(),"H"),
                val,
                //item.getLabelId(),
                null,
                item.getDataDt());
        return data;
    }




    /**
     * 量表
     * */
    public static Map<String, List<XXRuleRoamHealthItem>> getScale( List<ScaleResultBO> scaleList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(ScaleResultBO item:scaleList){
            if(item.getEndEvaluationDate()!=null){
                String dataKey= DateUtil.format(item.getEndEvaluationDate(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                getScale(item,list);
            }
        }
        return data;
    }


    public static void getScale(ScaleResultBO item,List<XXRuleRoamHealthItem> list){
        if(MapUtil.isEmpty(item.getScaleResult())){
            return;
        }
        for(String resultKey:item.getScaleResult().keySet()){
            Object resultVal= item.getScaleResult().get(resultKey);
            XXRuleRoamHealthItem dataItem =  getScale(item,resultKey,resultVal);
            list.add(dataItem);
        }
    }


    /**
     * 量表
     * */
    public static XXRuleRoamHealthItem getScale(ScaleResultBO item,String resultKey,Object resultVal){
        RoamItemDwdScale val=new RoamItemDwdScale();
        val.setId(item.getId());
        val.setPatientId(item.getPatientId());
        val.setScaleId(item.getScaleConfigId());
        val.setScaleCode(StrUtil.format("{}_{}",item.getScaleConfigId(),resultKey));
        val.setScaleResult(Convert.toStr(resultVal));
        val.setDataUpdateDate(item.getEndEvaluationDate());

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Label.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.ScaleResult.getCode(), item.getEndEvaluationDate(),val.getScaleCode()),
                val,
                null,
                item.getEndEvaluationDate());

        return data;
    }



    /**
     * 体检总报告
     * */
    @Deprecated
    public static Map<String, List<XXRuleRoamHealthItem>> getMedicalGeneralReport(List<DwdMedicalGeneralReportInfo> itemList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(DwdMedicalGeneralReportInfo item:itemList){
            if(item.getReportDate()!=null){
                String dataKey= DateUtil.format(item.getReportDate(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                list.add(getMedicalGeneralReport(item));
            }
        }
        return data;
    }

    /**
     * 体检总报告
     * */
    @Deprecated
    public static XXRuleRoamHealthItem getMedicalGeneralReport(DwdMedicalGeneralReportInfo item){
        RoamItemDwdMedicalGeneralReport val=new RoamItemDwdMedicalGeneralReport();
        BeanUtil.copyProperties(item,val);

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.Report_PhysicalExamine.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.Report_PhysicalExamine.getCode(), item.getOpDate()),
                val,
                item.getDepartmentCodeStandard(),
                item.getReportDate());
        return data;
    }

    /**
     *  收费记录
     * */
    @Deprecated
    public static Map<String, List<XXRuleRoamHealthItem>> getCollectFeeRecord(List<DwdCollectFeeRecordInfo> itemList){
        Map<String,List<XXRuleRoamHealthItem>> data=new HashMap<>();
        for(DwdCollectFeeRecordInfo item:itemList){
            if(item.getFeeDate()!=null){
                String dataKey= DateUtil.format(item.getFeeDate(), DatePattern.PURE_DATE_FORMAT);
                List<XXRuleRoamHealthItem> list=data.get(dataKey);
                if(list==null){
                    list=new ArrayList<>();
                    data.put(dataKey,list);
                }
                list.add(getCollectFeeRecord(item));
            }
        }
        return data;
    }


    /**
     *  收费记录
     * */
    @Deprecated
    public static XXRuleRoamHealthItem getCollectFeeRecord(DwdCollectFeeRecordInfo item){
        RoamItemDwdCollectFeeRecord val=new RoamItemDwdCollectFeeRecord();
        BeanUtil.copyProperties(item,val);

        XXRuleRoamHealthItem data=new XXRuleRoamHealthItem(
                RuleCondtionDataTypeEnum.CollectFee.getCode(),
                XXRuleRoamHealthItemHelper.getMateDateCode(RuleCondtionDataTypeEnum.CollectFee.getCode(), item.getOpDate()),
                val,
                item.getDepartmentCodeStandard(),
                item.getFeeDate());
        return data;

    }



}
