package com.xxh.cloud.dw.healthy.engine.modules.action.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.dw.healthy.engine.modules.action.service.ComXxhActionMethodService;
import com.xxh.cloud.healthy.serve.label.client.api.PatientLabelClientService;
import com.xxh.cloud.healthy.serve.label.client.api.bean.dto.LabelAddDTO;
import com.xxh.cloud.healthy.serve.label.client.api.bean.dto.LabelRemoveDTO;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.ClientActionMethodBO;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.ClientActionMethodParamBO;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.WorkFlowExecuteResultBO;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 患者标签服务
 * com.xxh.label
 * */
@Log4j2
@Service
public class ComXxhLabelMethodService implements ComXxhActionMethodService {

    @Autowired
    private PatientLabelClientService patientLabelClientService;


    @Override
    public void execute(WorkFlowExecuteResultBO params, Map<String, Date> eventData) {
        if("Add_Label".equals(params.getMethod().getF())){
            this.addLabel(params.getTaskId(),params.getTaskNodeId(),params.getRuleConfigId(),params.getNodeCode(),params.getMethod());
        }else if("Remove_Label".equals(params.getMethod().getF())){
            this.removeLabel(params.getTaskId(),params.getTaskNodeId(),params.getRuleConfigId(),params.getMethod());
        }else {
            System.out.println("不支持的方法 : "+params.getMethod().getF());
        }
    }

    /**
     * 新增标签
     * */
    public void addLabel(String taskId,String nodeTaskId,String ruleConfigId,String nodeCode,ClientActionMethodBO method){
        System.out.println("++++++++ 新增标签 +++++++++");
        //参数
        String patientId=null;
        Integer validityNum=null;
        List<String> labelIds=new ArrayList<>();

        for(ClientActionMethodParamBO amp: method.getP()){

            if("userId".equals(amp.getParamCode())){
                if(amp.getVal()!=null){
                    patientId = Convert.toStr(amp.getVal());
                }

            }

            if("labelId".equals(amp.getParamCode())){
                if(CollUtil.isNotEmpty(amp.getValList())){
                    amp.getValList().forEach(item->{
                        labelIds.add(Convert.toStr(item));
                    });
                }else if(amp.getVal()!=null){
                    labelIds.add(Convert.toStr(amp.getVal()));
                }
            }

            if("limitTime".equals(amp.getParamCode())){
                if(amp.getVal()!=null){
                    validityNum = Convert.toInt(amp.getVal());
                }
            }

        }

        if(StrUtil.isEmpty(patientId) || CollUtil.isEmpty(labelIds)){
            log.error("++++++ 引擎调用标签服务参数错误! taskId={} ,ruleConfigId={} ",taskId,ruleConfigId);
            return;
        }

        LabelAddDTO addParam=new LabelAddDTO();
        addParam.setPatientId(patientId);
        addParam.setValidityNum(validityNum);
        addParam.setLabelIds(labelIds);

        addParam.setSourceType(ObjectTypeEnum.OT_FlowNodeTask.getCode());
        addParam.setSourceRelId(taskId);
        addParam.setSourceRelTwoId(nodeCode);
        System.out.println("++++++++++++++新增标签参数");
        System.out.println(JSON.toJSONString(addParam));
        patientLabelClientService.add(addParam);

    }

    /**
     * 移除标签
     * */
    public void removeLabel(String taskId,String nodeTaskId,String ruleConfigId,ClientActionMethodBO method){
        System.out.println("++++++++ 移除标签 +++++++++");
        //参数
        String patientId=null;
        List<String> labelIds=new ArrayList<>();
        for(ClientActionMethodParamBO amp: method.getP()){
            if("userId".equals(amp.getParamCode())){
                if(amp.getVal()!=null){
                    patientId = Convert.toStr(amp.getVal());
                }
            }
            //
            if("cd".equals(amp.getParamCode())){
                if(CollUtil.isNotEmpty(amp.getValList())){
                    amp.getValList().forEach(item->{
                        labelIds.add(Convert.toStr(item));
                    });
                }else if(amp.getVal()!=null){
                    labelIds.add(Convert.toStr(amp.getVal()));
                }
            }
            //
            if("labelId".equals(amp.getParamCode())){
                if(CollUtil.isNotEmpty(amp.getValList())){
                    amp.getValList().forEach(item->{
                        labelIds.add(Convert.toStr(item));
                    });
                }else if(amp.getVal()!=null){
                    labelIds.add(Convert.toStr(amp.getVal()));
                }
            }
        }


//        //todo:check
//        if(StrUtil.isEmpty(patientId) || CollUtil.isEmpty(labelIds)){
//            //todo:
//            return;
//        }
//

        LabelRemoveDTO removeParam=new LabelRemoveDTO();
        removeParam.setPatientId(patientId);
        removeParam.setLabelIds(labelIds);
        patientLabelClientService.remove(removeParam);

    }



}
