package com.xxh.cloud.dw.healthy.engine.modules.action.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.dw.healthy.engine.modules.action.service.ComXxhActionMethodService;
import com.xxh.cloud.healthy.serve.label.client.api.PatientLabelClientService;
import com.xxh.cloud.healthy.serve.label.client.api.bean.dto.LabelAddDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.PatientPushServeClientService;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientPushServeAddClientDTO;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientPushServeRemoveClientDTO;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.ClientActionMethodBO;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.ClientActionMethodParamBO;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.WorkFlowExecuteResultBO;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 *
 * 患者推送服务
 * com.xxh.pushServer
 * */
@Log4j2
@Service
public class ComXxhPushServMethodService implements ComXxhActionMethodService {

    @Autowired
    private PatientPushServeClientService patientPushServeClientService;
    @Autowired
    private PatientLabelClientService patientLabelClientService;



    @Override
    public void execute(WorkFlowExecuteResultBO params ,Map<String, Date> eventData) {
        if("Push_Service_Content".equals(params.getMethod().getF())){
            this.addPushService(params.getTaskId(),params.getTaskNodeId(),params.getRuleConfigId(), params.getMethod(),eventData);
        }else if("Remove_Service_Content".equals(params.getMethod().getF())){
            this.removePushService(params.getTaskId(),params.getTaskNodeId(),params.getRuleConfigId(),params.getMethod());
        }else if("Push_Service_Content_label".equals(params.getMethod().getF())){
            this.addPushService2Label(params.getTaskId(),params.getTaskNodeId(),params.getRuleConfigId(),params.getNodeCode(),params.getMethod(),eventData);
        } else {
            System.out.println("不支持的方法 : "+params.getMethod().getF());
        }
    }

    /**
     * 新增服务
     * */
    public void addPushService(String taskId,String nodeTaskId,String ruleConfigId, ClientActionMethodBO method,Map<String, Date> eventData){
        System.out.println("++++++++ 新增服务 +++++++++");

        String param_patientId=null;
        String param_serviceId=null;

        for(ClientActionMethodParamBO amp: method.getP()){
            if("userId".equals(amp.getParamCode())){
                if(amp.getVal()!=null){
                    param_patientId = Convert.toStr(amp.getVal());
                }
            }
            if("serviceId".equals(amp.getParamCode())){
                if(amp.getVal()!=null){
                    param_serviceId = Convert.toStr(amp.getVal());
                }
            }
        }

        PatientPushServeAddClientDTO addParam=new PatientPushServeAddClientDTO();
        addParam.setPatientId(param_patientId);
        addParam.setPushServeId(param_serviceId);
        addParam.setBenchmarkDateMap(eventData);
        //addParam.setOpUser();

        System.out.println("++++++++++++++新增服务++++++++++++++++++++++");
        System.out.println(JSON.toJSONString(addParam));

        patientPushServeClientService.add(addParam);

    }

    /**
     * 移除服务
     * */
    public void removePushService(String taskId,String nodeTaskId,String ruleConfigId,ClientActionMethodBO method){
        System.out.println("++++++++ 移除服务 +++++++++");
        String param_patientId=null;
        List<String> param_serviceId=new ArrayList<>();
        for(ClientActionMethodParamBO amp: method.getP()){
            if("userId".equals(amp.getParamCode())){
                if(amp.getVal()!=null){
                    param_patientId = Convert.toStr(amp.getVal());
                }
            }
            if("service_id".equals(amp.getParamCode())){
                if(CollUtil.isNotEmpty(amp.getValList())){
                    amp.getValList().forEach(item->{
                        param_serviceId.add(Convert.toStr(item));
                    });
                }else if(amp.getVal()!=null){
                    param_serviceId.add(Convert.toStr(amp.getVal()));
                }
            }
        }

        PatientPushServeRemoveClientDTO removeParam=new PatientPushServeRemoveClientDTO();
        removeParam.setPatientId(param_patientId);
        removeParam.setIds(param_serviceId);
        //removeParam.setOpUser();
        patientPushServeClientService.remove(removeParam);
    }

    /**
     * 新增服务且新增标签
     * */
    public void addPushService2Label(String taskId,String nodeTaskId,String ruleConfigId,String nodeCode, ClientActionMethodBO method,Map<String, Date> eventData){

        System.out.println("++++++++ 新增服务且新增标签 +++++++++");

        String param_patientId=null;
        String param_serviceId=null;
        List<String> param_labelId=new ArrayList<>();
        Integer param_limitTime=null;

        for(ClientActionMethodParamBO amp: method.getP()){
            if("userId".equals(amp.getParamCode())){
                if(amp.getVal()!=null){
                    param_patientId = Convert.toStr(amp.getVal());
                }
            }
            if("serviceId".equals(amp.getParamCode())){
                if(amp.getVal()!=null){
                    param_serviceId = Convert.toStr(amp.getVal());
                }
            }
            if("labelId".equals(amp.getParamCode())){
                if(amp.getVal()!=null){
                    param_labelId.add(Convert.toStr(amp.getVal()));
                }
            }
            if("limitTime".equals(amp.getParamCode())){
                if(amp.getVal()!=null){
                    param_limitTime = Convert.toInt(amp.getVal());
                }
            }
        }


        PatientPushServeAddClientDTO addParam=new PatientPushServeAddClientDTO();
        addParam.setPatientId(param_patientId);
        addParam.setPushServeId(param_serviceId);
        addParam.setBenchmarkDateMap(eventData);
        //addParam.setOpUser();
        System.out.println("++++++++++++++新增服务且新增标签++++++++++++++++++++++");
        System.out.println(JSON.toJSONString(addParam));
        patientPushServeClientService.add(addParam);

        LabelAddDTO labelAddDTO=new LabelAddDTO();
        labelAddDTO.setPatientId(param_patientId);
        labelAddDTO.setValidityNum(param_limitTime);
        labelAddDTO.setLabelIds(param_labelId);
        labelAddDTO.setSourceType(ObjectTypeEnum.OT_FlowNodeTask.getCode());
        labelAddDTO.setSourceRelId(taskId);
        labelAddDTO.setSourceRelTwoId(nodeCode);
        patientLabelClientService.add(labelAddDTO);

    }




}
