package com.xxh.cloud.dw.healthy.engine.modules.action.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.dw.healthy.engine.modules.action.service.ComXxhActionMethodService;
import com.xxh.cloud.healthy.serve.risk.client.api.PatientRiskClientService;
import com.xxh.cloud.healthy.serve.risk.client.api.bean.dto.PatientRiskAddClientDTO;
import com.xxh.cloud.healthy.serve.risk.client.api.bean.dto.PatientRiskRemoveClientDTO;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.ClientActionMethodBO;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.ClientActionMethodParamBO;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.WorkFlowExecuteResultBO;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 *  患者风险服务
 *  com.xxh.risk
 * */
@Log4j2
@Service
public class ComXxhRiskMethodService implements ComXxhActionMethodService {



    @Autowired
    private PatientRiskClientService patientRiskClientService;


    @Override
    public void execute(WorkFlowExecuteResultBO params, Map<String, Date> eventData) {
        if("Add_Risk".equals(params.getMethod().getF())){
            this.addRisk(params.getTaskId(),params.getTaskNodeId(),params.getRuleConfigId(), params.getMethod());
        }else if("Remove_Risk".equals(params.getMethod().getF())){
            this.removeRisk(params.getTaskId(),params.getTaskNodeId(),params.getRuleConfigId(),params.getMethod());
        } else {
            System.out.println("不支持的方法 : "+params.getMethod().getF());
        }
    }

    /**
     *  为患者添加风险
     * */
    public void addRisk(String taskId, String nodeTaskId, String ruleConfigId, ClientActionMethodBO method){
        System.out.println("++++++++ 为患者添加风险 +++++++++");
        String param_patientId=null;
        List<String> param_riskIds=new ArrayList<>();

        for(ClientActionMethodParamBO amp: method.getP()){
            if("userId".equals(amp.getParamCode())){
                if(amp.getVal()!=null){
                    param_patientId = Convert.toStr(amp.getVal());
                }
            }

            if("riskId".equals(amp.getParamCode())){
                if(CollUtil.isNotEmpty(amp.getValList())){
                    amp.getValList().forEach(item->{
                        param_riskIds.add(Convert.toStr(item));
                    });
                }else if(amp.getVal()!=null){
                    param_riskIds.add(Convert.toStr(amp.getVal()));
                }
            }

        }


        if(StrUtil.isEmpty(param_patientId) || CollUtil.isEmpty(param_riskIds)){
            log.error("++++++ 引擎调用新增风险服务参数错误! taskId={} ,ruleConfigId={} ",taskId,ruleConfigId);
            return;
        }

        PatientRiskAddClientDTO riskAddClientDTO=new PatientRiskAddClientDTO();
        riskAddClientDTO.setPatientId(param_patientId);
        riskAddClientDTO.setRiskId(param_riskIds);
        riskAddClientDTO.setRemark("引擎新增");
        riskAddClientDTO.setOpUser("E");
        riskAddClientDTO.setDataDt(new Date());
        riskAddClientDTO.setSourceType(ObjectTypeEnum.OT_RuleInfo.getCode());
        riskAddClientDTO.setSourceRelId(ruleConfigId);
        patientRiskClientService.add(riskAddClientDTO);


    }

    /**
     * 为患者移除风险
     * */
    public void removeRisk(String taskId, String nodeTaskId, String ruleConfigId, ClientActionMethodBO method){
        System.out.println("++++++++ 为患者移除风险 +++++++++");
        String param_patientId=null;
        List<String> param_riskIds=new ArrayList<>();

        for(ClientActionMethodParamBO amp: method.getP()){
            if("userId".equals(amp.getParamCode())){
                if(amp.getVal()!=null){
                    param_patientId = Convert.toStr(amp.getVal());
                }
            }

            if("riskId".equals(amp.getParamCode())){
                if(CollUtil.isNotEmpty(amp.getValList())){
                    amp.getValList().forEach(item->{
                        param_riskIds.add(Convert.toStr(item));
                    });
                }else if(amp.getVal()!=null){
                    param_riskIds.add(Convert.toStr(amp.getVal()));
                }
            }

        }

        if(StrUtil.isEmpty(param_patientId) || CollUtil.isEmpty(param_riskIds)){
            log.error("++++++ 引擎调用移除风险服务参数错误! taskId={} ,ruleConfigId={} ",taskId,ruleConfigId);
            return;
        }


        PatientRiskRemoveClientDTO riskRemoveClientDTO=new PatientRiskRemoveClientDTO();
        riskRemoveClientDTO.setPatientId(param_patientId);
        riskRemoveClientDTO.setRiskId(param_riskIds);
        riskRemoveClientDTO.setRemark("引擎移除");
        riskRemoveClientDTO.setOpUser("E");
        riskRemoveClientDTO.setDataDt(new Date());
        riskRemoveClientDTO.setSourceType(ObjectTypeEnum.OT_RuleInfo.getCode());
        riskRemoveClientDTO.setSourceRelId(ruleConfigId);

        patientRiskClientService.remove(riskRemoveClientDTO);
    }




}
