package com.xxh.cloud.dw.healthy.engine.modules.exe.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.file.FileAppender;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.datalabel.client.api.RuleDataLabelClientService;
import com.xxh.cloud.datalabel.client.bean.dto.LabelRuleExecuteByAllDTO;
import com.xxh.cloud.datalabel.client.bean.dto.LabelRuleExecuteByRuleIdDTO;
import com.xxh.cloud.datalabel.client.bean.ros.LabelRuleExecuteResultRO;
import com.xxh.cloud.dw.healthy.engine.modules.data.service.RuleEngineDataService;
import com.xxh.cloud.dw.healthy.engine.modules.exe.bean.dto.PatientDataExtendDTO;
import com.xxh.cloud.dw.healthy.engine.modules.exe.helper.DataScopeChangeUtils;
import com.xxh.cloud.dw.healthy.engine.modules.exe.service.DwdDataExecuteLabelService;
import com.xxh.cloud.dw.healthy.engine.modules.exe.utils.LogFileUtils;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.utils.ConsoleUtils;
import com.xxh.cloud.healthy.serve.label.client.api.PatientLabelClientService;
import com.xxh.cloud.healthy.serve.label.client.api.bean.dto.LabelAddBatchDTO;
import com.xxh.cloud.healthy.serve.label.client.api.bean.dto.LabelAddDTO;
import com.xxh.cloud.healthy.serve.label.client.api.bean.dto.LabelDTO;
import com.xxh.cloud.log.bean.constant.ObjectTypeEnum;
import com.xxh.cloud.rule.basic.context.XXRuleRoam;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

@Log4j2
@Service
public class DwdDataExecuteLabelServiceImpl implements DwdDataExecuteLabelService {



    @Autowired
    private RuleDataLabelClientService ruleDataLabelClientService;

    @Autowired
    private RuleEngineDataService ruleEngineDataService;
    @Autowired
    private PatientLabelClientService patientLabelClientService;

    @Autowired
    private Environment environment;



    @Override
    public void executeLabelRuleByPatientId2RuleAll(String patientId,String dataDay,String dataScopeFlag,String dataScope,String opUser) {
        XXRuleRoam patientData=  ruleEngineDataService.getRuleRoam(patientId);
        this.executeLabelRuleByPatientId2RuleAll(patientId,patientData,dataDay,dataScopeFlag,dataScope,opUser);
    }

    @Override
    public void executeLabelRuleByPatientId2RuleAll(String patientId, XXRuleRoam patientData,String dataDay,String dataScopeFlag,String dataScope,String opUser) {
        //ConsoleUtils.log("+++++ 执行标签规则,请求 : "+JSON.toJSONString(patientData));

        LabelRuleExecuteByAllDTO executeParams=new LabelRuleExecuteByAllDTO();
        executeParams.setPatientId(patientId);
        executeParams.setData(patientData);

        if(StrUtil.isEmpty(dataDay)){
            executeParams.setDataDay(DateUtil.now());
        }else {
            executeParams.setDataDay(dataDay);
        }

        if(StrUtil.isEmpty(dataScopeFlag)){
            executeParams.setDataScopeFlag(YN.NO.getCode());
        }else {
            executeParams.setDataScopeFlag(dataScopeFlag);
        }

        if(YN.YES.getCode().equals(executeParams.getDataScopeFlag())){
            if(StrUtil.isEmpty(dataScope)){
                System.out.println("+++++无数据执行范围");
                return;
            }
            String cds= DataScopeChangeUtils.getDataScopeChangeValue(dataScope);
            if(StrUtil.isEmpty(cds)){
                System.out.println("+++++无数据执行范围");
                return;
            }
            executeParams.setDataScope(cds);
        }


        executeParams.setOpUserId(opUser);


        String serverPort =  environment.getProperty("server.port");
        FileAppender fileAppender= LogFileUtils.getFileAppender("LabelRuleExe",serverPort);

        fileAppender.append("------------------------执行标签规则,开始--------------------------");
        fileAppender.append("----- 执行标签规则,患者编号:"+patientId +" , 执行时间:"+DateUtil.now());


        List<LabelRuleExecuteResultRO> resultList= ruleDataLabelClientService.executeByAll(executeParams);
        if(CollUtil.isEmpty(resultList)){
            fileAppender.append("----执行标签规则,无结果");
            fileAppender.flush();
            return;
        }

        fileAppender.append("----执行标签规则,有结果");
        resultList.forEach(item->{
            fileAppender.append(JSON.toJSONString(item));
        });

        fileAppender.append("----执行标签新增操作参数");
        LabelAddBatchDTO labelAddBatchParams=new LabelAddBatchDTO();
        labelAddBatchParams.setPatientId(patientId);
        resultList.forEach(item->{
            Integer vnum=item.getLabelValidityNum()==null?9999:item.getLabelValidityNum();
            LabelDTO addLabelItem=new LabelDTO(item.getLabelId(),vnum,ObjectTypeEnum.OT_DataLabel_Rule.getCode(),item.getRuleId());
            addLabelItem.setSourceRelTwoId(item.getLogId());
            fileAppender.append(JSON.toJSONString(addLabelItem));
            labelAddBatchParams.addLabel(addLabelItem);
        });

        fileAppender.flush();
        patientLabelClientService.addBatch(labelAddBatchParams);
    }

    @Override
    public void executeLabelRuleByPatientId2RuleId(String patientId, String ruleId,String dataDay,String dataScopeFlag,String dataScope,String opUser,boolean isTest) {
        XXRuleRoam patientData=ruleEngineDataService.getRuleRoam(patientId);
//        System.out.println("+++++++++++++++++++++ 患者数据开始");
//        System.out.println(JSON.toJSONString(patientData));
//        System.out.println("+++++++++++++++++++++ 患者数据截止");
        this.executeLabelRuleByPatientId2RuleId(patientId,patientData,ruleId,dataDay,dataScopeFlag,dataScope,opUser,isTest);
    }

    @Override
    public void executeLabelRuleByPatientId2RuleId(String patientId, XXRuleRoam patientData, String ruleId,String dataDay,String dataScopeFlag,String dataScope,String opUser,boolean isTest) {
        ConsoleUtils.log("执行标签规则:"+patientId+" ,规则ID:"+ruleId);
        LabelRuleExecuteByRuleIdDTO executeParams=new LabelRuleExecuteByRuleIdDTO();
        executeParams.setRuleId(ruleId);
        executeParams.setData(patientData);
        executeParams.setPatientId(patientId);


        if(StrUtil.isEmpty(dataDay)){
            executeParams.setDataDay(DateUtil.now());
        }else {
            executeParams.setDataDay(dataDay);
        }

        if(StrUtil.isEmpty(dataScopeFlag)){
            executeParams.setDataScopeFlag(YN.NO.getCode());
        }else {
            executeParams.setDataScopeFlag(dataScopeFlag);
        }

        executeParams.setTest(isTest);

        if(YN.YES.getCode().equals(executeParams.getDataScopeFlag())){
            if(StrUtil.isEmpty(dataScope)){
                System.out.println("+++++无数据执行范围");
                return;
            }
            String cds= DataScopeChangeUtils.getDataScopeChangeValue(dataScope);
            if(StrUtil.isEmpty(cds)){
                System.out.println("+++++无数据执行范围");
                return;
            }
            executeParams.setDataScope(cds);
        }

        executeParams.setOpUserId(opUser);

        LabelRuleExecuteResultRO result = ruleDataLabelClientService.executeById(executeParams);
        if(result==null){
            System.out.println("+++++++2024 wjj +++++ 执行标签规则,无执行结果");
            return;
        }

        ConsoleUtils.log("++++2024 wjj ++++ 执行标签,执行结果 = {}",JSON.toJSONString(result));

        LabelAddDTO labelAddDTO=new LabelAddDTO();
        labelAddDTO.setPatientId(patientId);
        labelAddDTO.addLabelId(result.getLabelId());
        if(result.getLabelValidityNum()==null){
            labelAddDTO.setValidityNum(99999);
        }else {
            labelAddDTO.setValidityNum(result.getLabelValidityNum());
        }


        labelAddDTO.setSourceType(ObjectTypeEnum.OT_DataLabel_Rule.getCode());
        labelAddDTO.setSourceRelId(result.getRuleId());
        labelAddDTO.setSourceRelTwoId(result.getLogId());

        patientLabelClientService.add(labelAddDTO);
    }


    @Override
    public void executeLabelRuleByPatientIds(List<String> patientIds,Map<String, PatientDataExtendDTO> patientDataExtendMap,String opUser) {
        for(String patientId:patientIds){
            try {
                PatientDataExtendDTO extendData= patientDataExtendMap.get(patientId);
                if(extendData!=null){
                    this.executeLabelRuleByPatientId2RuleAll(patientId,extendData.getDataDay(),extendData.getDataScopeFlag(),extendData.getDataScope(),opUser);
                }
            }catch (Exception e){
                log.error("+++++ 标签规则执行,患者ID:"+patientId+" , 错误信息:"+e.getMessage());
                e.printStackTrace();
            }
        }
    }

    @Override
    public void executeLabelRuleByPatientData(Map<String, XXRuleRoam> patientDataMap,Map<String, PatientDataExtendDTO> patientDataExtendMap, String opUser) {
        for(String patientId:patientDataMap.keySet()){
            try {
                XXRuleRoam patientData = patientDataMap.get(patientId);
                PatientDataExtendDTO extendData= patientDataExtendMap.get(patientId);
                if(extendData!=null){
                    this.executeLabelRuleByPatientId2RuleAll(patientId,patientData,extendData.getDataDay(),extendData.getDataScopeFlag(),extendData.getDataScope(),opUser);
                }
            }catch (Exception e){
                log.error("+++++ 标签规则执行,患者ID:"+patientId+" , 错误信息:"+e.getMessage());
                e.printStackTrace();
            }
        }
    }




}
