package com.xxh.cloud.dw.healthy.engine.modules.exe.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.file.FileAppender;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.dw.healthy.engine.modules.action.service.ComXxhActionService;
import com.xxh.cloud.dw.healthy.engine.modules.data.service.PatientDataLabel2EventService;
import com.xxh.cloud.dw.healthy.engine.modules.data.service.RuleEngineDataService;
import com.xxh.cloud.dw.healthy.engine.modules.exe.bean.vos.FlowProcessResultVO;
import com.xxh.cloud.dw.healthy.engine.modules.exe.service.DwdDataExecuteRuleFlowService;
import com.xxh.cloud.dw.healthy.engine.modules.exe.utils.LogFileUtils;
import com.xxh.cloud.rule.basic.context.XXRuleRoam;
import com.xxh.cloud.rule.execute.inference.client.api.RuleExecute2WorkFlowClientService;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.WorkFlowExecuteProcessAllResultBO;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.WorkFlowExecuteProcessResultBO;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.WorkFlowExecuteResultBO;
import com.xxh.cloud.rule.execute.inference.client.bean.dto.WorkFlowExecuteAllDTO;
import com.xxh.cloud.rule.execute.inference.client.bean.dto.WorkFlowExecuteByRuleDTO;
import com.xxh.cloud.rule.execute.inference.client.bean.dto.WorkFlowNodeTaskExecuteDTO;
import com.xxh.cloud.rule.execute.inference.client.bean.dto.WorkFlowTriggerNodeTaskExecuteDTO;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Log4j2
@Service
public class DwdDataExecuteRuleFlowServiceImpl implements DwdDataExecuteRuleFlowService {

    @Autowired
    private RuleEngineDataService ruleEngineDataService;
    @Autowired
    private RuleExecute2WorkFlowClientService ruleExecute2WorkFlowClientService;

    @Autowired
    private ComXxhActionService comXxhActionService;

    @Autowired
    private PatientDataLabel2EventService patientDataLabel2EventService;

    @Autowired
    private Environment environment;





    /**
     * 定时任务，有数据变动直接执行，无数据变动。每1小时执行一次
     * */
    @Override
    public void executeAll(String patientId, XXRuleRoam ruleRoam,Map<String, Date> patientEventData) {
        System.out.println("+++++开始执行符合条件的流程流程:患者号="+patientId);
        System.out.println(JSON.toJSONString(ruleRoam));


        WorkFlowExecuteAllDTO executeParam=new WorkFlowExecuteAllDTO();
        executeParam.setPatientUserId(patientId);
        executeParam.setData(ruleRoam);
        WorkFlowExecuteProcessAllResultBO  resultBO =ruleExecute2WorkFlowClientService.executeWorkFlowAll(executeParam);


        String serverPort =  environment.getProperty("server.port");
        FileAppender fileAppender= LogFileUtils.getFileAppender("FlowRuleExe",serverPort);

        fileAppender.append("-------------执行流程引擎开始----------------------");
        fileAppender.append("********** 执行流程引擎,患者编号:"+patientId +" , 执行时间:"+ DateUtil.now());

        if(CollUtil.isNotEmpty(resultBO.getStartResultList())){
            fileAppender.append("----执行动作有结果,如下:");
            resultBO.getStartResultList().forEach(item->{
                fileAppender.append(JSON.toJSONString(item));
            });
        }else {
            fileAppender.append("----执行动作无结果");
        }

        if(CollUtil.isNotEmpty(resultBO.getTriggerResultList())){
            fileAppender.append("-----执行定时节点有结果,如下:");
            resultBO.getTriggerResultList().forEach(item->{
                fileAppender.append(JSON.toJSONString(item));
            });
        }else{
            fileAppender.append("----- 执行定时节点无结果");
        }
        fileAppender.flush();

        if(CollUtil.isNotEmpty(resultBO.getStartResultList())){
            comXxhActionService.execute(resultBO.getStartResultList(),patientEventData);
        }

        if(CollUtil.isNotEmpty(resultBO.getTriggerResultList())){
            comXxhActionService.execute(resultBO.getTriggerResultList(),patientEventData);
        }
    }

    @Override
    public void executeAll(Map<String, XXRuleRoam> patientDataMap, Map<String, Map<String, Date>> patientEventData) {
        for(String patientId:patientDataMap.keySet()){
            try {
                XXRuleRoam patientData = patientDataMap.get(patientId);
                Map<String, Date> eventData =patientEventData.get(patientId);
                this.executeAll(patientId,patientData,eventData);
            }catch (Exception e){
                log.error("+++++ 流程规则执行,患者ID:"+patientId+" , 错误信息:"+e.getMessage());
                e.printStackTrace();
            }
        }
    }


    @Override
    public void start2Rule(String patientId, String ruleId) {
        log.error("+++++start2Rule :  param : {} -- {}",patientId,ruleId);

        XXRuleRoam ruleRoam= ruleEngineDataService.getRuleRoam(patientId);
        Map<String, Date> eventData= patientDataLabel2EventService.getPatientEvent(patientId);

        System.out.println("+++++++++++++++++++++患者数据++++++++++++++++++++++++++++++++++");
        System.out.println(JSON.toJSONString(ruleRoam));

        WorkFlowExecuteByRuleDTO  executeParam=new WorkFlowExecuteByRuleDTO();
        executeParam.setRuleConfigId(ruleId);
        executeParam.setData(ruleRoam);

        List<WorkFlowExecuteResultBO> list= ruleExecute2WorkFlowClientService.startExecute(executeParam);
        System.out.println(JSON.toJSONString(list));

        System.out.println("**************************");
        comXxhActionService.execute(list,eventData);
    }

    @Override
    public List<WorkFlowExecuteResultBO> start2RuleNotExeBiz(String patientId, String ruleId) {

        XXRuleRoam ruleRoam= ruleEngineDataService.getRuleRoam(patientId);
        Map<String, Date> eventData= patientDataLabel2EventService.getPatientEvent(patientId);

        System.out.println("+++++++++++++++++++++患者数据++++++++++++++++++++++++++++++++++");
        System.out.println(JSON.toJSONString(ruleRoam));

        WorkFlowExecuteByRuleDTO  executeParam=new WorkFlowExecuteByRuleDTO();
        executeParam.setRuleConfigId(ruleId);
        executeParam.setData(ruleRoam);

        List<WorkFlowExecuteResultBO> list= ruleExecute2WorkFlowClientService.startExecute(executeParam);

        return list;

    }

    @Override
    public FlowProcessResultVO process(String nodeTaskId, String patientId, String handleStatus, String remark, String opUserId) {
        XXRuleRoam ruleRoam= ruleEngineDataService.getRuleRoam(patientId);
        Map<String, Date> eventData= patientDataLabel2EventService.getPatientEvent(patientId);

        WorkFlowNodeTaskExecuteDTO executeParam=new WorkFlowNodeTaskExecuteDTO();
        executeParam.setNodeTaskId(nodeTaskId);
        executeParam.setHandleStatus(handleStatus);
        executeParam.setData(ruleRoam);
        executeParam.setRemark(remark);
        executeParam.setOpUserId(opUserId);
        WorkFlowExecuteProcessResultBO resultList= ruleExecute2WorkFlowClientService.processNode(executeParam);

        System.out.println("**************************");
        System.out.println(JSON.toJSONString(resultList));

        if(CollUtil.isNotEmpty(resultList.getNextTask())){
            comXxhActionService.execute(resultList.getNextTask(),eventData);
        }

        return new FlowProcessResultVO(resultList.getNodeTaskStatus());
    }

    @Override
    public void executeTrigger(String nodeTaskId,String triggerExeDate, String patientId) {

        XXRuleRoam ruleRoam= ruleEngineDataService.getRuleRoam(patientId);
        Map<String, Date> eventData= patientDataLabel2EventService.getPatientEvent(patientId);

//        System.out.println("+++++++++++++++++++++++++++++++++++++++++++++++++++++++");
//        System.out.println(JSON.toJSONString(ruleRoam));

        WorkFlowTriggerNodeTaskExecuteDTO triggerParam=new WorkFlowTriggerNodeTaskExecuteDTO();
        triggerParam.setData(ruleRoam);
        triggerParam.setNodeTaskId(nodeTaskId);
        triggerParam.setTriggerExeDate(triggerExeDate);
        List<WorkFlowExecuteResultBO> list=ruleExecute2WorkFlowClientService.processTriggerNode(triggerParam);

        String serverPort =  environment.getProperty("server.port");
        FileAppender fileAppender= LogFileUtils.getFileAppender("FlowRuleExe-trigger",serverPort);
        fileAppender.append("-----------------执行流程引擎节点定时任务开始---------------");
        fileAppender.append("********** 执行流程引擎,患者编号:"+patientId +",节点任务ID = "+nodeTaskId +" , 执行时间:"+ DateUtil.now());

        if(CollUtil.isEmpty(list)){
            fileAppender.append("-----执行无结果");
        }else {
            fileAppender.append("-----执行有结果");
            list.forEach(item->{
                fileAppender.append(JSON.toJSONString(item));
            });
        }
        fileAppender.flush();
        System.out.println("**************************");
        comXxhActionService.execute(list,eventData);

    }




}
