package com.xxh.cloud.dw.healthy.engine.modules.exe.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.dw.healthy.engine.modules.exe.service.DwdDataExecuteRuleFlowService;
import com.xxh.cloud.dw.healthy.engine.modules.exe.service.DwdDataExecuteRuleTriggerService;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.utils.XXDateDifference;
import com.xxh.cloud.framework.common.utils.XXDateUtils;
import com.xxh.cloud.rule.execute.inference.client.api.RuleExecute2WorkFlowClientService;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.FlowEventTriggerNodeTaskResultBO;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Log4j2
@Service
public class DwdDataExecuteRuleTriggerServiceImpl implements DwdDataExecuteRuleTriggerService {


    @Autowired
    private DwdDataExecuteRuleFlowService dwdDataExecuteRuleFlowService;


    @Autowired
    private RuleExecute2WorkFlowClientService ruleExecute2WorkFlowClientService;

    @Autowired
    private RedisUtils redisUtils;


    private String getCacheKey(String day){
        return StrUtil.format("WFE:Trigger:{}",day);
    }

    private List<FlowEventTriggerNodeTaskResultBO> getDataByCache(String day){
        String key=getCacheKey(day);
        Object obj= redisUtils.get(key);
        if(obj==null){
            log.debug("+++++ 从缓存中获取无值,待查询");
            return null;
        }
        log.debug("+++++ 从缓存中返回数据");
        String str= Convert.toStr(obj);
        if("-1".equals(str)){
            return new ArrayList<>();
        }else {
            return JSON.parseArray(str,FlowEventTriggerNodeTaskResultBO.class);
        }
    }

    private void setDataByCache(String day, List<FlowEventTriggerNodeTaskResultBO> list){
        String key=getCacheKey(day);
        if(CollUtil.isNotEmpty(list)){
            redisUtils.set(key,JSON.toJSONString(list),180);
        }else {
            redisUtils.set(key,"-1",180);
        }

    }

    private List<FlowEventTriggerNodeTaskResultBO> readData(String day){
//        List<FlowEventTriggerNodeTaskResultBO> data=this.getDataByCache(day);
//        if(data!=null){
//            return data;
//        }
        log.debug("+++++ 查询引擎数据");
        List<FlowEventTriggerNodeTaskResultBO> dataList= ruleExecute2WorkFlowClientService.getTriggerNodeTaskByDay(day);

        if(CollUtil.isNotEmpty(dataList)){
            log.debug("++++++"+JSON.toJSONString(dataList));
        }

//        this.setDataByCache(day,dataList);
        return dataList;
    }


    @Override
    public void runTriggerNodeTask(Date date) {
        log.debug("+++++++ 自动节点执行: 时间 {}",DateUtil.formatDateTime(date));
        String day= DateUtil.format(date,"yyyyMMdd");

        List<FlowEventTriggerNodeTaskResultBO> data=this.readData(day);
        if(CollUtil.isEmpty(data)){
            log.debug("+++++ 无定时任务节点");
            return;
        }

        log.debug(" 1. ++++++"+JSON.toJSONString(data));

        for(FlowEventTriggerNodeTaskResultBO dataItem:data){
            if(StrUtil.isNotEmpty(dataItem.getTriggerDateListNo()) && dataItem.getTriggerDateStart()!=null && dataItem.getTriggerDateEnd()!=null){
                this.processOne(date,dataItem,10);
            }
        }
    }


    /**
     * 间隔分钟
     * */
    private void processOne(Date date, FlowEventTriggerNodeTaskResultBO dataItem,int intervalMinute){
        boolean flag= XXDateUtils.between(date,dataItem.getTriggerDateStart(),dataItem.getTriggerDateEnd());
        if(flag==false){
            log.debug("+++++不在开始时间和截止时间区间中");
            return;
        }


        XXDateDifference difference= XXDateUtils.difference(date, StrJoinSplitUtils.split2List(dataItem.getTriggerDateListNo()),intervalMinute);

        if(difference.isFlag()==false){
            log.debug("+++++ 无执行时间");
            return;
        }

        //todo:待优化
        this.executeOne(dataItem,difference);

    }


    /**
     * 之间执行
     * */
    private void executeOne(FlowEventTriggerNodeTaskResultBO dataItem,XXDateDifference difference){
        try {
            dwdDataExecuteRuleFlowService.executeTrigger(dataItem.getId(),difference.getVal(),dataItem.getPatientId());
        }catch (Exception e){
            log.debug("+++++ 执行定时节点错误,节点任务={} ,患者={}",dataItem.getId(),dataItem.getPatientId());
            e.printStackTrace();
        }
    }




}
