package com.xxh.cloud.dw.healthy.engine.modules.biz.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.file.FileAppender;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.bean.qr.RuleEventRunBO;
import com.xxh.cloud.common.mq.bean.qr.RuleEventRunItem;
import com.xxh.cloud.dw.healthy.engine.modules.data.bean.bos.DwdPatientDataBO;
import com.xxh.cloud.dw.healthy.engine.modules.data.service.PatientDataLabel2EventService;
import com.xxh.cloud.dw.healthy.engine.modules.data.service.RuleEngineDataService;
import com.xxh.cloud.dw.healthy.engine.modules.exe.service.DwdDataExecuteLabelService;
import com.xxh.cloud.dw.healthy.engine.modules.exe.service.DwdDataExecuteRuleEventService;
import com.xxh.cloud.dw.healthy.engine.modules.exe.utils.LogFileUtils;
import com.xxh.cloud.framework.common.utils.ConsoleUtils;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.rule.basic.context.XXRuleRoam;
import com.xxh.cloud.rule.execute.inference.client.bean.bos.WorkFlowExecuteResultBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class DataPatientEventRuleRunService {

    @Autowired
    private DataPatientCacheGetService dataPatientCacheGetService;
    @Autowired
    private PatientDataLabel2EventService patientDataLabel2EventService;
    @Autowired
    private DwdDataExecuteRuleEventService dwdDataExecuteRuleEventService;
    @Autowired
    private DataPatientRunExeCallbackService dataPatientRunExeCallbackService;
    @Autowired
    private DwdDataExecuteLabelService dwdDataExecuteLabelService;
    @Autowired
    private RuleEngineDataService ruleEngineDataService;
    @Autowired
    private Environment environment;





    public void process(List<RuleEventRunBO> list){
        List<String> patientIds=new ArrayList<>();

        list.forEach(item->{
            if(item!=null && StrUtil.isNotEmpty(item.getPid())){
                patientIds.add(item.getPid());
            }
        });

        if(CollUtil.isEmpty(patientIds)){
            return;
        }

        String serverPort =  environment.getProperty("server.port");
        FileAppender fileAppender= LogFileUtils.getFileAppender("Event_Exe",serverPort);

        List<DwdPatientDataBO> patientDataList=ruleEngineDataService.getDwdPatientDataList(patientIds);
        Map<String, XXRuleRoam> patientDataMap= ruleEngineDataService.getRuleRoamMapByData(patientDataList);

//        ConsoleUtils.log("+++++ 1. 执行患者标签引擎");
//        dwdDataExecuteLabelService.executeLabelRuleByPatientData(patientDataMap);

//        ConsoleUtils.log("+++++ 2. 重新获取患者标签数据");
//        List<DwdPatientDataBO> patientDataList2=ruleEngineDataService.reloadRuleRoamMap(patientIds,patientDataList);
//        Map<String, XXRuleRoam> patientDataMap2= ruleEngineDataService.getRuleRoamMapByData(patientDataList2);

        ConsoleUtils.log("+++++ 3. 执行患者事件引擎");
        Map<String,Map<String, Date>> eventData= patientDataLabel2EventService.getPatientEvent(patientIds);

        List<String> updateIds=new ArrayList<>();


        for(RuleEventRunBO item:list){
            if(item!=null && StrUtil.isNotEmpty(item.getPid())){
                XXRuleRoam ruleRoam= patientDataMap.get(item.getPid());
                Map<String, Date> eventMap= eventData.get(item.getPid());
                fileAppender.append("------------- 执行事件引擎开始----------------------");
                item.getEvent().forEach(event->{
                    fileAppender.append("********** 执行事件引擎,患者编号:"+item.getPid() +" ,事件ID: "+event.getEid() + " , 执行时间:"+ DateUtil.now());
                    try {
                        List<WorkFlowExecuteResultBO> eventResultList =  this.process2(event.getEid(),item.getPid(),ruleRoam,eventMap);
                        if(CollUtil.isNotEmpty(eventResultList)){
                            fileAppender.append("执行返回结果:");
                            eventResultList.forEach(eventResult->{
                                fileAppender.append(JSON.toJSONString(eventResult));
                            });
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        fileAppender.append("****执行保持:"+e.getMessage());
                    }
                });
                List<String> _ids= StrJoinSplitUtils.split2List(item.getIds());
                if(CollUtil.isNotEmpty(_ids)){
                    updateIds.addAll(_ids);
                }
            }
        }
        fileAppender.flush();
        dataPatientRunExeCallbackService.updateEventTaskStatus(updateIds);
    }


    public List<WorkFlowExecuteResultBO> processByPatient2NotCallBiz(String patientId,String eventId){
        XXRuleRoam patientData=ruleEngineDataService.getRuleRoam(patientId);
        Map<String, Date> eventData= patientDataLabel2EventService.getPatientEvent(patientId);
        List<WorkFlowExecuteResultBO> result = dwdDataExecuteRuleEventService.executeNotCallBiz(eventId,patientId,patientData,eventData);
        return result;
    }


    private  List<WorkFlowExecuteResultBO> process2(String eventId,String patientId, XXRuleRoam dataRoam,Map<String, Date> eventData){
        List<WorkFlowExecuteResultBO> result = dwdDataExecuteRuleEventService.execute(eventId,patientId,dataRoam,eventData);
        return result;
    }









}
