package com.xxh.cloud.data.run.modules.biz.run;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.IdUtil;
import com.xxh.cloud.config.datalabel.client.api.DataLabelRuleConfigClient;
import com.xxh.cloud.config.datalabel.client.bean.bos.DataLabelRuleConfigBO;
import com.xxh.cloud.config.datalabel.client.bean.dto.LabelRuleStatusUpdateDTO;
import com.xxh.cloud.config.datalabel.constant.LabelRuleExeStatusEnum;
import com.xxh.cloud.data.run.client.constant.RuleRunStatusEnum;
import com.xxh.cloud.data.run.modules.dw.bean.bos.DwsPatientDayDataBO;
import com.xxh.cloud.data.run.modules.dw.bean.dto.PatientDayDataListDTO;
import com.xxh.cloud.data.run.modules.dw.service.DwsPatientDayDataQueryBizService;
import com.xxh.cloud.data.run.modules.dw.utils.PatientEventTableWriteConfigUtils2;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.write.bean.DbTaskResult;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.framework.common.write.doris.DorisWriter;
import com.xxh.cloud.framework.common.write2.DorisWriteUtils;

import javax.sql.DataSource;
import java.util.*;

public class GenerateLabelRuleTask implements Runnable{




    private String cacheKey;

    //
    private DataLabelRuleConfigBO config;
    private String startTime;
    private String endTime;
    private DataSource dataSource;


    //@auto
    private DwsPatientDayDataQueryBizService patientDayDataQueryBizService;
    //@auto
    private RedisUtils redisUtils;
    //@auto
    private DataLabelRuleConfigClient dataLabelRuleConfigClient;




    public GenerateLabelRuleTask(DataLabelRuleConfigBO config ,
                                 RedisUtils redisUtils,
                                 DwsPatientDayDataQueryBizService patientDayDataQueryBizService,
                                 DataLabelRuleConfigClient dataLabelRuleConfigClient,
                                 DataSource dataSource){
        this.config=config;
        this.redisUtils=redisUtils;
        this.patientDayDataQueryBizService=patientDayDataQueryBizService;
        this.dataLabelRuleConfigClient=dataLabelRuleConfigClient;

        this.cacheKey=LabelRuleDataCacheKeys.getInitLabelRuleTaskFlag(config.getId());
        Date sd= DateUtil.offsetDay(config.getUpTime(),-config.getInitDateValue());
        this.startTime=DateUtil.format(sd,"yyyyMMdd");
        this.endTime=DateUtil.format(config.getUpTime(),"yyyyMMdd");

        this.dataSource=dataSource;
//        this.dburl=dburl;
//        this.dbusername=dbusername;
//        this.dbpassword=dbpassword;
    }


    @Override
    public void run() {

        long idx=0;
        int pageNum=10000;

        while (true){
            ThreadUtil.safeSleep(1000);
            Object cacheVal= this.redisUtils.get(this.cacheKey);
            if(cacheVal==null){
                return;
            }
            boolean flag= this.read2write(idx,pageNum);
            if(flag==false){
                System.out.println("++++++++++++ 读取数据结束");
                this.redisUtils.delete(this.cacheKey);
                return;
            }
            idx=idx+pageNum;
        }

    }


    private boolean read2write(long startNum ,long pageNum){
        System.out.println("++++++ read2write: "+startNum+" , " +pageNum);
        PatientDayDataListDTO params=new PatientDayDataListDTO();
        params.setStart(startNum);
        params.setPageNum(pageNum);
        params.setStartTime(this.startTime);
        params.setEndTime(this.endTime);
        List<DwsPatientDayDataBO> list= patientDayDataQueryBizService.listPatientDayData(params);
        if(CollUtil.isEmpty(list)){
            this.dataLabelRuleConfigClient.updateLabelRuleConfigStatus(
                    new LabelRuleStatusUpdateDTO(this.config.getId(), LabelRuleExeStatusEnum.LRES_Ing.getCode()));
            return false;
        }

        System.out.println("++++++++++++++读取条数:"+list.size());

        HwdsWriteDBConfigBean writeDBConfig= this.get_PatientHisTagRuleIngInfo();

//        writeDBConfig.setUrl(this.dburl);
//        writeDBConfig.setUsername(this.dbusername);
//        writeDBConfig.setPassword(this.dbpassword);
//        DorisWriter dorisWriter=new DorisWriter();

        List<Map<String, Object>> dataList=new ArrayList<>();

        for(DwsPatientDayDataBO item:list){
            Map<String, Object> dataItemMap=new HashMap<>();
            dataItemMap.put("id", IdUtil.getSnowflake(1,1).nextIdStr());
            dataItemMap.put("label_rule_id",this.config.getId());
            dataItemMap.put("label_id",this.config.getLabelIds());
            dataItemMap.put("patient_id",item.getPatientId());
            dataItemMap.put("op_status", RuleRunStatusEnum.None.getCode());
            dataItemMap.put("op_num",0);
            dataList.add(dataItemMap);
        }

        DorisWriteUtils.batchWrite(this.dataSource,writeDBConfig,dataList);


        //DbTaskResult writerResult= dorisWriter.writer(writeDBConfig,dataList);
        return true;
    }



    private HwdsWriteDBConfigBean get_PatientHisTagRuleIngInfo(){
        String tableCode="xe_patient_his_tag_rule_ing_info";
        Map<String,String> colEntityMap=new HashMap<>();
        colEntityMap.put("id","id");
        colEntityMap.put("label_rule_id","label_rule_id");
        colEntityMap.put("label_id","label_id");
        colEntityMap.put("patient_id","patient_id");
        colEntityMap.put("op_status","op_status");
        colEntityMap.put("op_num","op_num");
        HwdsWriteDBConfigBean config=new HwdsWriteDBConfigBean(tableCode,colEntityMap);
        return config;
    }








}
