package com.xxh.cloud.data.run.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.xxh.cloud.common.mq.bean.MQPatientEventBean;
import com.xxh.cloud.common.mq.bean.MQRuleLabelBean;
import com.xxh.cloud.common.mq.constant.PatientRuleOpTypeEnum;
import com.xxh.cloud.data.run.client.constant.RuleRunStatusEnum;
import com.xxh.cloud.data.run.modules.base.entity.PatientEventRuleIngInfo;
import com.xxh.cloud.data.run.modules.base.entity.PatientFlowRuleIngInfo;
import com.xxh.cloud.data.run.modules.base.service.PatientEventRuleIngInfoService;
import com.xxh.cloud.data.run.modules.base.service.PatientFlowRuleIngInfoService;
import com.xxh.cloud.data.run.modules.biz.bean.bos.PatientEventTaskNoneBO;
import com.xxh.cloud.data.run.modules.biz.bean.bos.PatientEventTaskNoneItem;
import com.xxh.cloud.data.run.modules.biz.bean.bos.PatientEventTaskNoneResp;
import com.xxh.cloud.data.run.modules.biz.bean.mos.PatientRuleTaskQueryMO;
import com.xxh.cloud.data.run.modules.biz.mapper.PatientLabelRuleHisDataQueryMapper;
import com.xxh.cloud.data.run.modules.biz.service.PatientEventRuleIngBizService;
import com.xxh.cloud.data.run.modules.biz.service.PatientFlowRuleIngBizService;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Log4j2
@Service
public class PatientEventRuleIngBizServiceImpl implements PatientEventRuleIngBizService {


    @Autowired
    private PatientFlowRuleIngInfoService patientFlowRuleIngInfoService;
    @Autowired
    private PatientEventRuleIngInfoService patientEventRuleIngInfoService;
    @Autowired
    private PatientLabelRuleHisDataQueryMapper patientLabelRuleHisDataQueryMapper;

    @Autowired
    private PatientFlowRuleIngBizService patientFlowRuleIngBizService;

    private String getEventEntityId(String patientId,String eventType,String eventDate,String relObjType,String relObjId){
        String str= StrUtil.format("{}:{}:{}:{}:{}",patientId,eventType,eventDate,relObjType,relObjId);
        return SecureUtil.md5(str).toUpperCase();
    }


    @Override
    public void addEventIng(List<MQPatientEventBean> list) {
        if(CollUtil.isEmpty(list)){
            return;
        }
        Map<String,MQPatientEventBean> patientEventMap=new HashMap<>();
        List<PatientEventRuleIngInfo> eventDataList=new ArrayList<>();

        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getPatientId())
                    && StrUtil.isNotEmpty(item.getEventType())
                    && StrUtil.isNotEmpty(item.getEventDate())
                    && StrUtil.isNotEmpty(item.getObjType())
                    && StrUtil.isNotEmpty(item.getObjId())){
                patientEventMap.put(this.getEventEntityId(item.getPatientId(),item.getEventType(),item.getEventDate(),item.getObjType(),item.getObjId()),item);
            }
        });

        if(MapUtil.isEmpty(patientEventMap)){
            System.out.println("+++++ 事件数据不符合");
            return;
        }

        List<String> dbIds=new ArrayList<>();
        List<PatientEventRuleIngInfo> dbList= patientEventRuleIngInfoService.listByIds(patientEventMap.keySet());
        if(CollUtil.isNotEmpty(dbList)){
            dbList.forEach(item->{
                dbIds.add(item.getId());
            });
        }

        for(String key:patientEventMap.keySet()){
            MQPatientEventBean item = patientEventMap.get(key);
            if(!dbIds.contains(key)) {
                PatientEventRuleIngInfo entity=new PatientEventRuleIngInfo();
                String id= this.getEventEntityId(item.getPatientId(),item.getEventType(),item.getEventDate(),item.getObjType(),item.getObjId());
                entity.setId(id);
                entity.setPatientId(item.getPatientId());
                entity.setEventId(item.getEventType());
                entity.setEventTime(Convert.toDate(item.getEventDate()));
                entity.setAddTime(new Date());
                entity.setOpStatus(RuleRunStatusEnum.None.getCode());
                entity.setOpNum(0);
                entity.setOpTime(new Date());
                entity.setRelObjType(item.getObjType());
                entity.setRelObjId(item.getObjId());
                eventDataList.add(entity);
            }
        }

        if(eventDataList.size()>0){
            patientEventRuleIngInfoService.saveBatch(eventDataList);
        }

        List<MQRuleLabelBean> ruleLabelBeanList =  this.changeMQRuleLabel(list);
        patientFlowRuleIngBizService.addIng(ruleLabelBeanList);

    }



    private List<MQRuleLabelBean> changeMQRuleLabel(List<MQPatientEventBean> list){
        List<MQRuleLabelBean> data=new ArrayList<>();
        list.forEach(item->{
            MQRuleLabelBean dataItem=new MQRuleLabelBean(item.getPatientId(), item.getObjType(),item.getEventDate(),item.getObjId());
            data.add(dataItem);
        });
        return data;
    }



    @Override
    public PatientEventTaskNoneResp getEventNoneTaskList(int num,int minute,String eventId) {
        Date date=new Date();
        Date start= DateUtil.offsetMinute(date,-120);
//        Date end=DateUtil.offsetMinute(date,-minute);
        List<PatientRuleTaskQueryMO> list= patientLabelRuleHisDataQueryMapper.queryPatientEventTask(
                new QueryWrapper<PatientRuleTaskQueryMO>()
                        .eq("op_status",RuleRunStatusEnum.None.getCode())
                        .eq(StrUtil.isNotEmpty(eventId),"event_id",eventId)
                        .ge("add_time",start)
//                        .le("add_time",end)
                        .groupBy("patient_id")
                        .last(" LIMIT " + num)
        );


        if(CollUtil.isEmpty(list)){
            return null;
        }

        Map<String,PatientEventTaskNoneBO> dataMap=new HashMap<>();
        Set<String> ids=new HashSet<>();
        list.forEach(item->{
            dataMap.put(item.getPatientId(),new PatientEventTaskNoneBO(item.getPatientId(),item.getIds()));
            List<String> _ids= StrJoinSplitUtils.split2List(item.getIds());
            ids.addAll(_ids);
        });


        List<PatientEventRuleIngInfo> eventRuleList= patientEventRuleIngInfoService.listByIds(ids);

        if(CollUtil.isNotEmpty(eventRuleList)){
            eventRuleList.forEach(item->{
                PatientEventTaskNoneBO patientEventTask = dataMap.get(item.getPatientId());
                if(patientEventTask!=null){
                    patientEventTask.addEvent(new PatientEventTaskNoneItem(item.getEventId(),item.getEventTime()));
                }
            });
        }



        List<PatientEventTaskNoneBO> data=new ArrayList<>();
        dataMap.forEach((pid,item)->{
            data.add(item);
        });

        PatientEventTaskNoneResp resp=new PatientEventTaskNoneResp();
        resp.setList(data);
        resp.setIds(ids);

        return resp;
    }

    @Override
    public void updateIng(Set<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return;
        }
        patientEventRuleIngInfoService.update(
                new UpdateWrapper<PatientEventRuleIngInfo>()
                        .set("op_status",RuleRunStatusEnum.Ing.getCode())
                        .set("op_time",new Date())
                        .set("op_num",1)
                        .in("id",ids)
                        .eq("op_status",RuleRunStatusEnum.None.getCode())
        );
    }

}
