package com.xxh.cloud.data.run.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.xxh.cloud.common.mq.bean.MQRuleLabelBean;
import com.xxh.cloud.common.mq.constant.DataSceneType;
import com.xxh.cloud.common.mq.constant.PatientRuleOpTypeEnum;
import com.xxh.cloud.data.run.client.constant.RuleRunStatusEnum;
import com.xxh.cloud.data.run.modules.base.entity.PatientEventRuleIngInfo;
import com.xxh.cloud.data.run.modules.base.entity.PatientFlowRuleIngInfo;
import com.xxh.cloud.data.run.modules.base.service.PatientFlowRuleIngInfoService;
import com.xxh.cloud.data.run.modules.biz.bean.bos.PatientFlowTaskNoneBO;
import com.xxh.cloud.data.run.modules.biz.bean.mos.PatientRuleTaskQueryMO;
import com.xxh.cloud.data.run.modules.biz.mapper.PatientLabelRuleHisDataQueryMapper;
import com.xxh.cloud.data.run.modules.biz.service.PatientFlowRuleIngBizService;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class PatientFlowRuleIngBizServiceImpl implements PatientFlowRuleIngBizService {

    @Autowired
    private PatientFlowRuleIngInfoService patientFlowRuleIngInfoService;
    @Autowired
    private PatientLabelRuleHisDataQueryMapper patientLabelRuleHisDataQueryMapper;


    private String getId(String type, String id){
        if(StrUtil.isNotEmpty(type) && DataSceneType.DS_ExamineReport.equals(type)){
            return "ER"+IdUtil.getSnowflake(3,1).nextIdStr();
        } else if(StrUtil.isNotEmpty(type) && StrUtil.isNotEmpty(id)){
            String str= StrUtil.format("REAL_{}_{}",type,id);
            return SecureUtil.md5(str).toUpperCase();
        }else {
            return IdUtil.getSnowflake(3,1).nextIdStr();
        }
    }

    @Override
    public void addIng(List<MQRuleLabelBean> data) {
        if(CollUtil.isEmpty(data)){
            return;
        }

        Set<String> ids=new HashSet<>();
        data.forEach(item->{
            if(StrUtil.isNotEmpty(item.getSt()) && StrUtil.isNotEmpty(item.getDid())){
               String id= this.getId(item.getSt(),item.getDid());
                ids.add(id);
            }
        });

        List<String> dbIds=new ArrayList<>();
        if(CollUtil.isNotEmpty(ids)){
            List<PatientFlowRuleIngInfo> pfrList= patientFlowRuleIngInfoService.listByIds(ids);
            if(CollUtil.isNotEmpty(pfrList)){
                pfrList.forEach(item->{
                    dbIds.add(item.getId());
                });
            }
        }

        List<PatientFlowRuleIngInfo> list=new ArrayList<>();

        data.forEach(item->{
            if(item!=null && StrUtil.isNotEmpty(item.getPid())){
                String id=  this.getId(item.getSt(),item.getDid());
                if(!dbIds.contains(id)){
                    PatientFlowRuleIngInfo entity=new PatientFlowRuleIngInfo();
                    entity.setId(id);
                    entity.setPatientId(item.getPid());
                    if(StrUtil.isNotEmpty(item.getSt())){
                        entity.setSceneType(item.getSt());
                    }else {
                        entity.setSceneType(DataSceneType.DS_Def);
                    }

                    entity.setDataId(item.getDid());

                    if(StrUtil.isNotEmpty(item.getDt())){
                        entity.setDataTime(item.getDt());
                    }else {
                        entity.setDataTime(DateUtil.now());
                    }
                    entity.setAddTime(new Date());
                    entity.setOpStatus(RuleRunStatusEnum.None.getCode());
                    entity.setOpNum(0);
                    entity.setOpTime(new Date());
                    entity.setOpType(PatientRuleOpTypeEnum.RealTime.getCode());
                    list.add(entity);
                }
            }
        });

        if(list.size()>0){
            patientFlowRuleIngInfoService.saveBatch(list);
        }

    }

    @Override
    public List<PatientFlowTaskNoneBO> getFlowNoneTaskList(int num,int minute,String opType,String sceneType,List<String> notSceneTypeList) {
        Date date=new Date();

        Date end=DateUtil.offsetMinute(date,-minute);




        List<PatientFlowTaskNoneBO> list= patientLabelRuleHisDataQueryMapper.queryPatientFlowTask(
                new QueryWrapper<PatientFlowTaskNoneBO>()
                        .eq("op_status",RuleRunStatusEnum.None.getCode())
                        .le("add_time",end)
                        .eq(StrUtil.isNotEmpty(opType),"op_type",opType)
                        .eq(StrUtil.isNotEmpty(sceneType),"scene_type",sceneType)
                        .notIn(CollUtil.isNotEmpty(notSceneTypeList),"scene_type",notSceneTypeList)
                        .groupBy("patient_id")
                        .orderByDesc("max(add_time)")
                        .last(" LIMIT " + num)
        );
        return list;
    }





    @Override
    public void updateIng(Set<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return;
        }
        patientFlowRuleIngInfoService.update(
                new UpdateWrapper<PatientFlowRuleIngInfo>()
                        .set("op_status",RuleRunStatusEnum.Ing.getCode())
                        .set("op_time",new Date())
                        .set("op_num",1)
                        .in("id",ids)
                        .eq("op_status",RuleRunStatusEnum.None.getCode())
        );
    }


}
