package com.xxh.cloud.data.run.modules.biz.service.impl;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.xxh.cloud.config.datalabel.client.api.DataLabelRuleConfigClient;
import com.xxh.cloud.config.datalabel.client.bean.bos.DataLabelRuleConfigBO;
import com.xxh.cloud.config.datalabel.client.bean.dto.LabelRuleStatusUpdateDTO;
import com.xxh.cloud.config.datalabel.constant.LabelRuleExeStatusEnum;
import com.xxh.cloud.config.datalabel.constant.LabelRuleInitDateType;
import com.xxh.cloud.data.run.modules.biz.bean.bos.DbUpdateBO;
import com.xxh.cloud.data.run.modules.biz.bean.dto.LabelRuleRunMgrDTO;
import com.xxh.cloud.data.run.modules.biz.bean.vos.LabelRuleRunMgrVO;
import com.xxh.cloud.data.run.modules.biz.mapper.PatientHisTagRuleOpMapper;
import com.xxh.cloud.data.run.modules.biz.run.GenerateLabelRuleTask;
import com.xxh.cloud.data.run.modules.biz.run.LabelRuleDataCacheKeys;
import com.xxh.cloud.data.run.modules.biz.service.PatientLabelRuleMgrBizService;
import com.xxh.cloud.data.run.modules.dw.service.DwsPatientDayDataQueryBizService;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.web.common.config.XmWriteConfig;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import javax.sql.DataSource;

@DS("master")
@Log4j2
@Service
public class PatientLabelRuleMgrBizServiceImpl implements PatientLabelRuleMgrBizService {

    @Autowired
    private DataLabelRuleConfigClient dataLabelRuleConfigClient;
    @Autowired
    private PatientHisTagRuleOpMapper patientHisTagRuleOpMapper;
    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private DwsPatientDayDataQueryBizService patientDayDataQueryBizService;

//    @Autowired
//    private XmWriteConfig xmWriteConfig;

    @Autowired
    private JdbcTemplate jdbcTemplate;


    private DataSource dataSource;

    @Override
    public LabelRuleRunMgrVO runManager(LabelRuleRunMgrDTO params) {

        DataLabelRuleConfigBO dataLabelRuleConfigBO= dataLabelRuleConfigClient.getLabelRuleConfigById(params.getId());
        if(dataLabelRuleConfigBO==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(!LabelRuleInitDateType.LRDT_Before.getCode().equals(dataLabelRuleConfigBO.getInitDateType())){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"非法操作[非提前执行]");
        }

        String exeStatus= dataLabelRuleConfigBO.getInitExeStatus();
        if(StrUtil.isEmpty(exeStatus)){
            exeStatus=LabelRuleExeStatusEnum.LRES_None.getCode();
        }


        if(LabelRuleExeStatusEnum.LRES_Stop.getCode().equals(params.getStatus())){
            if(!LabelRuleExeStatusEnum.LRES_Ing.getCode().equals(exeStatus)){
               throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"非法操作[非执行中]");
            }
            this.stop(dataLabelRuleConfigBO,params.getId());
        }

        if(LabelRuleExeStatusEnum.LRES_Start.getCode().equals(params.getStatus())){
            if(LabelRuleExeStatusEnum.LRES_Ing.getCode().equals(exeStatus)){
                throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"非法操作[执行中],请先停止执行");
            }
            this.start(dataLabelRuleConfigBO);
        }

        return null;
    }



    private void start(DataLabelRuleConfigBO config){
        //1.清除已生成的标签
//        patientHisTagRuleOpMapper.deletePatientLabel(
//                new UpdateWrapper<DbUpdateBO>().eq("b.label_rule_id",config.getId())
//        );
//
//        patientHisTagRuleOpMapper.deletePatientLabelHis(
//                new UpdateWrapper<DbUpdateBO>().eq("b.label_rule_id",config.getId()) );

        //2.清除已生成的任务
        patientHisTagRuleOpMapper.deletePatientLabelRuleTask(
                new UpdateWrapper<DbUpdateBO>().eq("label_rule_id",config.getId())
        );

        //3.重新生成任务
        this.generateTask(config);

    }


    private void generateTask(DataLabelRuleConfigBO config){
        String cacheKey = LabelRuleDataCacheKeys.getInitLabelRuleTaskFlag(config.getId());
        redisUtils.set(cacheKey,"Y");
        if(this.dataSource==null){
            System.out.println("++++ init dataSource ............");
            this.dataSource = jdbcTemplate.getDataSource();
        }
        GenerateLabelRuleTask task=new GenerateLabelRuleTask(config,redisUtils,patientDayDataQueryBizService,dataLabelRuleConfigClient, this.dataSource
        );
        ThreadUtil.execAsync(task,true);
    }


    private void stop(DataLabelRuleConfigBO config,String id){
        dataLabelRuleConfigClient.updateLabelRuleConfigStatus(new LabelRuleStatusUpdateDTO(id, LabelRuleExeStatusEnum.LRES_Stop.getCode()));
        String cacheKey = LabelRuleDataCacheKeys.getInitLabelRuleTaskFlag(config.getId());
        redisUtils.delete(cacheKey);
        //todo:
    }


}
