package com.xxh.cloud.data.run.modules.task;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.bean.MQCleanTableIds;
import com.xxh.cloud.common.mq.bean.qr.RuleEventRunBO;
import com.xxh.cloud.common.mq.bean.qr.RuleEventRunItem;
import com.xxh.cloud.common.mq.constant.HMQQueueName;
import com.xxh.cloud.common.mq.constant.HMQQueueName2EventExe;
import com.xxh.cloud.data.run.modules.biz.bean.bos.PatientEventTaskNoneBO;
import com.xxh.cloud.data.run.modules.biz.bean.bos.PatientEventTaskNoneResp;
import com.xxh.cloud.data.run.modules.biz.service.PatientEventRuleIngBizService;
import com.xxh.cloud.data.run.modules.config.DataRunTaskConfig;
import com.xxh.cloud.framework.activemq.constant.ActiveMQWayEnum;
import com.xxh.cloud.framework.activemq.utils.ActiveMQSendUtils;
import com.xxh.cloud.framework.common.utils.ConsoleUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Component
@Log4j2
public class DataRunEventTask {

    @Autowired
    private DataRunTaskConfig dataRunTaskConfig;

    @Autowired
    private PatientEventRuleIngBizService patientEventRuleIngBizService;


    /**
     * 发送默认事件到引擎
     * */
    @XxlJob("sendEventRunTask")
    public void sendEventRunTask(){
        ConsoleUtils.log("++++++ 发送事件规则执行任务:启动任务{}", DateUtil.now());
        //1.
        PatientEventTaskNoneResp data = patientEventRuleIngBizService.getEventNoneTaskList(dataRunTaskConfig.getEventDataNum(),dataRunTaskConfig.getEventIntervalMinute(),null);
        if(data==null || CollUtil.isEmpty(data.getList())){
            ConsoleUtils.log("++++++ 发送事件规则执行任务:无数据");
            return;
        }

        //2.
        List<List<PatientEventTaskNoneBO>> lists= ListUtil.split(data.getList(),5);
        for(List<PatientEventTaskNoneBO> objList:lists){
            List<RuleEventRunBO> rerList= this.getRuleEventRunBOList(objList);
            if(rerList.size()>0){
                //todo: wjj 改成实时队列，发送;
                ActiveMQSendUtils.send(ActiveMQWayEnum.MQ_SW_Queue, HMQQueueName2EventExe.Que_Event_Exe_Def, JSON.toJSONString(rerList));
            }
        }
        //3.
        patientEventRuleIngBizService.updateIng(data.getIds());
    }


    /**
     * 发送指定事件到指定引擎
     * */
    @XxlJob("sendEventBySceneTask")
    public void sendEventBySceneTask(){
        ConsoleUtils.log("++++++ 发送指定事件类型:启动任务{}", DateUtil.now());

        Map<String,Object> configMap = DataRunTaskUtils.getXxlParamMap();
        if(MapUtil.isEmpty(configMap)){
            System.out.println("+++++ 参数不能为空");
            return;
        }
        Object eventIdObj= configMap.get("eventId");
        String eventId=eventIdObj==null?null:Convert.toStr(eventIdObj);
        if(StrUtil.isEmpty(eventId)){
            System.out.println("+++++ 参数[eventId]不能为空");
            return;
        }

        String queueName= HMQQueueName2EventExe.getQueueName(eventId);
        if(StrUtil.isEmpty(queueName)){
            System.out.println("+++++ 参数[eventId]未配置执行队列");
            return;
        }

        //1.
        PatientEventTaskNoneResp data = patientEventRuleIngBizService.getEventNoneTaskList(dataRunTaskConfig.getEventDataNum(),dataRunTaskConfig.getEventIntervalMinute(), eventId);
        if(data==null || CollUtil.isEmpty(data.getList())){
            ConsoleUtils.log("++++++ 发送事件规则执行任务:无数据");
            return;
        }

        //2.
        List<List<PatientEventTaskNoneBO>> lists= ListUtil.split(data.getList(),5);
        for(List<PatientEventTaskNoneBO> objList:lists){
            List<RuleEventRunBO> rerList= this.getRuleEventRunBOList(objList);
            if(rerList.size()>0){
                System.out.println("------------- 队列名称 :"+queueName);
                ActiveMQSendUtils.send(ActiveMQWayEnum.MQ_SW_Queue, queueName, JSON.toJSONString(rerList));
            }
        }

        //3.
        patientEventRuleIngBizService.updateIng(data.getIds());
    }




    private List<RuleEventRunBO> getRuleEventRunBOList(List<PatientEventTaskNoneBO> list){
        List<RuleEventRunBO> data=new ArrayList<>();
        list.forEach(item->{
            data.add(this.getRuleEventRunBO(item));
        });
        return data;
    }


    private RuleEventRunBO getRuleEventRunBO(PatientEventTaskNoneBO entity){
        List<RuleEventRunItem> event=new ArrayList<>();
        entity.getEvents().forEach(item->{
            event.add(new RuleEventRunItem(item.getEventId(),item.getEventTime()));
        });
        RuleEventRunBO data=new RuleEventRunBO(entity.getPatientId(),entity.getIds(),event);
        return data;
    }





}
