package com.xxh.cloud.data.run.modules.task;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.bean.qr.RuleLabelRunBO;
import com.xxh.cloud.common.mq.constant.HMQQueueName;
import com.xxh.cloud.common.mq.constant.HMQQueueName2RuleExe;
import com.xxh.cloud.common.mq.constant.PatientRuleOpTypeEnum;
import com.xxh.cloud.data.run.modules.biz.bean.bos.PatientFlowTaskNoneBO;
import com.xxh.cloud.data.run.modules.biz.service.PatientFlowRuleIngBizService;
import com.xxh.cloud.data.run.modules.config.DataRunTaskConfig;
import com.xxh.cloud.framework.activemq.constant.ActiveMQWayEnum;
import com.xxh.cloud.framework.activemq.utils.ActiveMQSendUtils;
import com.xxh.cloud.framework.common.utils.ConsoleUtils;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

@Component
@Log4j2
public class DataRunLabelTask {


    @Autowired
    private DataRunTaskConfig dataRunTaskConfig;

    @Autowired
    private PatientFlowRuleIngBizService patientFlowRuleIngBizService;


    /**
     * 发送标签规则执行任务-历史
     * */
    @XxlJob("sendLabelRunTask_his")
    public void sendLabelTaskHis(){
        ConsoleUtils.log("++++++ 发送标签规则执行任务:启动任务 - 历史 {}", DateUtil.now());
        String queueType= DataRunTaskUtils.getQueueType();
        //1.
        List<PatientFlowTaskNoneBO> list= patientFlowRuleIngBizService.getFlowNoneTaskList(
                dataRunTaskConfig.getLableDataMum(),
                dataRunTaskConfig.getLableIntervalMinute(),
                PatientRuleOpTypeEnum.History.getCode(),null,null);
        if(CollUtil.isEmpty(list)){
            ConsoleUtils.log("++++++ 发送标签规则执行任务 - 历史 :无数据");
            return;
        }
        this.process(HMQQueueName2RuleExe.Que_Label_RunHis,list);
    }



    /**
     * 发送标签规则执行任务-实时
     * */
    @XxlJob("sendLabelRunTask_real")
    public void sendLabelTaskReal(){
        ConsoleUtils.log("++++++ 发送标签规则执行任务:启动任务 - 实时 {}", DateUtil.now());
        String queueType= DataRunTaskUtils.getQueueType();

        //1.
        List<PatientFlowTaskNoneBO> list= patientFlowRuleIngBizService.getFlowNoneTaskList(
                dataRunTaskConfig.getLableDataMum(),
                dataRunTaskConfig.getLableIntervalMinute(),
                PatientRuleOpTypeEnum.RealTime.getCode(),null,null);

        if(CollUtil.isEmpty(list)){
            ConsoleUtils.log("++++++ 发送标签规则执行任务 - 实时 :无数据");
            return;
        }

        this.process(HMQQueueName2RuleExe.Que_Label_RunReal,list);
    }





    private void process(String mqQueueName,List<PatientFlowTaskNoneBO> list){

        //2.
        Set<String> ids=new HashSet<>();
        List<List<PatientFlowTaskNoneBO>> lists= ListUtil.split(list,3);

        for(List<PatientFlowTaskNoneBO> objList:lists){
            List<RuleLabelRunBO> mqDataList=new ArrayList<>();
            objList.forEach(objItem->{
                mqDataList.add(new RuleLabelRunBO(objItem.getPatientId(),objItem.getIds(),objItem.getSceneTypes(),objItem.getMdt()));
                List<String> _ids= StrJoinSplitUtils.split2List(objItem.getIds());
                ids.addAll(_ids);
            });

            System.out.println("++++++ 发送队列数据:"+mqQueueName);
            System.out.println(JSON.toJSONString(mqDataList));

            ActiveMQSendUtils.send(ActiveMQWayEnum.MQ_SW_Queue, mqQueueName, JSON.toJSONString(mqDataList));
        }
        //3.
        patientFlowRuleIngBizService.updateIng(ids);
    }





}
