package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdInpatientRecordInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.mapper.DeptRelationBizMapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DeptRelationBizService;

import cn.hutool.core.collection.CollectionUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class DeptRelationBizServiceImpl implements DeptRelationBizService {

    @Autowired
    private DeptRelationBizMapper deptRelationBizMapper;

    @Override
    public List<String> listByDeptId(String deptId) {
        String parentDeptId = deptRelationBizMapper.getParentDeptId(deptId);
        if ("0".equals(parentDeptId)) {
            parentDeptId = deptId; // 若上级部门为0则本部门就是顶级部门
        }

        List<String> deptIds = deptRelationBizMapper.getSubDeptId(parentDeptId);
        deptIds.add(parentDeptId); // 把上级部门id也加入
        return deptIds;
    }

    @Override
    public Map<String, List<String>> listByDeptIds(List<String> deptIds) {
        log.info("1.DeptRelationBizServiceImpl.listByDeptIds>>>>>> {}", JSON.toJSONString(deptIds));

        Map<String, List<String>> deptIdMap = new HashMap<>(); // key为住院科室id，value为住院申请科室的id
        
        List<Map<String, String>> parentDepts = deptRelationBizMapper.getParentDeptIds(new QueryWrapper<DwdInpatientRecordInfo>()
                .in("id", deptIds)
        );
        log.info("2.查询科室信息>>>>>> {}", JSON.toJSONString(parentDepts));
        if (CollectionUtil.isEmpty(parentDepts)) {
            return deptIdMap;
        }

        Map<String, String> idPidMap = new HashMap<>(); // key为实际住院科室id，value为key对应的pid

        List<String> pids = new ArrayList<>();
        parentDepts.forEach(i -> {
            String id = String.valueOf(i.get("id"));
            String pid = String.valueOf(i.get("pid"));
            if ("0".equals(pid)) {
                pids.add(id);
                idPidMap.put(id, id);
            } else {
                pids.add(pid);
                idPidMap.put(id, pid);
            }
        });
        log.info("3.整理科室信息>>>>>> {}", JSON.toJSONString(idPidMap));
        
        Map<String, List<String>> subDeptMap = new HashMap<>(); // key为住院科室对应的pid，value为住院申请科室的id

        List<Map<String, String>> subDepts = deptRelationBizMapper.getSubDeptIds(new QueryWrapper<DwdInpatientRecordInfo>()
                .in("pid", pids)
        );
        log.info("4.根据pid查询所有子科室>>>>>> {}", JSON.toJSONString(subDepts));
        if (CollectionUtil.isNotEmpty(subDepts)) {
            subDepts.forEach(i -> {
                String id = String.valueOf(i.get("id"));
                String pid = String.valueOf(i.get("pid"));

                List<String> subDeptIds = subDeptMap.get(pid);
                if (null == subDeptIds) {
                    subDeptIds = new ArrayList<>();
                    subDeptMap.put(pid, subDeptIds);
                }
                subDeptIds.add(id);
            });
        }

        log.info("5.整理科室信息>>>>>> {}", JSON.toJSONString(subDepts));

        idPidMap.forEach((k, v) -> {
            List<String> l = subDeptMap.get(v);
            if (null == l) {
                l = new ArrayList<>();
            }
            l.add(v);
            deptIdMap.put(k, l);
        });

        log.info("6.整理科室信息>>>>>> {}", JSON.toJSONString(deptIdMap));

        return deptIdMap;
    }

}
