package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdDiagnosisReportInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdDiagnosisReportInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdDiagnosisReportQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdDiagnosisReportVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.mapper.DwdPatientHealthInfoBizMapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdDiagnosisReportBizService;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * DWD诊断数据表 服务类
 * */
@DS("wd")
@Service
public class DwdDiagnosisReportBizServiceImpl implements DwdDiagnosisReportBizService {

    @Autowired
    private DwdDiagnosisReportInfoService dwdDiagnosisReportInfoService;
    @Autowired
    private DwdPatientHealthInfoBizMapper dwdPatientHealthInfoBizMapper;

    @Override
    public PageData<DwdDiagnosisReportVO> query(DwdDiagnosisReportQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        IPage<DwdDiagnosisReportInfo> page = dwdDiagnosisReportInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DwdDiagnosisReportInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .eq(StrUtil.isNotBlank(params.getSceneTypeRaw()), "scene_type_raw", params.getSceneTypeRaw())
                .in(CollectionUtil.isNotEmpty(params.getSceneTypeRaws()), "scene_type_raw", params.getSceneTypeRaws())
                .eq(StrUtil.isNotBlank(params.getSceneTypeStandard()), "scene_type_standard", params.getSceneTypeStandard())
                .in(CollectionUtil.isNotEmpty(params.getSceneTypeStandards()), "scene_type_standard", params.getSceneTypeStandards())
                .eq(null != params.getSortIdx(), "sort_idx", params.getSortIdx())
                .in(CollectionUtil.isNotEmpty(params.getSortIdxs()), "sort_idx", params.getSortIdxs())
                .ge(null != params.getStartDate(), "diagnosis_time", params.getStartDate())
                .lt(null != params.getEndDate(), "diagnosis_time", params.getEndDate())
                .orderBy(true, true, "sort_idx")
                .orderBy(true, false, "diagnosis_time")
        );
        
        PageData<DwdDiagnosisReportVO> data = new PageConverter<DwdDiagnosisReportInfo, DwdDiagnosisReportVO>() {
            @Override
            public DwdDiagnosisReportVO converter(DwdDiagnosisReportInfo entity) {
                DwdDiagnosisReportVO vo = PatientHealthyHelper.getDwdDiagnosisReportVO(entity);
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<DwdDiagnosisReportVO> list(DwdDiagnosisReportQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }
        
        List<DwdDiagnosisReportInfo> data = dwdDiagnosisReportInfoService.list(new QueryWrapper<DwdDiagnosisReportInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .eq(StrUtil.isNotBlank(params.getSceneTypeRaw()), "scene_type_raw", params.getSceneTypeRaw())
                .in(CollectionUtil.isNotEmpty(params.getSceneTypeRaws()), "scene_type_raw", params.getSceneTypeRaws())
                .eq(StrUtil.isNotBlank(params.getSceneTypeStandard()), "scene_type_standard", params.getSceneTypeStandard())
                .in(CollectionUtil.isNotEmpty(params.getSceneTypeStandards()), "scene_type_standard", params.getSceneTypeStandards())
                .eq(null != params.getSortIdx(), "sort_idx", params.getSortIdx())
                .in(CollectionUtil.isNotEmpty(params.getSortIdxs()), "sort_idx", params.getSortIdxs())
                .ge(null != params.getStartDate(), "diagnosis_time", params.getStartDate())
                .lt(null != params.getEndDate(), "diagnosis_time", params.getEndDate())
                .orderBy(true, true, "sort_idx")
                .orderBy(true, false, "diagnosis_time")
        );
        return data.stream().map(PatientHealthyHelper::getDwdDiagnosisReportVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdDiagnosisReportInfo> listByPatientId(String patientId) {
        List<DwdDiagnosisReportInfo> data =  dwdDiagnosisReportInfoService.list(new QueryWrapper<DwdDiagnosisReportInfo>()
                .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdDiagnosisReportInfo> listByPatientId(List<String> patientIds) {
        List<DwdDiagnosisReportInfo> data =  dwdDiagnosisReportInfoService.list(new QueryWrapper<DwdDiagnosisReportInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }

    @Override
    public List<DwdDiagnosisReportInfo> listBeanByPatient(String patientId) {
        List<DwdDiagnosisReportInfo> data =  dwdDiagnosisReportInfoService.list(new QueryWrapper<DwdDiagnosisReportInfo>()
                .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdDiagnosisReportInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdDiagnosisReportInfo> data =  dwdDiagnosisReportInfoService.list(new QueryWrapper<DwdDiagnosisReportInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }

    @Override
    public PageData<DwdDiagnosisReportVO> groupQuery(DwdDiagnosisReportQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }
        
        IPage<DwdDiagnosisReportInfo> page = dwdPatientHealthInfoBizMapper.diagnosisReportGroupPage(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DwdDiagnosisReportInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "a.patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "a.patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "a.visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "a.visit_id", params.getVisitIds())
                .eq(StrUtil.isNotBlank(params.getSceneTypeRaw()), "a.scene_type_raw", params.getSceneTypeRaw())
                .in(CollectionUtil.isNotEmpty(params.getSceneTypeRaws()), "a.scene_type_raw", params.getSceneTypeRaws())
                .eq(StrUtil.isNotBlank(params.getSceneTypeStandard()), "a.scene_type_standard", params.getSceneTypeStandard())
                .in(CollectionUtil.isNotEmpty(params.getSceneTypeStandards()), "a.scene_type_standard", params.getSceneTypeStandards())
                .eq(null != params.getSortIdx(), "a.sort_idx", params.getSortIdx())
                .in(CollectionUtil.isNotEmpty(params.getSortIdxs()), "a.sort_idx", params.getSortIdxs())
                .ge(null != params.getStartDate(), "a.diagnosis_time", params.getStartDate())
                .lt(null != params.getEndDate(), "a.diagnosis_time", params.getEndDate())
        );

        if (null == page || CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        PageData<DwdDiagnosisReportVO> data = new PageConverter<DwdDiagnosisReportInfo, DwdDiagnosisReportVO>() {
            @Override
            public DwdDiagnosisReportVO converter(DwdDiagnosisReportInfo entity) {
                DwdDiagnosisReportVO vo = PatientHealthyHelper.getDwdDiagnosisReportVO(entity);
                return vo;
            }
        }.getPageData(page);
        return data;
    }

}
