package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdDischargeContentInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdDischargeContentInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdDischargeContentQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdDischargeContentVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdDischargeContentBizService;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * DWD出院小结信息表 服务类
 * </p>
 */
@DS("wd")
@Service
public class DwdDischargeContentBizServiceImpl implements DwdDischargeContentBizService {

    @Autowired
    private DwdDischargeContentInfoService dwdDischargeContentInfoService;

    @Override
    public PageData<DwdDischargeContentVO> query(DwdDischargeContentQueryDTO params) {
        IPage<DwdDischargeContentInfo> page =  dwdDischargeContentInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DwdDischargeContentInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
        );

        PageData<DwdDischargeContentVO> data = new PageConverter<DwdDischargeContentInfo, DwdDischargeContentVO>() {
            @Override
            public DwdDischargeContentVO converter(DwdDischargeContentInfo entity) {
                DwdDischargeContentVO vo = PatientHealthyHelper.getDwdDischargeContentVO(entity);
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<DwdDischargeContentVO> list(DwdDischargeContentQueryDTO params) {
        List<DwdDischargeContentInfo> data =  dwdDischargeContentInfoService.list(new QueryWrapper<DwdDischargeContentInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .last(" limit 1000 ") // TODO 防止不传参数查询所有导致爆库
        );
        return data.stream().map(PatientHealthyHelper::getDwdDischargeContentVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdDischargeContentVO> listByPatientId(String patientId) {
        List<DwdDischargeContentInfo> data =  dwdDischargeContentInfoService.list(new QueryWrapper<DwdDischargeContentInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdDischargeContentVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdDischargeContentVO> listByPatientId(List<String> patientIds) {
        List<DwdDischargeContentInfo> data =  dwdDischargeContentInfoService.list(new QueryWrapper<DwdDischargeContentInfo>()
                .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdDischargeContentVO).collect(Collectors.toList());
    }

    @Override
    public DwdDischargeContentVO getById(String id) {
        DwdDischargeContentInfo entity = dwdDischargeContentInfoService.getById(id);
        return PatientHealthyHelper.getDwdDischargeContentVO(entity);
    }

    @Override
    public DwdDischargeContentVO getOne(DwdDischargeContentQueryDTO params) {
        if (StrUtil.isBlank(params.getId())) {
            return null;
        }
        DwdDischargeContentInfo entity = dwdDischargeContentInfoService.getOne(new QueryWrapper<DwdDischargeContentInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .eq("id", params.getId())
        );
        if (null == entity) {
            return null;
        }
        return PatientHealthyHelper.getDwdDischargeContentVO(entity);
    }

    @Override
    public List<DwdDischargeContentInfo> listBeanByPatient(String patientId) {
        List<DwdDischargeContentInfo> data =  dwdDischargeContentInfoService.list(
                new QueryWrapper<DwdDischargeContentInfo>()
                        .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdDischargeContentInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdDischargeContentInfo> data =  dwdDischargeContentInfoService.list(
                new QueryWrapper<DwdDischargeContentInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }

}
