package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdDischargeRecordInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdDischargeRecordInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdDischargeContentQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdDischargeRecordQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdDischargeContentVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdDischargeRecordVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdDischargeContentBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdDischargeRecordBizService;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * DWD出院记录信息表 服务类
 * </p>
 */
@DS("wd")
@Service
public class DwdDischargeRecordBizServiceImpl implements DwdDischargeRecordBizService {

    @Autowired
    private DwdDischargeRecordInfoService dwdDischargeRecordInfoService;
    @Autowired
    private DwdDischargeContentBizService dwdDischargeContentBizService;

    @Override
    public PageData<DwdDischargeRecordVO> query(DwdDischargeRecordQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }
        
        IPage<DwdDischargeRecordInfo> page =  dwdDischargeRecordInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DwdDischargeRecordInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .ge(null != params.getStartDate(), "leave_date", params.getStartDate())
                .lt(null != params.getEndDate(), "leave_date", params.getEndDate())
                .orderByDesc("leave_date")
        );

        if (null == page || CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }
        
        // 出院小结
        List<String> ids = page.getRecords().stream().map(i -> i.getId()).collect(Collectors.toList());
        Map<String, List<DwdDischargeContentVO>> dischargeContentVOMap = this.getDwdDischargeContentVOMap(ids);

        PageData<DwdDischargeRecordVO> data = new PageConverter<DwdDischargeRecordInfo, DwdDischargeRecordVO>() {
            @Override
            public DwdDischargeRecordVO converter(DwdDischargeRecordInfo entity) {
                DwdDischargeRecordVO vo = PatientHealthyHelper.getDwdDischargeRecordVO(entity);
                vo.setDwdDischargeContentVOList(dischargeContentVOMap.get(vo.getVisitId()));
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    // 检验报告单明细
    private Map<String, List<DwdDischargeContentVO>> getDwdDischargeContentVOMap(List<String> ids) {
        DwdDischargeContentQueryDTO queryDTO = new DwdDischargeContentQueryDTO();
        queryDTO.setIds(ids);
        List<DwdDischargeContentVO> voList = dwdDischargeContentBizService.list(queryDTO);

        Map<String, List<DwdDischargeContentVO>> voMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(voList)) {
            voList.forEach(i -> {
                List<DwdDischargeContentVO> vos = voMap.get(i.getId());
                if (null == vos) {
                    vos = new ArrayList<>();
                    voMap.put(i.getId(), vos);
                }
                vos.add(i);
            });
        }
        return voMap;
    }

    @Override
    public List<DwdDischargeRecordVO> list(DwdDischargeRecordQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }
        
        List<DwdDischargeRecordInfo> data =  dwdDischargeRecordInfoService.list(new QueryWrapper<DwdDischargeRecordInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .ge(null != params.getStartDate(), "leave_date", params.getStartDate())
                .lt(null != params.getEndDate(), "leave_date", params.getEndDate())
                .orderByDesc("leave_date")
                .last("limit 1000")
        );

        return data.stream().map(PatientHealthyHelper::getDwdDischargeRecordVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdDischargeRecordVO> listByPatientId(String patientId) {
        List<DwdDischargeRecordInfo> data =  dwdDischargeRecordInfoService.list(new QueryWrapper<DwdDischargeRecordInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdDischargeRecordVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdDischargeRecordVO> listByPatientId(List<String> patientIds) {
        List<DwdDischargeRecordInfo> data =  dwdDischargeRecordInfoService.list(new QueryWrapper<DwdDischargeRecordInfo>()
                .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdDischargeRecordVO).collect(Collectors.toList());
    }

    @Override
    public DwdDischargeRecordVO getById(String id) {
        DwdDischargeRecordInfo entity = dwdDischargeRecordInfoService.getById(id);
        return PatientHealthyHelper.getDwdDischargeRecordVO(entity);
    }

    @Override
    public List<DwdDischargeRecordInfo> listBeanByPatient(String patientId) {
        List<DwdDischargeRecordInfo> data =  dwdDischargeRecordInfoService.list(
                new QueryWrapper<DwdDischargeRecordInfo>()
                        .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdDischargeRecordInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdDischargeRecordInfo> data =  dwdDischargeRecordInfoService.list(
                new QueryWrapper<DwdDischargeRecordInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }

}
