package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.client.api.constant.DwdConstant;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdExaminationPatientInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdExaminationPatientInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdExamineReportQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdLaboratoryReportQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.OdsExaminationFinalReportQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdExaminationPatientQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.OdsExaminationRegisterQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.OdsExaminationReportResultQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdExamineReportVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdLaboratoryReportVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.OdsExaminationFinalReportVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdExaminationPatientVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.OdsExaminationRegisterVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.OdsExaminationReportResultVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdExamineReportBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdLaboratoryReportBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.OdsExaminationFinalReportBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdExaminationPatientBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.OdsExaminationRegisterBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.OdsExaminationReportResultBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@DS("wd")
@Slf4j
@Service
public class DwdExaminationPatientBizServiceImpl implements DwdExaminationPatientBizService {

    @Autowired
    private DwdExaminationPatientInfoService dwdExaminationPatientInfoService;
    @Autowired
    private OdsExaminationRegisterBizService odsExaminationRegisterBizService;
    @Autowired
    private OdsExaminationReportResultBizService odsExaminationReportResultBizService;
    @Autowired
    private OdsExaminationFinalReportBizService odsExaminationFinalReportBizService;
    @Autowired
    private DwdExamineReportBizService dwdExamineReportBizService;
    @Autowired
    private DwdLaboratoryReportBizService dwdLaboratoryReportBizService;
    
    @Override
    public List<DwdExaminationPatientVO> list(DwdExaminationPatientQueryDTO params) {
        log.info("OdsExaminationPatientBizServiceImpl.list::params >>>>>> {}", JSON.toJSONString(params));
        
        // 查询体检患者信息
        List<DwdExaminationPatientInfo> data =  dwdExaminationPatientInfoService.list(new QueryWrapper<DwdExaminationPatientInfo>()
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .eq(StrUtil.isNotBlank(params.getMainPersonId()), "main_person_id", params.getMainPersonId())
                .in(CollectionUtil.isNotEmpty(params.getMainPersonIds()), "main_person_id", params.getMainPersonIds())
                .like(StrUtil.isNotBlank(params.getPatientName()), "patient_name", params.getPatientName())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "data_create_date", params.getStartDate())
                .le(StrUtil.isNotBlank(params.getEndDate()), "data_create_date", params.getEndDate())
                .orderByDesc("data_create_date")
        );

        List<DwdExaminationPatientVO> list = data.stream().map(PatientHealthyHelper::getOdsExaminationPatientVO).collect(Collectors.toList());
        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        List<String> visitIds = new ArrayList<>();
        list.forEach(i -> {
            visitIds.add(i.getVisitId());
        });
        
        // 查询体检登记信息
        Map<String, OdsExaminationRegisterVO> odsExaminationRegisterVOMap = this.getOdsExaminationRegisterVOMap(visitIds, params.getDataType());
        // 查询体检报告信息
        Map<String, OdsExaminationReportResultVO> odsExaminationReportResultVOMap = this.getOdsExaminationReportResultVOMap(visitIds, params.getDataType());
        // 查询体检最终报告信息
        Map<String, OdsExaminationFinalReportVO> odsExaminationFinalReportVOMap = this.getOdsExaminationFinalReportVOMap(visitIds, params.getDataType());
        // 检查报告单
        Map<String, List<DwdExamineReportVO>> dwdExamineReportVOMap = this.getDwdExamineReportVOMap(visitIds, params.getDataType());
        // 检验报告单
        Map<String, List<DwdLaboratoryReportVO>> dwdLaboratoryReportVOMap = this.getDwdLaboratoryReportVOMap(visitIds, params.getDataType());
        
        list.forEach(i -> {
            i.setOdsExaminationRegisterVO(odsExaminationRegisterVOMap.get(i.getVisitId()));
            i.setOdsExaminationReportResultVO(odsExaminationReportResultVOMap.get(i.getVisitId()));
            i.setOdsExaminationFinalReportVO(odsExaminationFinalReportVOMap.get(i.getVisitId()));

            i.setDwdExamineReportVOs(dwdExamineReportVOMap.get(i.getVisitId()));
            i.setDwdLaboratoryReportVOs(dwdLaboratoryReportVOMap.get(i.getVisitId()));
        });

        return list;
    }

    // 体检登记
    private Map<String, OdsExaminationRegisterVO> getOdsExaminationRegisterVOMap(List<String> visitIds, String dataType) {
        if (StrUtil.isNotBlank(dataType)) {
            return new HashMap<>(0);
        }

        OdsExaminationRegisterQueryDTO queryDTO = new OdsExaminationRegisterQueryDTO();
        // queryDTO.setPatientId(patientId);
        queryDTO.setVisitIds(visitIds);
        List<OdsExaminationRegisterVO> voList = odsExaminationRegisterBizService.list(queryDTO);

        Map<String, OdsExaminationRegisterVO> voMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(voList)) {
            voList.forEach(i -> voMap.put(i.getVisitId(), i));
        }
        return voMap;
    }

    // 体检报告
    private Map<String, OdsExaminationReportResultVO> getOdsExaminationReportResultVOMap(List<String> visitIds, String dataType) {
        if (StrUtil.isNotBlank(dataType)) {
            return new HashMap<>(0);
        }

        OdsExaminationReportResultQueryDTO queryDTO = new OdsExaminationReportResultQueryDTO();
        // queryDTO.setPatientId(patientId);
        queryDTO.setVisitIds(visitIds);
        List<OdsExaminationReportResultVO> voList = odsExaminationReportResultBizService.list(queryDTO);

        Map<String, OdsExaminationReportResultVO> voMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(voList)) {
            voList.forEach(i -> voMap.put(i.getVisitId(), i));
        }
        return voMap;
    }

    // 体检结论
    private Map<String, OdsExaminationFinalReportVO> getOdsExaminationFinalReportVOMap(List<String> visitIds, String dataType) {
        if (StrUtil.isNotBlank(dataType)) {
            return new HashMap<>(0);
        }

        OdsExaminationFinalReportQueryDTO queryDTO = new OdsExaminationFinalReportQueryDTO();
        // queryDTO.setPatientId(patientId);
        queryDTO.setVisitIds(visitIds);
        List<OdsExaminationFinalReportVO> voList = odsExaminationFinalReportBizService.list(queryDTO);

        Map<String, OdsExaminationFinalReportVO> voMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(voList)) {
            voList.forEach(i -> voMap.put(i.getVisitId(), i));
        }
        return voMap;
    }

    // 检查报告单
    private Map<String, List<DwdExamineReportVO>> getDwdExamineReportVOMap(List<String> patientIds, String dataType) {
        if (StrUtil.isNotBlank(dataType) && !DwdConstant.DataTypeEnum.ExamineReport.getCode().equals(dataType)) {
            return new HashMap<>(0);
        }

        DwdExamineReportQueryDTO queryDTO = new DwdExamineReportQueryDTO();
        queryDTO.setPatientIds(patientIds);
        List<DwdExamineReportVO> voList = dwdExamineReportBizService.list(queryDTO);

        Map<String, List<DwdExamineReportVO>> voMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(voList)) {
            voList.forEach(i -> {
                List<DwdExamineReportVO> vos = voMap.get(i.getPatientId());
                if (null == vos) {
                    vos = new ArrayList<>();
                    voMap.put(i.getPatientId(), vos);
                }
                vos.add(i);
            });
        }
        return voMap;
    }

    // 检验报告单
    private Map<String, List<DwdLaboratoryReportVO>> getDwdLaboratoryReportVOMap(List<String> patientIds, String dataType) {
        if (StrUtil.isNotBlank(dataType) && !DwdConstant.DataTypeEnum.LaboratoryReport.getCode().equals(dataType)) {
            return new HashMap<>(0);
        }

        DwdLaboratoryReportQueryDTO queryDTO = new DwdLaboratoryReportQueryDTO();
        queryDTO.setPatientIds(patientIds);
        List<DwdLaboratoryReportVO> voList = dwdLaboratoryReportBizService.list(queryDTO);

        Map<String, List<DwdLaboratoryReportVO>> voMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(voList)) {
            voList.forEach(i -> {
                List<DwdLaboratoryReportVO> vos = voMap.get(i.getPatientId());
                if (null == vos) {
                    vos = new ArrayList<>();
                    voMap.put(i.getPatientId(), vos);
                }
                vos.add(i);
            });
        }
        return voMap;
    }

}
