package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdExamineApplyInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdExamineApplyInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdExamineApplyItemQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdExamineApplyQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdExamineApplyItemVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdExamineApplyVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdExamineApplyBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdExamineApplyItemBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * DWD检查申请单数据表 服务类
 * </p>
 */
@DS("wd")
@Service
public class DwdExamineApplyBizServiceImpl implements DwdExamineApplyBizService {

    @Autowired
    private DwdExamineApplyInfoService dwdExamineApplyInfoService;
    @Autowired
    private DwdExamineApplyItemBizService dwdExamineApplyItemBizService;

    @Override
    public List<DwdExamineApplyVO> list(DwdExamineApplyQueryDTO params) {
        List<DwdExamineApplyInfo> data = dwdExamineApplyInfoService.list(new QueryWrapper<DwdExamineApplyInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
        );
        List<DwdExamineApplyVO> list = data.stream().map(PatientHealthyHelper::getDwdExamineApplyVO).collect(Collectors.toList());

        // 检查申请单项目
        if (CollectionUtil.isNotEmpty(list)) {
            List<String> applyIds = list.stream().map(DwdExamineApplyVO::getId).collect(Collectors.toList());
            Map<String, List<DwdExamineApplyItemVO>> dwdExamineApplyItemVOMap = this.getDwdExamineApplyItemVOMap(params.getPatientId(), applyIds);
            if (CollectionUtil.isNotEmpty(dwdExamineApplyItemVOMap)) {
                list.forEach(i -> i.setDwdExamineApplyItemVOs(dwdExamineApplyItemVOMap.get(i.getId())));
            }
        }

        return list;
    }

    // 检查申请单项目
    private Map<String, List<DwdExamineApplyItemVO>> getDwdExamineApplyItemVOMap(String patientId, List<String> applyIds) {
        DwdExamineApplyItemQueryDTO queryDTO = new DwdExamineApplyItemQueryDTO();
        queryDTO.setPatientId(patientId);
        queryDTO.setApplyIds(applyIds);
        List<DwdExamineApplyItemVO> voList = dwdExamineApplyItemBizService.list(queryDTO);

        Map<String, List<DwdExamineApplyItemVO>> voMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(voList)) {
            voList.forEach(i -> {
                List<DwdExamineApplyItemVO> vos = voMap.get(i.getApplyId());
                if (null == vos) {
                    vos = new ArrayList<>();
                    voMap.put(i.getApplyId(), vos);
                }
                vos.add(i);
            });
        }
        return voMap;
    }

    @Override
    public List<DwdExamineApplyVO> listByPatientId(String patientId) {
        List<DwdExamineApplyInfo> data = dwdExamineApplyInfoService.list(new QueryWrapper<DwdExamineApplyInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdExamineApplyVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdExamineApplyVO> listByPatientId(List<String> patientIds) {
        List<DwdExamineApplyInfo> data = dwdExamineApplyInfoService.list(
                new QueryWrapper<DwdExamineApplyInfo>()
                        .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdExamineApplyVO).collect(Collectors.toList());
    }

    @Override
    public DwdExamineApplyVO getById(String id) {
        DwdExamineApplyInfo entity = dwdExamineApplyInfoService.getById(id);
        return PatientHealthyHelper.getDwdExamineApplyVO(entity);
    }

    @Override
    public List<DwdExamineApplyInfo> listBeanByPatient(String patientId) {
        List<DwdExamineApplyInfo> data = dwdExamineApplyInfoService.list(
                new QueryWrapper<DwdExamineApplyInfo>()
                        .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdExamineApplyInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdExamineApplyInfo> data = dwdExamineApplyInfoService.list(
                new QueryWrapper<DwdExamineApplyInfo>()
                        .in("patient_id",patientIds)
        );
        return data;
    }

}
