package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdExamineReportInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdExamineReportInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdExamineReportGetDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdExamineReportQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdExamineReportVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdExamineReportBizService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;


/**
 * <p>
 * DWD检查报告单信息表 服务类
 * </p>
 */
@DS("wd")
@Service
public class DwdExamineReportBizServiceImpl implements DwdExamineReportBizService {

    @Autowired
    private DwdExamineReportInfoService dwdExamineReportInfoService;

    @Override
    public PageData<DwdExamineReportVO> query(DwdExamineReportQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }
        
        IPage<DwdExamineReportInfo> page = dwdExamineReportInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DwdExamineReportInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .ge(null != params.getStartDate(), "reports_time", params.getStartDate())
                .lt(null != params.getEndDate(), "reports_time", params.getEndDate())
                .eq(StrUtil.isNotBlank(params.getExamineTypeRaw()), "examine_type_raw", params.getExamineTypeRaw())
                .eq(StrUtil.isNotBlank(params.getExamineTypeStandard()), "examine_type_standard", params.getExamineTypeStandard())
                .like(StrUtil.isNotBlank(params.getReportNameRaw()), "report_name_raw", params.getReportNameRaw())
                .eq(StrUtil.isNotBlank(params.getReportNameStandard()), "report_name_standard", params.getReportNameStandard())
                .in(CollectionUtil.isNotEmpty(params.getReportNameStandards()), "report_name_standard", params.getReportNameStandards())
                .eq(StrUtil.isNotBlank(params.getActivityNo()), "activity_no", params.getActivityNo())
                .in(CollectionUtil.isNotEmpty(params.getActivityNos()), "activity_no", params.getActivityNos())
                .and(StrUtil.isNotBlank(params.getIsActivity()) && YN.YES.getCode().equals(params.getIsActivity()), i -> {
                    i.ne("activity_no", "").or().isNotNull("activity_no");
                })
                .orderByDesc("reports_time")
        );

        if (null == page || CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }
        
        PageData<DwdExamineReportVO> data = new PageConverter<DwdExamineReportInfo, DwdExamineReportVO>() {
            @Override
            public DwdExamineReportVO converter(DwdExamineReportInfo entity) {
                DwdExamineReportVO vo = PatientHealthyHelper.getDwdExamineReportVO(entity);
                return vo;
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public List<DwdExamineReportVO> list(DwdExamineReportQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }
        
        List<DwdExamineReportInfo> data = dwdExamineReportInfoService.list(new QueryWrapper<DwdExamineReportInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .ge(null != params.getStartDate(), "reports_time", params.getStartDate())
                .lt(null != params.getEndDate(), "reports_time", params.getEndDate())
                .eq(StrUtil.isNotBlank(params.getExamineTypeRaw()), "examine_type_raw", params.getExamineTypeRaw())
                .eq(StrUtil.isNotBlank(params.getExamineTypeStandard()), "examine_type_standard", params.getExamineTypeStandard())
                .like(StrUtil.isNotBlank(params.getReportNameRaw()), "report_name_raw", params.getReportNameRaw())
                .eq(StrUtil.isNotBlank(params.getReportNameStandard()), "report_name_standard", params.getReportNameStandard())
                .in(CollectionUtil.isNotEmpty(params.getReportNameStandards()), "report_name_standard", params.getReportNameStandards())
                .eq(StrUtil.isNotBlank(params.getActivityNo()), "activity_no", params.getActivityNo())
                .in(CollectionUtil.isNotEmpty(params.getActivityNos()), "activity_no", params.getActivityNos())
                .and(StrUtil.isNotBlank(params.getIsActivity()) && YN.YES.getCode().equals(params.getIsActivity()), i -> {
                    i.ne("activity_no", "").or().isNotNull("activity_no");
                })
                .orderByDesc("reports_time")
                .last(" limit 1000 ") // TODO 防止不传参数查询所有导致爆库
        );
        return data.stream().map(PatientHealthyHelper::getDwdExamineReportVO).collect(Collectors.toList());
    }

    @Override
    public DwdExamineReportVO detail(DwdExamineReportGetDTO params) {
        DwdExamineReportInfo entity = dwdExamineReportInfoService.getOne(new QueryWrapper<DwdExamineReportInfo>()
                .eq("id", params.getId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
        );
        if (null == entity) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return PatientHealthyHelper.getDwdExamineReportVO(entity);
    }

    @Override
    public List<DwdExamineReportVO> listByPatientId(String patientId) {
        List<DwdExamineReportInfo> data = dwdExamineReportInfoService.list(new QueryWrapper<DwdExamineReportInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdExamineReportVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdExamineReportVO> listByPatientIds(List<String> patientIds) {
        List<DwdExamineReportInfo> data = dwdExamineReportInfoService.list(new QueryWrapper<DwdExamineReportInfo>()
                .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdExamineReportVO).collect(Collectors.toList());
    }

    @Override
    public DwdExamineReportVO getById(String id) {
        DwdExamineReportInfo entity = dwdExamineReportInfoService.getById(id);
        return PatientHealthyHelper.getDwdExamineReportVO(entity);
    }

    @Override
    public List<DwdExamineReportInfo> listBeanByPatient(String patientId) {
        List<DwdExamineReportInfo> data = dwdExamineReportInfoService.list(new QueryWrapper<DwdExamineReportInfo>()
                .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdExamineReportInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdExamineReportInfo> data = dwdExamineReportInfoService.list(new QueryWrapper<DwdExamineReportInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }

}
