package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdExamineReserveInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdExamineReserveInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdExamineReserveQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdExamineReserveVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdExamineReserveBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;


/**
 * <p>
 * DWD检查预约记录信息
 * </p>
 */
@DS("wd")
@Service
public class DwdExamineReserveBizServiceImpl implements DwdExamineReserveBizService {

    @Autowired
    private DwdExamineReserveInfoService dwdExamineReserveInfoService;

    @Override
    public List<DwdExamineReserveVO> list(DwdExamineReserveQueryDTO params) {
        List<DwdExamineReserveInfo> data = dwdExamineReserveInfoService.list(new QueryWrapper<DwdExamineReserveInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
        );
        return data.stream().map(PatientHealthyHelper::getDwdExamineReserveVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdExamineReserveVO> listByPatientId(String patientId) {
        List<DwdExamineReserveInfo> data = dwdExamineReserveInfoService.list(new QueryWrapper<DwdExamineReserveInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdExamineReserveVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdExamineReserveVO> listByPatientId(List<String> patientIds) {
        List<DwdExamineReserveInfo> data = dwdExamineReserveInfoService.list(new QueryWrapper<DwdExamineReserveInfo>()
                .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdExamineReserveVO).collect(Collectors.toList());
    }

    @Override
    public DwdExamineReserveVO getById(String id) {
        DwdExamineReserveInfo entity = dwdExamineReserveInfoService.getById(id);
        return PatientHealthyHelper.getDwdExamineReserveVO(entity);
    }

    @Override
    public List<DwdExamineReserveInfo> listBeanByPatient(String patientId) {
        List<DwdExamineReserveInfo> data = dwdExamineReserveInfoService.list(new QueryWrapper<DwdExamineReserveInfo>()
                .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdExamineReserveInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdExamineReserveInfo> data = dwdExamineReserveInfoService.list(new QueryWrapper<DwdExamineReserveInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }

}
