package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdInpatientChangeInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdInpatientChangeInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdInpatientChangeQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdInpatientChangeVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdInpatientChangeBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * DWD预约住院信息记录信息表
 * </p>
 */
@DS("wd")
@Service
public class DwdInpatientChangeBizServiceImpl implements DwdInpatientChangeBizService {

    @Autowired
    private DwdInpatientChangeInfoService dwdInpatientChangeInfoService;

    @Override
    public List<DwdInpatientChangeVO> list(DwdInpatientChangeQueryDTO params) {
        List<DwdInpatientChangeInfo> data =  dwdInpatientChangeInfoService.list(new QueryWrapper<DwdInpatientChangeInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getInpatientId()), "inpatient_id", params.getInpatientId())
                .in(CollectionUtil.isNotEmpty(params.getInpatientIds()), "inpatient_id", params.getInpatientIds())
                .in(CollectionUtil.isNotEmpty(params.getChangeTypes()), "change_type", params.getChangeTypes())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "change_time", params.getStartDate())
                .le(StrUtil.isNotBlank(params.getEndDate()), "change_time", params.getEndDate())
                .last(" limit 1000 ")
        );
        return data.stream().map(PatientHealthyHelper::getDwdInpatientChangeVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdInpatientChangeVO> listByPatientId(String patientId) {
        List<DwdInpatientChangeInfo> data =  dwdInpatientChangeInfoService.list(new QueryWrapper<DwdInpatientChangeInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdInpatientChangeVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdInpatientChangeVO> listByPatientId(List<String> patientIds) {
        List<DwdInpatientChangeInfo> data =  dwdInpatientChangeInfoService.list(new QueryWrapper<DwdInpatientChangeInfo>()
                .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdInpatientChangeVO).collect(Collectors.toList());
    }

    @Override
    public DwdInpatientChangeVO getById(String id) {
        DwdInpatientChangeInfo entity = dwdInpatientChangeInfoService.getById(id);
        return PatientHealthyHelper.getDwdInpatientChangeVO(entity);
    }

    @Override
    public List<DwdInpatientChangeInfo> listBeanByPatient(String patientId) {
        List<DwdInpatientChangeInfo> data =  dwdInpatientChangeInfoService.list(
                new QueryWrapper<DwdInpatientChangeInfo>()
                        .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdInpatientChangeInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdInpatientChangeInfo> data =  dwdInpatientChangeInfoService.list(
                new QueryWrapper<DwdInpatientChangeInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }

    @Override
    public List<DwdInpatientChangeInfo> listBeanByPatient(String patientId, List<String> changeTypes) {
        List<DwdInpatientChangeInfo> data =  dwdInpatientChangeInfoService.list(
                new QueryWrapper<DwdInpatientChangeInfo>()
                        .eq("patient_id",patientId)
                        .in(CollectionUtil.isNotEmpty(changeTypes), "change_type", changeTypes)
        );
        return data;
    }

    @Override
    public List<DwdInpatientChangeInfo> listBeanByPatient(List<String> patientIds, List<String> changeTypes) {
        List<DwdInpatientChangeInfo> data =  dwdInpatientChangeInfoService.list(
                new QueryWrapper<DwdInpatientChangeInfo>()
                        .in("patient_id",patientIds)
                        .in(CollectionUtil.isNotEmpty(changeTypes), "change_type", changeTypes)
        );
        return data;
    }

}
