package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdInpatientReserveInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdInpatientReserveInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdInpatientReserveQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdInpatientReserveVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdInpatientReserveBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * DWD预约住院信息记录信息表
 * </p>
 */
@DS("wd")
@Service
public class DwdInpatientReserveBizServiceImpl implements DwdInpatientReserveBizService {

    @Autowired
    private DwdInpatientReserveInfoService dwdInpatientReserveInfoService;

    @Override
    public List<DwdInpatientReserveVO> list(DwdInpatientReserveQueryDTO params) {
        List<DwdInpatientReserveInfo> data =  dwdInpatientReserveInfoService.list(new QueryWrapper<DwdInpatientReserveInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .eq(StrUtil.isNotBlank(params.getApplyDepartmentCodeStandard()), "apply_department_code_standard", params.getApplyDepartmentCodeStandard())
                .in(CollectionUtil.isNotEmpty(params.getApplyDepartmentCodeStandards()), "apply_department_code_standard", params.getApplyDepartmentCodeStandards())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "apply_date", params.getStartDate())
                .le(StrUtil.isNotBlank(params.getEndDate()), "apply_date", params.getEndDate())
                .last(" limit 1000 ")
        );
        return data.stream().map(PatientHealthyHelper::getDwdInpatientReserveVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdInpatientReserveVO> listByPatientId(String patientId) {
        List<DwdInpatientReserveInfo> data =  dwdInpatientReserveInfoService.list(new QueryWrapper<DwdInpatientReserveInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdInpatientReserveVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdInpatientReserveVO> listByPatientId(List<String> patientIds) {
        List<DwdInpatientReserveInfo> data =  dwdInpatientReserveInfoService.list(new QueryWrapper<DwdInpatientReserveInfo>()
                .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdInpatientReserveVO).collect(Collectors.toList());
    }

    @Override
    public DwdInpatientReserveVO getById(String id) {
        DwdInpatientReserveInfo entity = dwdInpatientReserveInfoService.getById(id);
        return PatientHealthyHelper.getDwdInpatientReserveVO(entity);
    }

    @Override
    public List<DwdInpatientReserveInfo> listBeanByPatient(String patientId) {
        List<DwdInpatientReserveInfo> data =  dwdInpatientReserveInfoService.list(
                new QueryWrapper<DwdInpatientReserveInfo>()
                        .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdInpatientReserveInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdInpatientReserveInfo> data =  dwdInpatientReserveInfoService.list(
                new QueryWrapper<DwdInpatientReserveInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }

}
