package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdLaboratoryApplyInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdLaboratoryApplyInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdLaboratoryApplyGetDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdLaboratoryApplyProjectQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdLaboratoryApplyQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdLaboratoryApplyProjectVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdLaboratoryApplyVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdLaboratoryApplyBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdLaboratoryApplyProjectBizService;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * DWD检验申请单数据
 * </p>
 */
@DS("wd")
@Service
public class DwdLaboratoryApplyBizServiceImpl implements DwdLaboratoryApplyBizService {

    @Autowired
    private DwdLaboratoryApplyInfoService dwdLaboratoryApplyInfoService;
    @Autowired
    private DwdLaboratoryApplyProjectBizService dwdLaboratoryApplyProjectBizService;

    @Override
    public List<DwdLaboratoryApplyVO> list(DwdLaboratoryApplyQueryDTO params) {
        List<DwdLaboratoryApplyInfo> data = dwdLaboratoryApplyInfoService.list(new QueryWrapper<DwdLaboratoryApplyInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
        );
        List<DwdLaboratoryApplyVO> list = data.stream().map(PatientHealthyHelper::getDwdLaboratoryApplyVO).collect(Collectors.toList());

        // 检验申请单项目
        if (CollectionUtil.isNotEmpty(list)) {
            List<String> applyIds = list.stream().map(DwdLaboratoryApplyVO::getId).collect(Collectors.toList());
            Map<String, List<DwdLaboratoryApplyProjectVO>> dwdLaboratoryApplyProjectVOMap = this.getDwdLaboratoryApplyProjectVOMap(params.getPatientId(), applyIds);
            if (CollectionUtil.isNotEmpty(dwdLaboratoryApplyProjectVOMap)) {
                list.forEach(i -> i.setDwdLaboratoryApplyProjectVOs(dwdLaboratoryApplyProjectVOMap.get(i.getId())));
            }
        }

        return list;
    }

    // 检验申请单项目
    private Map<String, List<DwdLaboratoryApplyProjectVO>> getDwdLaboratoryApplyProjectVOMap(String patientId, List<String> applyIds) {
        DwdLaboratoryApplyProjectQueryDTO queryDTO = new DwdLaboratoryApplyProjectQueryDTO();
        queryDTO.setPatientId(patientId);
        queryDTO.setLaboratoryApplyIds(applyIds);
        List<DwdLaboratoryApplyProjectVO> voList = dwdLaboratoryApplyProjectBizService.list(queryDTO);

        Map<String, List<DwdLaboratoryApplyProjectVO>> voMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(voList)) {
            voList.forEach(i -> {
                List<DwdLaboratoryApplyProjectVO> vos = voMap.get(i.getLaboratoryApplyId());
                if (null == vos) {
                    vos = new ArrayList<>();
                    voMap.put(i.getLaboratoryApplyId(), vos);
                }
                vos.add(i);
            });
        }
        return voMap;
    }

    @Override
    public DwdLaboratoryApplyVO detail(DwdLaboratoryApplyGetDTO params) {
        DwdLaboratoryApplyInfo entity = dwdLaboratoryApplyInfoService.getOne(new QueryWrapper<DwdLaboratoryApplyInfo>()
                .eq("id", params.getId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
        );
        if (null == entity) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        DwdLaboratoryApplyVO vo = PatientHealthyHelper.getDwdLaboratoryApplyVO(entity);
        
        DwdLaboratoryApplyProjectQueryDTO dwdLaboratoryApplyProjectQueryDTO = new DwdLaboratoryApplyProjectQueryDTO();
        dwdLaboratoryApplyProjectQueryDTO.setLaboratoryApplyId(params.getId());
        List<DwdLaboratoryApplyProjectVO> dwdLaboratoryApplyProjectVOList = dwdLaboratoryApplyProjectBizService.list(dwdLaboratoryApplyProjectQueryDTO);
        vo.setDwdLaboratoryApplyProjectVOs(dwdLaboratoryApplyProjectVOList);

        return vo;
    }

    @Override
    public List<DwdLaboratoryApplyVO> listByPatientId(String patientId) {
        List<DwdLaboratoryApplyInfo> data = dwdLaboratoryApplyInfoService.list(new QueryWrapper<DwdLaboratoryApplyInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdLaboratoryApplyVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdLaboratoryApplyVO> listByPatientId(List<String> patientIds) {
        List<DwdLaboratoryApplyInfo> data = dwdLaboratoryApplyInfoService.list(new QueryWrapper<DwdLaboratoryApplyInfo>()
                .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdLaboratoryApplyVO).collect(Collectors.toList());
    }

    @Override
    public DwdLaboratoryApplyVO getById(String id) {
        DwdLaboratoryApplyInfo entity = dwdLaboratoryApplyInfoService.getById(id);
        if (null == entity) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        DwdLaboratoryApplyVO vo = PatientHealthyHelper.getDwdLaboratoryApplyVO(entity);
        
        DwdLaboratoryApplyProjectQueryDTO dwdLaboratoryApplyProjectQueryDTO = new DwdLaboratoryApplyProjectQueryDTO();
        dwdLaboratoryApplyProjectQueryDTO.setLaboratoryApplyId(id);
        List<DwdLaboratoryApplyProjectVO> dwdLaboratoryApplyProjectVOList = dwdLaboratoryApplyProjectBizService.list(dwdLaboratoryApplyProjectQueryDTO);
        vo.setDwdLaboratoryApplyProjectVOs(dwdLaboratoryApplyProjectVOList);

        return vo;
    }

    @Override
    public List<DwdLaboratoryApplyInfo> listBeanByPatient(String patientId) {
        List<DwdLaboratoryApplyInfo> data = dwdLaboratoryApplyInfoService.list(new QueryWrapper<DwdLaboratoryApplyInfo>()
                .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdLaboratoryApplyInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdLaboratoryApplyInfo> data = dwdLaboratoryApplyInfoService.list(new QueryWrapper<DwdLaboratoryApplyInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }



}
