package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdLaboratoryApplyProjectInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdLaboratoryApplyProjectInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdLaboratoryApplyProjectQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdLaboratoryApplyProjectVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdLaboratoryApplyProjectBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * DWD检验申请单项目数据 服务类
 * </p>
 */
@DS("wd")
@Service
public class DwdLaboratoryApplyProjectBizServiceImpl implements DwdLaboratoryApplyProjectBizService {

    @Autowired
    private DwdLaboratoryApplyProjectInfoService dwdLaboratoryApplyProjectInfoService;

    @Override
    public List<DwdLaboratoryApplyProjectVO> list(DwdLaboratoryApplyProjectQueryDTO params) {
        List<DwdLaboratoryApplyProjectInfo> data = dwdLaboratoryApplyProjectInfoService.list(new QueryWrapper<DwdLaboratoryApplyProjectInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .eq(StrUtil.isNotBlank(params.getLaboratoryApplyId()), "laboratory_apply_id", params.getLaboratoryApplyId())
                .in(CollectionUtil.isNotEmpty(params.getLaboratoryApplyIds()), "laboratory_apply_id", params.getLaboratoryApplyIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
        );
        return data.stream().map(PatientHealthyHelper::getDwdLaboratoryApplyProjectVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdLaboratoryApplyProjectVO> listByPatientId(String patientId) {
        List<DwdLaboratoryApplyProjectInfo> data = dwdLaboratoryApplyProjectInfoService.list(new QueryWrapper<DwdLaboratoryApplyProjectInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdLaboratoryApplyProjectVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdLaboratoryApplyProjectVO> listByPatientId(List<String> patientIds) {
        List<DwdLaboratoryApplyProjectInfo> data = dwdLaboratoryApplyProjectInfoService.list(new QueryWrapper<DwdLaboratoryApplyProjectInfo>()
                .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdLaboratoryApplyProjectVO).collect(Collectors.toList());
    }

    @Override
    public DwdLaboratoryApplyProjectVO getById(String id) {
        DwdLaboratoryApplyProjectInfo entity = dwdLaboratoryApplyProjectInfoService.getById(id);
        return PatientHealthyHelper.getDwdLaboratoryApplyProjectVO(entity);
    }

    @Override
    public List<DwdLaboratoryApplyProjectInfo> listBeanByPatient(String patientId) {
        List<DwdLaboratoryApplyProjectInfo> data = dwdLaboratoryApplyProjectInfoService.list(new QueryWrapper<DwdLaboratoryApplyProjectInfo>()
                .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdLaboratoryApplyProjectInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdLaboratoryApplyProjectInfo> data = dwdLaboratoryApplyProjectInfoService.list(new QueryWrapper<DwdLaboratoryApplyProjectInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }

}
