package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdLaboratoryReportInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdLaboratoryReportInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdLaboratoryReportGetDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdLaboratoryReportItemQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdLaboratoryReportQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdLaboratoryReportItemVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdLaboratoryReportVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdLaboratoryReportBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdLaboratoryReportItemBizService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@DS("wd")
@Service
public class DwdLaboratoryReportBizServiceImpl implements DwdLaboratoryReportBizService {

    @Autowired
    private DwdLaboratoryReportInfoService dwdLaboratoryReportInfoService;
    @Autowired
    private DwdLaboratoryReportItemBizService dwdLaboratoryReportItemBizService;

    @Override
    public PageData<DwdLaboratoryReportVO> query(DwdLaboratoryReportQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        IPage<DwdLaboratoryReportInfo> page = dwdLaboratoryReportInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DwdLaboratoryReportInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .ge(null != params.getStartDate(), "reports_time", params.getStartDate())
                .lt(null != params.getEndDate(), "reports_time", params.getEndDate())
                .like(null != params.getReportName(), "report_name", params.getReportName())
                .eq(null != params.getSampleTypeRaw(), "sample_type_raw", params.getSampleTypeRaw())
                .eq(null != params.getSampleTypeStandard(), "sample_type_standard", params.getSampleTypeStandard())
                .eq(StrUtil.isNotBlank(params.getActivityNo()), "activity_no", params.getActivityNo())
                .in(CollectionUtil.isNotEmpty(params.getActivityNos()), "activity_no", params.getActivityNos())
                .and(StrUtil.isNotBlank(params.getIsActivity()) && YN.YES.getCode().equals(params.getIsActivity()), i -> {
                    i.ne("activity_no", "").or().isNotNull("activity_no");
                })
                .orderByDesc("reports_time")
        );

        if (null == page || CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }
        
        // 检验报告单明细
        List<String> reportIds = page.getRecords().stream().map(DwdLaboratoryReportInfo::getId).collect(Collectors.toList());
        Map<String, List<DwdLaboratoryReportItemVO>> dwdLaboratoryReportItemVOMap = this.getDwdLaboratoryReportItemVOMap(params.getPatientId(), reportIds);

        PageData<DwdLaboratoryReportVO> data = new PageConverter<DwdLaboratoryReportInfo, DwdLaboratoryReportVO>() {
            @Override
            public DwdLaboratoryReportVO converter(DwdLaboratoryReportInfo entity) {
                DwdLaboratoryReportVO vo = PatientHealthyHelper.getDwdLaboratoryReportVO(entity);
                vo.setDwdLaboratoryReportItemVOs(dwdLaboratoryReportItemVOMap.get(vo.getId()));
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<DwdLaboratoryReportVO> list(DwdLaboratoryReportQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<DwdLaboratoryReportInfo> data = dwdLaboratoryReportInfoService.list(new QueryWrapper<DwdLaboratoryReportInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .ge(null != params.getStartDate(), "reports_time", params.getStartDate())
                .lt(null != params.getEndDate(), "reports_time", params.getEndDate())
                .like(null != params.getReportName(), "report_name", params.getReportName())
                .eq(null != params.getSampleTypeRaw(), "sample_type_raw", params.getSampleTypeRaw())
                .eq(null != params.getSampleTypeStandard(), "sample_type_standard", params.getSampleTypeStandard())
                .eq(StrUtil.isNotBlank(params.getActivityNo()), "activity_no", params.getActivityNo())
                .in(CollectionUtil.isNotEmpty(params.getActivityNos()), "activity_no", params.getActivityNos())
                .and(StrUtil.isNotBlank(params.getIsActivity()) && YN.YES.getCode().equals(params.getIsActivity()), i -> {
                    i.ne("activity_no", "").or().isNotNull("activity_no");
                })
                .orderByDesc("reports_time")
                .last(" limit 1000 ") // TODO 防止不传参数查询所有导致爆库
        );

        List<DwdLaboratoryReportVO> list = data.stream().map(PatientHealthyHelper::getDwdLaboratoryReportVO).collect(Collectors.toList());

        // 检验报告单明细
        if (CollectionUtil.isNotEmpty(list)) {
            List<String> reportIds = list.stream().map(DwdLaboratoryReportVO::getId).collect(Collectors.toList());
            Map<String, List<DwdLaboratoryReportItemVO>> dwdLaboratoryReportItemVOMap = this.getDwdLaboratoryReportItemVOMap(params.getPatientId(), reportIds);
            if (CollectionUtil.isNotEmpty(dwdLaboratoryReportItemVOMap)) {
                list.forEach(i -> i.setDwdLaboratoryReportItemVOs(dwdLaboratoryReportItemVOMap.get(i.getId())));
            }
        }

        return list;
    }

    // 检验报告单明细
    private Map<String, List<DwdLaboratoryReportItemVO>> getDwdLaboratoryReportItemVOMap(String patientId, List<String> reportIds) {
        DwdLaboratoryReportItemQueryDTO queryDTO = new DwdLaboratoryReportItemQueryDTO();
        queryDTO.setPatientId(patientId);
        queryDTO.setLaboratoryReportIds(reportIds);
        List<DwdLaboratoryReportItemVO> voList = dwdLaboratoryReportItemBizService.list(queryDTO);

        Map<String, List<DwdLaboratoryReportItemVO>> voMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(voList)) {
            voList.forEach(i -> {
                List<DwdLaboratoryReportItemVO> vos = voMap.get(i.getLaboratoryReportId());
                if (null == vos) {
                    vos = new ArrayList<>();
                    voMap.put(i.getLaboratoryReportId(), vos);
                }
                vos.add(i);
            });
        }
        return voMap;
    }

    @Override
    public DwdLaboratoryReportVO detail(DwdLaboratoryReportGetDTO params) {
        DwdLaboratoryReportInfo entity = dwdLaboratoryReportInfoService.getOne(new QueryWrapper<DwdLaboratoryReportInfo>()
                .eq("id", params.getId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
        );
        if (null == entity) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        DwdLaboratoryReportVO vo = PatientHealthyHelper.getDwdLaboratoryReportVO(entity);

        DwdLaboratoryReportItemQueryDTO dwdLaboratoryReportItemQueryDTO = new DwdLaboratoryReportItemQueryDTO();
        dwdLaboratoryReportItemQueryDTO.setLaboratoryReportId(entity.getId());
        List<DwdLaboratoryReportItemVO> dwdLaboratoryReportItemVOList = dwdLaboratoryReportItemBizService.list(dwdLaboratoryReportItemQueryDTO);
        vo.setDwdLaboratoryReportItemVOs(dwdLaboratoryReportItemVOList);

        return vo;
    }

    @Override
    public List<DwdLaboratoryReportVO> listByPatientId(String patientId) {
        List<DwdLaboratoryReportInfo> data = dwdLaboratoryReportInfoService.list(new QueryWrapper<DwdLaboratoryReportInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdLaboratoryReportVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdLaboratoryReportVO> listByPatientIds(List<String> patientIds) {
        List<DwdLaboratoryReportInfo> data = dwdLaboratoryReportInfoService.list(new QueryWrapper<DwdLaboratoryReportInfo>()
                .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdLaboratoryReportVO).collect(Collectors.toList());
    }

    @Override
    public DwdLaboratoryReportVO getById(String id) {
        DwdLaboratoryReportInfo entity = dwdLaboratoryReportInfoService.getById(id);
        if (null == entity) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        DwdLaboratoryReportVO vo = PatientHealthyHelper.getDwdLaboratoryReportVO(entity);

        DwdLaboratoryReportItemQueryDTO dwdLaboratoryReportItemQueryDTO = new DwdLaboratoryReportItemQueryDTO();
        dwdLaboratoryReportItemQueryDTO.setLaboratoryReportId(id);
        List<DwdLaboratoryReportItemVO> dwdLaboratoryReportItemVOList = dwdLaboratoryReportItemBizService.list(dwdLaboratoryReportItemQueryDTO);
        vo.setDwdLaboratoryReportItemVOs(dwdLaboratoryReportItemVOList);

        return vo;
    }

    @Override
    public List<DwdLaboratoryReportInfo> listBeanByPatient(String patientId) {
        List<DwdLaboratoryReportInfo> data = dwdLaboratoryReportInfoService.list(new QueryWrapper<DwdLaboratoryReportInfo>()
                .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdLaboratoryReportInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdLaboratoryReportInfo> data = dwdLaboratoryReportInfoService.list(new QueryWrapper<DwdLaboratoryReportInfo>()
                .in("patient_id",patientIds)
                //
        );
        return data;
    }

}
