package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdLaboratoryReportItemInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdLaboratoryReportItemInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdLaboratoryReportItemQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdLaboratoryReportItemVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdLaboratoryReportItemBizService;
import com.xxh.cloud.framework.common.constant.YN;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@DS("wd")
@Service
public class DwdLaboratoryReportItemBizServiceImpl implements DwdLaboratoryReportItemBizService {

    @Autowired
    private DwdLaboratoryReportItemInfoService dwdLaboratoryReportItemInfoService;

    @Override
    public List<DwdLaboratoryReportItemVO> list(DwdLaboratoryReportItemQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<DwdLaboratoryReportItemInfo> data = dwdLaboratoryReportItemInfoService.list(new QueryWrapper<DwdLaboratoryReportItemInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getLaboratoryReportId()), "laboratory_report_id", params.getLaboratoryReportId())
                .in(CollectionUtil.isNotEmpty(params.getLaboratoryReportIds()), "laboratory_report_id", params.getLaboratoryReportIds())
                .ne(StrUtil.isNotBlank(params.getIsAbnormal()) && YN.YES.getCode().equals(params.getIsAbnormal()), "abnormal_indicator", "正常") // TODO 临时使用原始数据，之后换成标准数据
                .ge(null != params.getStartDate(), "reports_time", params.getStartDate())
                .lt(null != params.getEndDate(), "reports_time", params.getEndDate())
                .eq(StrUtil.isNotBlank(params.getProjectIdRaw()), "project_id_raw", params.getProjectIdRaw())
                .eq(StrUtil.isNotBlank(params.getProjectIdStandard()), "project_id_standard", params.getProjectIdStandard())
                .in(CollectionUtil.isNotEmpty(params.getProjectIdStandards()), "project_id_standard", params.getProjectIdStandards())
                .eq(StrUtil.isNotBlank(params.getNameRaw()), "name_raw", params.getNameRaw())
                .eq(StrUtil.isNotBlank(params.getNameStandard()), "name_standard", params.getNameStandard())
                .orderByDesc("reports_time")
                .last(" limit 10000 ") // TODO 防止不传参数查询所有导致爆库
        );
        return data.stream().map(PatientHealthyHelper::getDwdLaboratoryReportItemVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdLaboratoryReportItemVO> listByPatientId(String patientId) {
        List<DwdLaboratoryReportItemInfo> data = dwdLaboratoryReportItemInfoService.list(new QueryWrapper<DwdLaboratoryReportItemInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdLaboratoryReportItemVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdLaboratoryReportItemVO> listByPatientIds(List<String> patientIds) {
        List<DwdLaboratoryReportItemInfo> data = dwdLaboratoryReportItemInfoService.list(new QueryWrapper<DwdLaboratoryReportItemInfo>()
                .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdLaboratoryReportItemVO).collect(Collectors.toList());
    }

    @Override
    public DwdLaboratoryReportItemVO getById(String id) {
        DwdLaboratoryReportItemInfo entity = dwdLaboratoryReportItemInfoService.getById(id);
        return PatientHealthyHelper.getDwdLaboratoryReportItemVO(entity);
    }

    @Override
    public List<DwdLaboratoryReportItemInfo> listBeanByPatient(String patientId) {
        List<DwdLaboratoryReportItemInfo> data = dwdLaboratoryReportItemInfoService.list(new QueryWrapper<DwdLaboratoryReportItemInfo>()
                .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdLaboratoryReportItemInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdLaboratoryReportItemInfo> data = dwdLaboratoryReportItemInfoService.list(new QueryWrapper<DwdLaboratoryReportItemInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }

}
