package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdOperationApplyInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdOperationApplyInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdOperationApplyItemQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdOperationApplyQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdOperationApplyItemVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdOperationApplyVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdOperationApplyBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdOperationApplyItemBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@DS("wd")
@Service
public class DwdOperationApplyBizServiceImpl implements DwdOperationApplyBizService {

    @Autowired
    private DwdOperationApplyInfoService dwdOperationApplyInfoService;
    @Autowired
    private DwdOperationApplyItemBizService dwdOperationApplyItemBizService;

    @Override
    public List<DwdOperationApplyVO> list(DwdOperationApplyQueryDTO params) {
        List<DwdOperationApplyInfo> data = dwdOperationApplyInfoService.list(new QueryWrapper<DwdOperationApplyInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
        );
        List<DwdOperationApplyVO> list = data.stream().map(PatientHealthyHelper::getDwdOperationApplyVO).collect(Collectors.toList());

        // 手术申请明细
        if (CollectionUtil.isNotEmpty(list)) {
            List<String> ids = list.stream().map(DwdOperationApplyVO::getId).collect(Collectors.toList());
            Map<String, List<DwdOperationApplyItemVO>> dwdRecipeMedicineItemVOMap = this.getDwdOperationApplyItemVOMap(params.getPatientId(), ids);
            if (CollectionUtil.isNotEmpty(dwdRecipeMedicineItemVOMap)) {
                list.forEach(i -> i.setDwdOperationApplyItemVOs(dwdRecipeMedicineItemVOMap.get(i.getId())));
            }
        }

        return list;
    }

    // 手术申请明细
    private Map<String, List<DwdOperationApplyItemVO>> getDwdOperationApplyItemVOMap(String patientId, List<String> applyIds) {
        DwdOperationApplyItemQueryDTO queryDTO = new DwdOperationApplyItemQueryDTO();
        queryDTO.setPatientId(patientId);
        queryDTO.setOperationApplyIds(applyIds);
        List<DwdOperationApplyItemVO> voList = dwdOperationApplyItemBizService.list(queryDTO);

        Map<String, List<DwdOperationApplyItemVO>> voMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(voList)) {
            voList.forEach(i -> {
                List<DwdOperationApplyItemVO> vos = voMap.get(i.getOperationApplyId());
                if (null == vos) {
                    vos = new ArrayList<>();
                    voMap.put(i.getOperationApplyId(), vos);
                }
                vos.add(i);
            });
        }
        return voMap;
    }

    @Override
    public List<DwdOperationApplyVO> listByPatientId(String patientId) {
        List<DwdOperationApplyInfo> data = dwdOperationApplyInfoService.list(new QueryWrapper<DwdOperationApplyInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdOperationApplyVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdOperationApplyVO> listByPatientId(List<String> patientIds) {
        List<DwdOperationApplyInfo> data = dwdOperationApplyInfoService.list(new QueryWrapper<DwdOperationApplyInfo>()
                .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdOperationApplyVO).collect(Collectors.toList());
    }

    @Override
    public DwdOperationApplyVO getById(String id) {
        DwdOperationApplyInfo entity = dwdOperationApplyInfoService.getById(id);
        return PatientHealthyHelper.getDwdOperationApplyVO(entity);
    }

    @Override
    public List<DwdOperationApplyInfo> listBeanByPatient(String patientId) {
        List<DwdOperationApplyInfo> data = dwdOperationApplyInfoService.list(new QueryWrapper<DwdOperationApplyInfo>()
                .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdOperationApplyInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdOperationApplyInfo> data = dwdOperationApplyInfoService.list(new QueryWrapper<DwdOperationApplyInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }

}
