package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdOperationRecordInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdOperationRecordInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdOperationRecordItemQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdOperationRecordQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdOperationRecordItemVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdOperationRecordVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdOperationRecordBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdOperationRecordItemBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@DS("wd")
@Service
public class DwdOperationRecordBizServiceImpl implements DwdOperationRecordBizService {

    @Autowired
    private DwdOperationRecordInfoService dwdOperationRecordInfoService;
    @Autowired
    private DwdOperationRecordItemBizService dwdOperationRecordItemBizService;

    @Override
    public List<DwdOperationRecordVO> list(DwdOperationRecordQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<DwdOperationRecordInfo> data = dwdOperationRecordInfoService.list(new QueryWrapper<DwdOperationRecordInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .ge(null != params.getStartDate(), "pay_time", params.getStartDate())
                .lt(null != params.getEndDate(), "pay_time", params.getEndDate())
                .orderByDesc("pay_time")
                .last(" limit 1000 ") // TODO 防止不传参数查询所有导致爆库
        );
        List<DwdOperationRecordVO> list = data.stream().map(PatientHealthyHelper::getDwdOperationRecordVO).collect(Collectors.toList());

        // 手术记录明细
        if (CollectionUtil.isNotEmpty(list)) {
            List<String> ids = list.stream().map(DwdOperationRecordVO::getId).collect(Collectors.toList());
            Map<String, List<DwdOperationRecordItemVO>> dwdRecipeMedicineItemVOMap = this.getDwdOperationRecordItemVOMap(params.getPatientId(), ids);
            if (CollectionUtil.isNotEmpty(dwdRecipeMedicineItemVOMap)) {
                list.forEach(i -> i.setDwdOperationRecordItemVOs(dwdRecipeMedicineItemVOMap.get(i.getId())));
            }
        }

        return list;
    }

    // 手术记录明细
    private Map<String, List<DwdOperationRecordItemVO>> getDwdOperationRecordItemVOMap(String patientId, List<String> recordIds) {
        DwdOperationRecordItemQueryDTO queryDTO = new DwdOperationRecordItemQueryDTO();
        queryDTO.setPatientId(patientId);
        queryDTO.setOperationRecordIds(recordIds);
        List<DwdOperationRecordItemVO> voList = dwdOperationRecordItemBizService.list(queryDTO);

        Map<String, List<DwdOperationRecordItemVO>> voMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(voList)) {
            voList.forEach(i -> {
                List<DwdOperationRecordItemVO> vos = voMap.get(i.getOperationRecordId());
                if (null == vos) {
                    vos = new ArrayList<>();
                    voMap.put(i.getOperationRecordId(), vos);
                }
                vos.add(i);
            });
        }
        return voMap;
    }

    @Override
    public List<DwdOperationRecordVO> listByPatientId(String patientId) {
        List<DwdOperationRecordInfo> data = dwdOperationRecordInfoService.list(new QueryWrapper<DwdOperationRecordInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdOperationRecordVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdOperationRecordVO> listByPatientId(List<String> patientIds) {
        List<DwdOperationRecordInfo> data = dwdOperationRecordInfoService.list(new QueryWrapper<DwdOperationRecordInfo>()
                .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdOperationRecordVO).collect(Collectors.toList());
    }

    @Override
    public DwdOperationRecordVO getById(String id) {
        DwdOperationRecordInfo entity = dwdOperationRecordInfoService.getById(id);
        return PatientHealthyHelper.getDwdOperationRecordVO(entity);
    }

    @Override
    public List<DwdOperationRecordInfo> listBeanByPatient(String patientId) {
        List<DwdOperationRecordInfo> data = dwdOperationRecordInfoService.list(new QueryWrapper<DwdOperationRecordInfo>()
                .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdOperationRecordInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdOperationRecordInfo> data = dwdOperationRecordInfoService.list(new QueryWrapper<DwdOperationRecordInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }

}
