package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdOperationRecordItemInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdOperationRecordItemInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdOperationRecordItemQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdOperationRecordItemVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdOperationRecordItemBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@DS("wd")
@Service
public class DwdOperationRecordItemBizServiceImpl implements DwdOperationRecordItemBizService {

    @Autowired
    private DwdOperationRecordItemInfoService dwdOperationRecordItemInfoService;

    @Override
    public List<DwdOperationRecordItemVO> list(DwdOperationRecordItemQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<DwdOperationRecordItemInfo> data = dwdOperationRecordItemInfoService.list(new QueryWrapper<DwdOperationRecordItemInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getOperationRecordId()), "operation_record_id", params.getOperationRecordId())
                .in(CollectionUtil.isNotEmpty(params.getOperationRecordIds()), "operation_record_id", params.getOperationRecordIds())
                .ge(null != params.getStartDate(), "act_end_time", params.getStartDate())
                .lt(null != params.getEndDate(), "act_end_time", params.getEndDate())
                .orderByDesc("act_end_time")
                .last(" limit 1000 ") // TODO 防止不传参数查询所有导致爆库
        );
        return data.stream().map(PatientHealthyHelper::getDwdOperationRecordItemVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdOperationRecordItemVO> listByPatientId(String patientId) {
        List<DwdOperationRecordItemInfo> data = dwdOperationRecordItemInfoService.list(new QueryWrapper<DwdOperationRecordItemInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdOperationRecordItemVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdOperationRecordItemVO> listByPatientId(List<String> patientIds) {
        List<DwdOperationRecordItemInfo> data = dwdOperationRecordItemInfoService.list(new QueryWrapper<DwdOperationRecordItemInfo>()
                .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdOperationRecordItemVO).collect(Collectors.toList());
    }

    @Override
    public DwdOperationRecordItemVO getById(String id) {
        DwdOperationRecordItemInfo entity = dwdOperationRecordItemInfoService.getById(id);
        return PatientHealthyHelper.getDwdOperationRecordItemVO(entity);
    }

    @Override
    public List<DwdOperationRecordItemInfo> listBeanByPatient(String patientId) {
        List<DwdOperationRecordItemInfo> data = dwdOperationRecordItemInfoService.list(new QueryWrapper<DwdOperationRecordItemInfo>()
                .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdOperationRecordItemInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdOperationRecordItemInfo> data = dwdOperationRecordItemInfoService.list(new QueryWrapper<DwdOperationRecordItemInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }

}
