package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdPatientInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdPatientInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdPatientQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdPatientBizService;
import com.xxh.cloud.framework.common.constant.YN;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@DS("wd")
@Service
public class DwdPatientBizServiceImpl implements DwdPatientBizService {

    @Autowired
    private DwdPatientInfoService dwdPatientInfoService;


    @Override
    public IPage<DwdPatientInfo> query(DwdPatientQueryDTO params) {
        IPage<DwdPatientInfo> page = dwdPatientInfoService.page(
                new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<DwdPatientInfo>()
                .eq(StrUtil.isNotEmpty(params.getName()),"patient_name",params.getName())
        );
        return page;
    }


//    @Override
//    public List<DwdPatientInfo> listByIdCardNo(String cardNoType, String cardNo) {
//        List<DwdPatientInfo> data =  dwdPatientInfoService.list(new QueryWrapper<DwdPatientInfo>()
//                .eq("card_no_type_standard",cardNoType)
//                .eq("card_no",cardNo)
//        );
//        return data;
//    }



    @Override
    public List<DwdPatientInfo> listByIds(List<String> patientIds) {
       List<DwdPatientInfo> data =  dwdPatientInfoService.list(new QueryWrapper<DwdPatientInfo>()
                .in("patient_id",patientIds)
               .ne("death_status", YN.NO.getCode())
       );
        return data;
    }

    @Override
    public DwdPatientInfo getById(String patientId) {
        return  dwdPatientInfoService.getById(patientId);
    }


    @Override
    public List<String> getPatientIdByMainPersonId(String mainPersonId) {
        if (StrUtil.isBlank(mainPersonId)) {
            return new ArrayList<>(0);
        }

        List<String> patientIds = new ArrayList<>();
        patientIds.add(mainPersonId);

        List<DwdPatientInfo> patientList = dwdPatientInfoService.list(new QueryWrapper<DwdPatientInfo>()
            .eq("main_person_id", mainPersonId)
        );
        if (CollectionUtil.isNotEmpty(patientList)) {
            patientList.forEach(i -> patientIds.add(i.getPatientId()));
        }

        return patientIds;
    }

    @Override
    public List<String> getPatientIdsByMainPersonIds(List<String> mainPersonIds) {
        if (CollectionUtil.isEmpty(mainPersonIds)) {
            return new ArrayList<>(0);
        }

        List<String> patientIds = new ArrayList<>();
        patientIds.addAll(mainPersonIds);

        List<DwdPatientInfo> patientList = dwdPatientInfoService.list(new QueryWrapper<DwdPatientInfo>()
            .in("main_person_id", mainPersonIds)
        );
        if (CollectionUtil.isNotEmpty(patientList)) {
            patientList.forEach(i -> patientIds.add(i.getPatientId()));
        }

        return patientIds;
    }

    @Override
    public Map<String, List<String>> mapPatientIdsByMainPersonIds(List<String> mainPersonIds) {
        if (CollectionUtil.isEmpty(mainPersonIds)) {
            return new HashMap<>(0);
        }
        
        Map<String, List<String>> map = new HashMap<>();
        mainPersonIds.forEach(i -> {
            List<String> l = map.get(i);
            if (l == null) {
                l = new ArrayList<>();
                map.put(i, l);
            }
            l.add(i);
        });


        List<DwdPatientInfo> list = dwdPatientInfoService.list(new QueryWrapper<DwdPatientInfo>()
            .in("main_person_id", mainPersonIds)
        );
        if (CollectionUtil.isNotEmpty(list)) {
            list.forEach(i -> {
                List<String> l = map.get(i.getMainPersonId());
                if (l == null) {
                    l = new ArrayList<>();
                    map.put(i.getMainPersonId(), l);
                }
                l.add(i.getPatientId());
            });
        }

        return map;
    }

    @Override
    public String getMainPersonIdByPatientId(String patientId) {
        if (StrUtil.isBlank(patientId)) {
            return null;
        }
        
        DwdPatientInfo entity = dwdPatientInfoService.getOne(new QueryWrapper<DwdPatientInfo>()
            .eq("patient_id", patientId)
        );
        if (null == entity) {
            return null;
        }
        return entity.getMainPersonId();
    }


    @Override
    public List<String> getMainPersonIdsByPatientIds(List<String> patientIds) {
        if (CollectionUtil.isEmpty(patientIds)) {
            return new ArrayList<>(0);
        }
        List<DwdPatientInfo> list = dwdPatientInfoService.list(new QueryWrapper<DwdPatientInfo>()
            .in("patient_id", patientIds)
        );
        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }
        return list.stream().map(DwdPatientInfo::getMainPersonId).collect(Collectors.toList());
    }

    @Override
    public Map<String, String> mapMainPersonIdsByPatientIds(List<String> patientIds) {
        if (CollectionUtil.isEmpty(patientIds)) {
            return new HashMap<>(0);
        }
        List<DwdPatientInfo> list = dwdPatientInfoService.list(new QueryWrapper<DwdPatientInfo>()
            .in("patient_id", patientIds)
        );
        if (CollectionUtil.isEmpty(list)) {
            return new HashMap<>(0);
        }
        return list.stream().collect(Collectors.toMap(DwdPatientInfo::getPatientId, DwdPatientInfo::getMainPersonId));
    }

}
