package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.alibaba.fastjson2.JSON;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.client.api.constant.DwdConstant;
import com.xxh.cloud.dw.healthy.data.client.api.constant.DwdConstant.DataSceneEnum;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdExaminationPatientInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdInpatientRecordInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdVisitRecordInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdExaminationPatientInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdInpatientRecordInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdVisitRecordInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdExaminationPatientQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdInpatientRecordQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdPatientDataQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdVisitRecordQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdExaminationPatientVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdInpatientRecordVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdPatientDataVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdVisitRecordVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.mapper.DwdPatientHealthInfoBizMapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdExaminationPatientBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdInpatientRecordBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdPatientDataBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdVisitRecordBizService;
import com.xxh.cloud.framework.common.constant.YN;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@DS("wd")
@Slf4j
@Service
public class DwdPatientDataBizServiceImpl implements DwdPatientDataBizService {

    @Autowired
    private DwdVisitRecordInfoService dwdVisitRecordInfoService;
    @Autowired
    private DwdVisitRecordBizService dwdVisitRecordBizService;

    @Autowired
    private DwdInpatientRecordInfoService dwdInpatientRecordInfoService;
    @Autowired
    private DwdInpatientRecordBizService dwdInpatientRecordBizService;

    @Autowired
    private DwdExaminationPatientInfoService dwdExaminationPatientInfoService;
    @Autowired
    private DwdExaminationPatientBizService dwdExaminationPatientBizService;

    @Autowired
    private DwdPatientHealthInfoBizMapper dwdPatientHealthInfoBizMapper;

    // 移步PatientDwdDataBizServiceImpl.list()
    @Override
    public List<DwdPatientDataVO> list(DwdPatientDataQueryDTO params) {
        log.info("DwdPatientDataBizServiceImpl.list::params >>>>>> {}", JSON.toJSONString(params));
        
        // 就诊信息
        List<DwdVisitRecordVO> dwdVisitRecordVOs = this.visitRecordList(params);
        // 住院信息
        List<DwdInpatientRecordVO> dwdInpatientRecordVOs = this.inpatientRecordList(params);

        if (CollectionUtil.isEmpty(dwdVisitRecordVOs) && CollectionUtil.isEmpty(dwdInpatientRecordVOs)) {
            return new ArrayList<>(0);
        }

        // 合并信息
        List<DwdPatientDataVO> list = new ArrayList<>(dwdVisitRecordVOs.size() + dwdInpatientRecordVOs.size());

        int visitRecordIndex = 0;
        int inpatientRecordIndex = 0;
        while (visitRecordIndex < dwdVisitRecordVOs.size() && inpatientRecordIndex < dwdInpatientRecordVOs.size()) {
            if (dwdVisitRecordVOs.get(visitRecordIndex).getVisitDate().after(dwdInpatientRecordVOs.get(inpatientRecordIndex).getInpatientDate())) {
                DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                dwdPatientDataVO.setDwdVisitRecordVO(dwdVisitRecordVOs.get(visitRecordIndex));
                list.add(dwdPatientDataVO);
                
                visitRecordIndex++;
            } else {
                DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                dwdPatientDataVO.setDwdInpatientRecordVO(dwdInpatientRecordVOs.get(inpatientRecordIndex));
                list.add(dwdPatientDataVO);

                inpatientRecordIndex++;
            }
        }

        while (visitRecordIndex < dwdVisitRecordVOs.size()) {
            DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
            dwdPatientDataVO.setDwdVisitRecordVO(dwdVisitRecordVOs.get(visitRecordIndex));
            list.add(dwdPatientDataVO);

            visitRecordIndex++;
        }

        while (inpatientRecordIndex < dwdInpatientRecordVOs.size()) {
            DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
            dwdPatientDataVO.setDwdInpatientRecordVO(dwdInpatientRecordVOs.get(inpatientRecordIndex));
            list.add(dwdPatientDataVO);

            inpatientRecordIndex++;
        }

        return list;
    }

    // 根据参数查询就诊数据
    @Override
    public List<DwdVisitRecordVO> visitRecordList(DwdPatientDataQueryDTO params) {
        // 住院场景不需要查询就诊信息
        if (StrUtil.isNotBlank(params.getDataScene()) && !DataSceneEnum.MZ.getCode().equals(params.getDataScene()) && !DataSceneEnum.JZ.getCode().equals(params.getDataScene())) {
            return new ArrayList<>(0);
        }

        if (StrUtil.isNotBlank(params.getDataType())) {
            return this.visitRecordWithDataType(params); // 指定数据类型
        }
        return this.visitRecord(params); // 全部数据类型
    }

    // 指定数据类型的就诊数据
    private List<DwdVisitRecordVO> visitRecordWithDataType(DwdPatientDataQueryDTO params) {
        List<DwdVisitRecordInfo> data = null;

        QueryWrapper<DwdVisitRecordInfo> queryWrapper = new QueryWrapper<DwdVisitRecordInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "a.patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "a.patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getDataScene()), "a.scene_type_standard", params.getDataScene())
                .lt(StrUtil.isNotBlank(params.getQueryDate()), "a.visit_date", params.getQueryDate())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.visit_date", params.getStartDate())
                .eq((StrUtil.isNotBlank(params.getIsDept()) && YN.YES.getCode().equals(params.getIsDept())) && StrUtil.isNotBlank(params.getDeptId()), "a.department_code_standard", params.getDeptId()) // 查询指定部门的数据
                .ne((StrUtil.isNotBlank(params.getIsDept()) && YN.NO.getCode().equals(params.getIsDept())) && StrUtil.isNotBlank(params.getDeptId()), "a.department_code_standard", params.getDeptId()); // 过滤指定部门的数据

        if (DwdConstant.DataTypeEnum.RegisterRecord.getCode().equals(params.getDataType())) { // 挂号
            data = dwdPatientHealthInfoBizMapper.visitListWithRegisterRecord(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.ExamineApply.getCode().equals(params.getDataType())) { // 检查申请单
            data = dwdPatientHealthInfoBizMapper.visitListWithExamineApply(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.LaboratoryApply.getCode().equals(params.getDataType())) { // 检验申请单
            data = dwdPatientHealthInfoBizMapper.visitListWithLaboratoryApply(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.RecipeMedicine.getCode().equals(params.getDataType())) { // 医嘱用药
            data = dwdPatientHealthInfoBizMapper.visitListWithRecipeMedicine(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.OperationApply.getCode().equals(params.getDataType())) { // 手术申请
            data = dwdPatientHealthInfoBizMapper.visitListWithOperationApply(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.ExamineReserve.getCode().equals(params.getDataType())) { // 检查预约单
            data = dwdPatientHealthInfoBizMapper.visitListWithExamineReserve(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.ExamineReport.getCode().equals(params.getDataType())) { // 检查报告单
            data = dwdPatientHealthInfoBizMapper.visitListWithExamineReport(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.LaboratoryReport.getCode().equals(params.getDataType())) { // 检验报告单
            data = dwdPatientHealthInfoBizMapper.visitListWithLaboratoryReport(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.OperationRecord.getCode().equals(params.getDataType())) { // 手术记录
            data = dwdPatientHealthInfoBizMapper.visitListWithOperationRecord(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.DiagnosisReport.getCode().equals(params.getDataType())) { // 诊断
            data = dwdPatientHealthInfoBizMapper.visitListWithDiagnosisReport(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.RecipeTherapy.getCode().equals(params.getDataType())) { // 治疗
            data = dwdPatientHealthInfoBizMapper.visitListWithRecipeTherapy(queryWrapper);
        }

        if (null == data || CollectionUtil.isEmpty(data)) {
            return new ArrayList<>(0);
        }

        DwdVisitRecordQueryDTO queryDTO = new DwdVisitRecordQueryDTO();
        queryDTO.setPatientId(params.getPatientId());
        queryDTO.setPatientIds(params.getPatientIds());
        queryDTO.setIds(data.stream().map(DwdVisitRecordInfo::getId).collect(Collectors.toList()));
        queryDTO.setDataType(params.getDataType());
        return dwdVisitRecordBizService.list(queryDTO);
    }

    // 全部数据类型的就诊数据
    private List<DwdVisitRecordVO> visitRecord(DwdPatientDataQueryDTO params) {
        // 根据指定的时间查询固定条数的就诊数据对应的时间
        List<DwdVisitRecordInfo> data = dwdVisitRecordInfoService.list(new QueryWrapper<DwdVisitRecordInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getDataScene()), "scene_type_standard", params.getDataScene())
                .lt(StrUtil.isNotBlank(params.getQueryDate()), "visit_date", params.getQueryDate())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "visit_date", params.getStartDate())
                .eq((StrUtil.isNotBlank(params.getIsDept()) && YN.YES.getCode().equals(params.getIsDept())) && StrUtil.isNotBlank(params.getDeptId()), "department_code_standard", params.getDeptId()) // 查询指定部门的数据
                .ne((StrUtil.isNotBlank(params.getIsDept()) && YN.NO.getCode().equals(params.getIsDept())) && StrUtil.isNotBlank(params.getDeptId()), "department_code_standard", params.getDeptId()) // 过滤指定部门的数据
                .orderByDesc("visit_date")
                .last(" limit 5 ")
        );

        if (CollectionUtil.isEmpty(data)) {
            return new ArrayList<>(0);
        }

        String startDate = DateUtil.formatDate(data.get(data.size() -1).getVisitDate());
        String endDate = DateUtil.formatDate(DateUtil.offsetDay(data.get(0).getVisitDate(), 1));

        DwdVisitRecordQueryDTO queryDTO = new DwdVisitRecordQueryDTO();
        queryDTO.setPatientId(params.getPatientId());
        queryDTO.setPatientIds(params.getPatientIds());
        queryDTO.setStartDate(startDate);
        queryDTO.setEndDate(endDate);

        queryDTO.setDeptId(params.getDeptId());
        queryDTO.setIsDept(params.getIsDept());

        return dwdVisitRecordBizService.list(queryDTO);
    }

    // 住院数据
    @Override
    public List<DwdInpatientRecordVO> inpatientRecordList(DwdPatientDataQueryDTO params) {
        // 住院场景不需要查询就诊信息
        if (StrUtil.isNotBlank(params.getDataScene()) && !DataSceneEnum.ZY.getCode().equals(params.getDataScene())) {
            return new ArrayList<>(0);
        }

        if (StrUtil.isNotBlank(params.getDataType())) {
            return this.inpatientRecordWithDataType(params); // 指定数据类型
        }
        return this.inpatientRecord(params); // 全部数据类型
    }

    // 住院数据
    private List<DwdInpatientRecordVO> inpatientRecordWithDataType(DwdPatientDataQueryDTO params) {
        List<DwdInpatientRecordInfo> data = null;

        QueryWrapper<DwdInpatientRecordInfo> queryWrapper = new QueryWrapper<DwdInpatientRecordInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "a.patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "a.patient_id", params.getPatientIds())
                .lt(StrUtil.isNotBlank(params.getQueryDate()), "a.inpatient_date", params.getQueryDate())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.inpatient_date", params.getStartDate())
                .eq((StrUtil.isNotBlank(params.getIsDept()) && YN.YES.getCode().equals(params.getIsDept())) && StrUtil.isNotBlank(params.getDeptId()), "a.admit_department_code_standard", params.getDeptId()) // 查询指定部门的数据
                .ne((StrUtil.isNotBlank(params.getIsDept()) && YN.NO.getCode().equals(params.getIsDept())) && StrUtil.isNotBlank(params.getDeptId()), "a.admit_department_code_standard", params.getDeptId()); // 过滤指定部门的数据

        if (DwdConstant.DataTypeEnum.InpatientReserve.getCode().equals(params.getDataType())) { // 住院预约，单独处理
            // data = dwdPatientHealthInfoBizMapper.inHospitalListWithInpatientReserve(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.DischargeRecord.getCode().equals(params.getDataType())) { // 出院
            data = dwdPatientHealthInfoBizMapper.inHospitalListWithDischargeRecord(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.ExamineApply.getCode().equals(params.getDataType())) { // 检查申请单
            data = dwdPatientHealthInfoBizMapper.inHospitalListWithExamineApply(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.LaboratoryApply.getCode().equals(params.getDataType())) { // 检验申请单
            data = dwdPatientHealthInfoBizMapper.inHospitalListWithLaboratoryApply(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.RecipeMedicine.getCode().equals(params.getDataType())) { // 医嘱用药
            data = dwdPatientHealthInfoBizMapper.inHospitalListWithRecipeMedicine(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.OperationApply.getCode().equals(params.getDataType())) { // 手术申请
            data = dwdPatientHealthInfoBizMapper.inHospitalListWithOperationApply(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.ExamineReserve.getCode().equals(params.getDataType())) { // 检查预约单
            data = dwdPatientHealthInfoBizMapper.inHospitalListWithExamineReserve(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.ExamineReport.getCode().equals(params.getDataType())) { // 检查报告单
            data = dwdPatientHealthInfoBizMapper.inHospitalListWithExamineReport(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.LaboratoryReport.getCode().equals(params.getDataType())) { // 检验报告单
            data = dwdPatientHealthInfoBizMapper.inHospitalListWithLaboratoryReport(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.OperationRecord.getCode().equals(params.getDataType())) { // 手术记录
            data = dwdPatientHealthInfoBizMapper.inHospitalListWithOperationRecord(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.DiagnosisReport.getCode().equals(params.getDataType())) { // 诊断
            data = dwdPatientHealthInfoBizMapper.inHospitalListWithDiagnosisReport(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.RecipeTherapy.getCode().equals(params.getDataType())) { // 治疗
            data = dwdPatientHealthInfoBizMapper.inHospitalListWithRecipeTherapy(queryWrapper);
        }

        if (null == data || CollectionUtil.isEmpty(data)) {
            return new ArrayList<>(0);
        }

        DwdInpatientRecordQueryDTO queryDTO = new DwdInpatientRecordQueryDTO();
        queryDTO.setPatientId(params.getPatientId());
        queryDTO.setPatientIds(params.getPatientIds());
        queryDTO.setIds(data.stream().map(DwdInpatientRecordInfo::getId).collect(Collectors.toList()));
        queryDTO.setDataType(params.getDataType());
        return dwdInpatientRecordBizService.list(queryDTO);
    }

    // 全部数据类型的住院数据
    private List<DwdInpatientRecordVO> inpatientRecord(DwdPatientDataQueryDTO params) {
        // 根据指定的时间查询固定条数的就诊数据对应的时间
        List<DwdInpatientRecordInfo> data = dwdInpatientRecordInfoService.list(new QueryWrapper<DwdInpatientRecordInfo>()
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .lt(StrUtil.isNotBlank(params.getQueryDate()), "inpatient_date", params.getQueryDate())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "inpatient_date", params.getStartDate())
                .eq((StrUtil.isNotBlank(params.getIsDept()) && YN.YES.getCode().equals(params.getIsDept())) && StrUtil.isNotBlank(params.getDeptId()), "admit_department_code_standard", params.getDeptId()) // 查询指定部门的数据
                .ne((StrUtil.isNotBlank(params.getIsDept()) && YN.NO.getCode().equals(params.getIsDept())) && StrUtil.isNotBlank(params.getDeptId()), "admit_department_code_standard", params.getDeptId()) // 过滤指定部门的数据
                .orderByDesc("inpatient_date")
                .last(" limit 5 ")
        );

        if (CollectionUtil.isEmpty(data)) {
            return new ArrayList<>(0);
        }

        String startDate = DateUtil.formatDate(data.get(data.size() -1).getInpatientDate());
        String endDate = DateUtil.formatDate(DateUtil.offsetDay(data.get(0).getInpatientDate(), 1));

        DwdInpatientRecordQueryDTO queryDTO = new DwdInpatientRecordQueryDTO();
        queryDTO.setPatientId(params.getPatientId());
        queryDTO.setPatientIds(params.getPatientIds());
        queryDTO.setStartDate(startDate);
        queryDTO.setEndDate(endDate);

        queryDTO.setDeptId(params.getDeptId());
        queryDTO.setIsDept(params.getIsDept());

        return dwdInpatientRecordBizService.list(queryDTO);
    }

    // 体检数据
    @Override
    public List<DwdExaminationPatientVO> examinationPatientList(DwdPatientDataQueryDTO params) {
        if (StrUtil.isBlank(params.getMainPersonId())) {
            return new ArrayList<>(0);
        }
        // 住院场景不需要查询就诊信息
        if (StrUtil.isNotBlank(params.getDataScene()) && !DataSceneEnum.TJ.getCode().equals(params.getDataScene())) {
            return new ArrayList<>(0);
        }

        if (StrUtil.isNotBlank(params.getDataType())) {
            return this.examinationPatientWithDataType(params); // 指定数据类型
        }
        return this.examinationPatient(params); // 全部数据类型
    }

    // 体检数据
    private List<DwdExaminationPatientVO> examinationPatientWithDataType(DwdPatientDataQueryDTO params) {
        List<DwdExaminationPatientInfo> data = null;

        QueryWrapper<DwdExaminationPatientInfo> queryWrapper = new QueryWrapper<DwdExaminationPatientInfo>()
                .eq(StrUtil.isNotBlank(params.getMainPersonId()), "a.main_person_id", params.getMainPersonId())
                .lt(StrUtil.isNotBlank(params.getQueryDate()), "a.data_create_date", params.getQueryDate())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "a.data_create_date", params.getStartDate());

        if (DwdConstant.DataTypeEnum.ExamineReport.getCode().equals(params.getDataType())) { // 检查报告单
            data = dwdPatientHealthInfoBizMapper.examinationListWithExamineReport(queryWrapper);
        } else if (DwdConstant.DataTypeEnum.LaboratoryReport.getCode().equals(params.getDataType())) { // 检验报告单
            data = dwdPatientHealthInfoBizMapper.examinationListWithLaboratoryReport(queryWrapper);
        }

        if (null == data || CollectionUtil.isEmpty(data)) {
            return new ArrayList<>(0);
        }

        DwdExaminationPatientQueryDTO queryDTO = new DwdExaminationPatientQueryDTO();
        queryDTO.setMainPersonId(params.getMainPersonId());
        queryDTO.setVisitIds(data.stream().map(DwdExaminationPatientInfo::getVisitId).collect(Collectors.toList()));
        queryDTO.setDataType(params.getDataType());
        return dwdExaminationPatientBizService.list(queryDTO);
    }

    // 全部数据类型的体检数据
    private List<DwdExaminationPatientVO> examinationPatient(DwdPatientDataQueryDTO params) {
        // 根据指定的时间查询固定条数的就诊数据对应的时间
        List<DwdExaminationPatientInfo> data = dwdExaminationPatientInfoService.list(new QueryWrapper<DwdExaminationPatientInfo>()
                .eq(StrUtil.isNotBlank(params.getMainPersonId()), "main_person_id", params.getMainPersonId())
                .lt(StrUtil.isNotBlank(params.getQueryDate()), "data_create_date", params.getQueryDate())
                .ge(StrUtil.isNotBlank(params.getStartDate()), "data_create_date", params.getStartDate())
                .orderByDesc("data_create_date")
                .last(" limit 5 ")
        );

        if (CollectionUtil.isEmpty(data)) {
            return new ArrayList<>(0);
        }

        String startDate = DateUtil.formatDate(data.get(data.size() -1).getDataCreateDate());
        String endDate = DateUtil.formatDate(DateUtil.offsetDay(data.get(0).getDataCreateDate(), 1));

        DwdExaminationPatientQueryDTO queryDTO = new DwdExaminationPatientQueryDTO();
        queryDTO.setMainPersonId(params.getMainPersonId());
        queryDTO.setStartDate(startDate);
        queryDTO.setEndDate(endDate);
        return dwdExaminationPatientBizService.list(queryDTO);
    }

}
