package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdPatientMedicalRecordContentInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdPatientMedicalRecordContentInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdPatientMedicalRecordContentGetDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdPatientMedicalRecordContentQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdPatientMedicalRecordContentVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdPatientMedicalRecordContentBizService;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;

/**
 * <p>
 * DWD电子病例内容信息表
 * </p>
 */
@DS("wd")
@Service
public class DwdPatientMedicalRecordContentBizServiceImpl implements DwdPatientMedicalRecordContentBizService {

    @Autowired
    private DwdPatientMedicalRecordContentInfoService dwdPatientMedicalRecordContentInfoService;

    @Override
    public PageData<DwdPatientMedicalRecordContentVO> query(DwdPatientMedicalRecordContentQueryDTO params) {
        IPage<DwdPatientMedicalRecordContentInfo> page = dwdPatientMedicalRecordContentInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DwdPatientMedicalRecordContentInfo>()
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .in(CollUtil.isNotEmpty(params.getIds()), "id", params.getIds())
            .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
            .in(CollUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
        );

        if (null == page || CollUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        PageData<DwdPatientMedicalRecordContentVO> data = new PageConverter<DwdPatientMedicalRecordContentInfo, DwdPatientMedicalRecordContentVO>() {
            @Override
            public DwdPatientMedicalRecordContentVO converter(DwdPatientMedicalRecordContentInfo entity) {
                DwdPatientMedicalRecordContentVO vo = PatientHealthyHelper.getDwdPatientMedicalRecordContentVO(entity);
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<DwdPatientMedicalRecordContentVO> list(DwdPatientMedicalRecordContentQueryDTO params) {
        List<DwdPatientMedicalRecordContentInfo> list = dwdPatientMedicalRecordContentInfoService.list(new QueryWrapper<DwdPatientMedicalRecordContentInfo>()
            .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
            .in(CollUtil.isNotEmpty(params.getIds()), "id", params.getIds())
            .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
            .in(CollUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
            .last(" limit 1000 ")
        );

        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        return list.stream().map(PatientHealthyHelper::getDwdPatientMedicalRecordContentVO).collect(Collectors.toList());
    }

    @Override
    public DwdPatientMedicalRecordContentVO detail(DwdPatientMedicalRecordContentGetDTO params) {
        DwdPatientMedicalRecordContentInfo entity = dwdPatientMedicalRecordContentInfoService.getOne(new QueryWrapper<DwdPatientMedicalRecordContentInfo>()
            .eq("id", params.getId())
            .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
        );

        return PatientHealthyHelper.getDwdPatientMedicalRecordContentVO(entity);
    }

}
