package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdRecipeMedicineInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdRecipeMedicineInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdRecipeMedicineGetDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdRecipeMedicineItemQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdRecipeMedicineQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdRecipeMedicineItemVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdRecipeMedicineVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdRecipeMedicineBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdRecipeMedicineItemBizService;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@DS("wd")
@Service
public class DwdRecipeMedicineBizServiceImpl implements DwdRecipeMedicineBizService {

    @Autowired
    private DwdRecipeMedicineInfoService dwdRecipeMedicineInfoService;
    @Autowired
    private DwdRecipeMedicineItemBizService dwdRecipeMedicineItemBizService;

    @Override
    public PageData<DwdRecipeMedicineVO> query(DwdRecipeMedicineQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        IPage<DwdRecipeMedicineInfo> page = dwdRecipeMedicineInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<DwdRecipeMedicineInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .ge(null != params.getStartDate(), "recipe_time", params.getStartDate())
                .lt(null != params.getEndDate(), "recipe_time", params.getEndDate())
                .orderByDesc("recipe_time")
                .last(" limit 1000 ") // TODO 防止不传参数查询所有导致爆库
        );

        if (null == page || CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }
        
        // 医嘱用药明细
        List<String> recipeMedicineIds = page.getRecords().stream().map(DwdRecipeMedicineInfo::getId).collect(Collectors.toList());
        Map<String, List<DwdRecipeMedicineItemVO>> dwdRecipeMedicineItemVOMap = this.getDwdRecipeMedicineItemVOMap(params.getPatientId(), recipeMedicineIds);

        PageData<DwdRecipeMedicineVO> data = new PageConverter<DwdRecipeMedicineInfo, DwdRecipeMedicineVO>() {
            @Override
            public DwdRecipeMedicineVO converter(DwdRecipeMedicineInfo entity) {
                DwdRecipeMedicineVO vo = PatientHealthyHelper.getDwdRecipeMedicineVO(entity);
                vo.setDwdRecipeMedicineItemVOs(dwdRecipeMedicineItemVOMap.get(vo.getId()));
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<DwdRecipeMedicineVO> list(DwdRecipeMedicineQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<DwdRecipeMedicineInfo> data = dwdRecipeMedicineInfoService.list(new QueryWrapper<DwdRecipeMedicineInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .ge(null != params.getStartDate(), "recipe_time", params.getStartDate())
                .lt(null != params.getEndDate(), "recipe_time", params.getEndDate())
                .orderByDesc("recipe_time")
                .last(" limit 1000 ") // TODO 防止不传参数查询所有导致爆库
        );
        List<DwdRecipeMedicineVO> list = data.stream().map(PatientHealthyHelper::getDwdRecipeMedicineVO).collect(Collectors.toList());

        // 医嘱用药明细
        if (CollectionUtil.isNotEmpty(list)) {
            List<String> recipeMedicineIds = list.stream().map(DwdRecipeMedicineVO::getId).collect(Collectors.toList());
            Map<String, List<DwdRecipeMedicineItemVO>> dwdRecipeMedicineItemVOMap = this.getDwdRecipeMedicineItemVOMap(params.getPatientId(), recipeMedicineIds);
            if (CollectionUtil.isNotEmpty(dwdRecipeMedicineItemVOMap)) {
                list.forEach(i -> i.setDwdRecipeMedicineItemVOs(dwdRecipeMedicineItemVOMap.get(i.getId())));
            }
        }

        return list;
    }

    // 医嘱用药明细
    private Map<String, List<DwdRecipeMedicineItemVO>> getDwdRecipeMedicineItemVOMap(String patientId, List<String> recipeMedicineIds) {
        DwdRecipeMedicineItemQueryDTO queryDTO = new DwdRecipeMedicineItemQueryDTO();
        queryDTO.setPatientId(patientId);
        queryDTO.setRecipeMedicineIds(recipeMedicineIds);
        List<DwdRecipeMedicineItemVO> voList = dwdRecipeMedicineItemBizService.list(queryDTO);

        Map<String, List<DwdRecipeMedicineItemVO>> voMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(voList)) {
            voList.forEach(i -> {
                List<DwdRecipeMedicineItemVO> vos = voMap.get(i.getRecipeMedicineId());
                if (null == vos) {
                    vos = new ArrayList<>();
                    voMap.put(i.getRecipeMedicineId(), vos);
                }
                vos.add(i);
            });
        }
        return voMap;
    }

    @Override
    public DwdRecipeMedicineVO detail(DwdRecipeMedicineGetDTO params) {
        DwdRecipeMedicineInfo entity = dwdRecipeMedicineInfoService.getOne(new QueryWrapper<DwdRecipeMedicineInfo>()
                .eq("id", params.getId())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
        );
        if (null == entity) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        DwdRecipeMedicineVO vo = PatientHealthyHelper.getDwdRecipeMedicineVO(entity);

        DwdRecipeMedicineItemQueryDTO dwdRecipeMedicineItemQueryDTO = new DwdRecipeMedicineItemQueryDTO();
        dwdRecipeMedicineItemQueryDTO.setRecipeMedicineId(entity.getId());
        List<DwdRecipeMedicineItemVO> dwdRecipeMedicineItemVOList = dwdRecipeMedicineItemBizService.list(dwdRecipeMedicineItemQueryDTO);
        vo.setDwdRecipeMedicineItemVOs(dwdRecipeMedicineItemVOList);

        return vo;
    }

    @Override
    public List<DwdRecipeMedicineVO> listByPatientId(String patientId) {
        List<DwdRecipeMedicineInfo> data = dwdRecipeMedicineInfoService.list(new QueryWrapper<DwdRecipeMedicineInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdRecipeMedicineVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdRecipeMedicineVO> listByPatientId(List<String> patientIds) {
        List<DwdRecipeMedicineInfo> data = dwdRecipeMedicineInfoService.list(new QueryWrapper<DwdRecipeMedicineInfo>()
                .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdRecipeMedicineVO).collect(Collectors.toList());
    }

    @Override
    public DwdRecipeMedicineVO getById(String id) {
        DwdRecipeMedicineInfo entity = dwdRecipeMedicineInfoService.getById(id);
        if (null == entity) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        DwdRecipeMedicineVO vo = PatientHealthyHelper.getDwdRecipeMedicineVO(entity);

        DwdRecipeMedicineItemQueryDTO dwdRecipeMedicineItemQueryDTO = new DwdRecipeMedicineItemQueryDTO();
        dwdRecipeMedicineItemQueryDTO.setRecipeMedicineId(id);
        List<DwdRecipeMedicineItemVO> dwdRecipeMedicineItemVOList = dwdRecipeMedicineItemBizService.list(dwdRecipeMedicineItemQueryDTO);
        vo.setDwdRecipeMedicineItemVOs(dwdRecipeMedicineItemVOList);

        return vo;
    }

    @Override
    public List<DwdRecipeMedicineInfo> listBeanByPatient(String patientId) {
        List<DwdRecipeMedicineInfo> data = dwdRecipeMedicineInfoService.list(new QueryWrapper<DwdRecipeMedicineInfo>()
                .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdRecipeMedicineInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdRecipeMedicineInfo> data = dwdRecipeMedicineInfoService.list(new QueryWrapper<DwdRecipeMedicineInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }

}
