package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdRecipeMedicineItemInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdRecipeMedicineItemInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdRecipeMedicineItemQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdRecipeMedicineItemVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdRecipeMedicineItemBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@DS("wd")
@Service
public class DwdRecipeMedicineItemBizServiceImpl implements DwdRecipeMedicineItemBizService {

    @Autowired
    private DwdRecipeMedicineItemInfoService dwdRecipeMedicineItemInfoService;

    @Override
    public List<DwdRecipeMedicineItemVO> list(DwdRecipeMedicineItemQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<DwdRecipeMedicineItemInfo> data = dwdRecipeMedicineItemInfoService.list(new QueryWrapper<DwdRecipeMedicineItemInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getRecipeMedicineId()), "recipe_medicine_id", params.getRecipeMedicineId())
                .in(CollectionUtil.isNotEmpty(params.getRecipeMedicineIds()), "recipe_medicine_id", params.getRecipeMedicineIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .ge(null != params.getStartDate(), "recipe_time", params.getStartDate())
                .lt(null != params.getEndDate(), "recipe_time", params.getEndDate())
                .orderByDesc("recipe_time")
                .last(" limit 1000 ") // TODO 防止不传参数查询所有导致爆库
        );
        return data.stream().map(PatientHealthyHelper::getDwdRecipeMedicineItemVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdRecipeMedicineItemVO> listByPatientId(String patientId) {
        List<DwdRecipeMedicineItemInfo> data = dwdRecipeMedicineItemInfoService.list(new QueryWrapper<DwdRecipeMedicineItemInfo>()
                .eq("patient_id",patientId)
        );
        return data.stream().map(PatientHealthyHelper::getDwdRecipeMedicineItemVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdRecipeMedicineItemVO> listByPatientId(List<String> patientIds) {
        List<DwdRecipeMedicineItemInfo> data = dwdRecipeMedicineItemInfoService.list(new QueryWrapper<DwdRecipeMedicineItemInfo>()
                .in("patient_id",patientIds)
        );
        return data.stream().map(PatientHealthyHelper::getDwdRecipeMedicineItemVO).collect(Collectors.toList());
    }

    @Override
    public DwdRecipeMedicineItemVO getById(String id) {
        DwdRecipeMedicineItemInfo entity = dwdRecipeMedicineItemInfoService.getById(id);
        return PatientHealthyHelper.getDwdRecipeMedicineItemVO(entity);
    }

    @Override
    public List<DwdRecipeMedicineItemInfo> listBeanByPatient(String patientId) {
        List<DwdRecipeMedicineItemInfo> data = dwdRecipeMedicineItemInfoService.list(new QueryWrapper<DwdRecipeMedicineItemInfo>()
                .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdRecipeMedicineItemInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdRecipeMedicineItemInfo> data = dwdRecipeMedicineItemInfoService.list(new QueryWrapper<DwdRecipeMedicineItemInfo>()
                .in("patient_id",patientIds)
        );
        return data;
    }

}
