package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdRecipeTherapyInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdRecipeTherapyInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdRecipeTherapyItemQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdRecipeTherapyQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdRecipeTherapyItemVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdRecipeTherapyVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdRecipeTherapyBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdRecipeTherapyItemBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@DS("wd")
@Service
public class DwdRecipeTherapyBizServiceImpl implements DwdRecipeTherapyBizService {

    @Autowired
    private DwdRecipeTherapyInfoService dwdRecipeTherapyInfoService;
    @Autowired
    private DwdRecipeTherapyItemBizService dwdRecipeTherapyItemBizService;

    @Override
    public List<DwdRecipeTherapyVO> list(DwdRecipeTherapyQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<DwdRecipeTherapyInfo> data = dwdRecipeTherapyInfoService.list(new QueryWrapper<DwdRecipeTherapyInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .ge(null != params.getStartDate(), "recipe_time", params.getStartDate())
                .lt(null != params.getEndDate(), "recipe_time", params.getEndDate())
                .orderByDesc("recipe_time")
                .last(" limit 1000 ") // TODO 防止不传参数查询所有导致爆库
        );
        List<DwdRecipeTherapyVO> list = data.stream().map(PatientHealthyHelper::getDwdRecipeTherapyVO).collect(Collectors.toList());

        // 治疗明细
        if (CollectionUtil.isNotEmpty(list)) {
            List<String> therapyIds = list.stream().map(DwdRecipeTherapyVO::getId).collect(Collectors.toList());
            Map<String, List<DwdRecipeTherapyItemVO>> dwdRecipeTherapyItemVOMap = this.getDwdRecipeTherapyItemVOMap(params.getPatientId(), therapyIds);
            if (CollectionUtil.isNotEmpty(dwdRecipeTherapyItemVOMap)) {
                list.forEach(i -> i.setDwdRecipeTherapyItemVOs(dwdRecipeTherapyItemVOMap.get(i.getId())));
            }
        }

        return list;
    }

    // 治疗明细
    private Map<String, List<DwdRecipeTherapyItemVO>> getDwdRecipeTherapyItemVOMap(String patientId, List<String> therapyIds) {
        DwdRecipeTherapyItemQueryDTO queryDTO = new DwdRecipeTherapyItemQueryDTO();
        queryDTO.setPatientId(patientId);
        queryDTO.setTherapyIds(therapyIds);
        List<DwdRecipeTherapyItemVO> voList = dwdRecipeTherapyItemBizService.list(queryDTO);

        Map<String, List<DwdRecipeTherapyItemVO>> voMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(voList)) {
            voList.forEach(i -> {
                List<DwdRecipeTherapyItemVO> vos = voMap.get(i.getTherapyId());
                if (null == vos) {
                    vos = new ArrayList<>();
                    voMap.put(i.getTherapyId(), vos);
                }
                vos.add(i);
            });
        }
        return voMap;
    }

    @Override
    public List<DwdRecipeTherapyInfo> listByPatientId(String patientId) {
        List<DwdRecipeTherapyInfo> data = dwdRecipeTherapyInfoService.list(
                new QueryWrapper<DwdRecipeTherapyInfo>()
                        .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdRecipeTherapyInfo> listByPatientId(List<String> patientIds) {
        List<DwdRecipeTherapyInfo> data = dwdRecipeTherapyInfoService.list(
                new QueryWrapper<DwdRecipeTherapyInfo>()
                        .in("patient_id",patientIds)
        );
        return data;
    }

    @Override
    public DwdRecipeTherapyInfo getById(String id) {
        return dwdRecipeTherapyInfoService.getById(id);
    }

    @Override
    public List<DwdRecipeTherapyInfo> listBeanByPatient(String patientId) {
        List<DwdRecipeTherapyInfo> data = dwdRecipeTherapyInfoService.list(
                new QueryWrapper<DwdRecipeTherapyInfo>()
                        .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdRecipeTherapyInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdRecipeTherapyInfo> data = dwdRecipeTherapyInfoService.list(
                new QueryWrapper<DwdRecipeTherapyInfo>()
                        .in("patient_id",patientIds)
        );
        return data;
    }

}
