package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.entity.DwdRecipeTherapyItemInfo;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.base.service.DwdRecipeTherapyItemInfoService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdRecipeTherapyItemQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdRecipeTherapyItemVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.helper.PatientHealthyHelper;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdRecipeTherapyItemBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@DS("wd")
@Service
public class DwdRecipeTherapyItemBizServiceImpl implements DwdRecipeTherapyItemBizService {

    @Autowired
    private DwdRecipeTherapyItemInfoService dwdRecipeTherapyItemInfoService;

    @Override
    public List<DwdRecipeTherapyItemVO> list(DwdRecipeTherapyItemQueryDTO params) {
        if (null != params.getEndDate()) {
            params.setEndDate(DateUtil.offsetDay(params.getEndDate(), 1));
        }

        List<DwdRecipeTherapyItemInfo> data = dwdRecipeTherapyItemInfoService.list(new QueryWrapper<DwdRecipeTherapyItemInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollectionUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollectionUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotBlank(params.getTherapyId()), "therapy_id", params.getTherapyId())
                .in(CollectionUtil.isNotEmpty(params.getTherapyIds()), "therapy_id", params.getTherapyIds())
                .eq(StrUtil.isNotBlank(params.getVisitId()), "visit_id", params.getVisitId())
                .in(CollectionUtil.isNotEmpty(params.getVisitIds()), "visit_id", params.getVisitIds())
                .ge(null != params.getStartDate(), "recipe_time", params.getStartDate())
                .lt(null != params.getEndDate(), "recipe_time", params.getEndDate())
                .orderByDesc("recipe_time")
                .last(" limit 1000 ") // TODO 防止不传参数查询所有导致爆库
        );
        return data.stream().map(PatientHealthyHelper::getDwdRecipeTherapyItemVO).collect(Collectors.toList());
    }

    @Override
    public List<DwdRecipeTherapyItemInfo> listBeanByPatient(String patientId) {
        List<DwdRecipeTherapyItemInfo> data = dwdRecipeTherapyItemInfoService.list(
                new QueryWrapper<DwdRecipeTherapyItemInfo>()
                        .eq("patient_id",patientId)
        );
        return data;
    }

    @Override
    public List<DwdRecipeTherapyItemInfo> listBeanByPatient(List<String> patientIds) {
        List<DwdRecipeTherapyItemInfo> data = dwdRecipeTherapyItemInfoService.list(
                new QueryWrapper<DwdRecipeTherapyItemInfo>()
                        .in("patient_id",patientIds)
        );
        return data;
    }

}
