package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.dw.healthy.data.client.api.constant.DwdConstant;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdInpatientRecordQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdInpatientReserveQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdPatientDataQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdExaminationPatientVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdInpatientRecordVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdInpatientReserveVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdVisitRecordVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DeptRelationBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdInpatientRecordBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdInpatientReserveBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdPatientDataBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.PatientDwdDataAsyncBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
@Async("archivesExecutor")
public class PatientDwdDataAsyncBizServiceImpl implements PatientDwdDataAsyncBizService {

    @Autowired
    private DwdPatientDataBizService dwdPatientDataBizService;
    @Autowired
    private DwdInpatientRecordBizService dwdInpatientRecordBizService;
    @Autowired
    private DeptRelationBizService deptRelationBizService;
    @Autowired
    private DwdInpatientReserveBizService dwdInpatientReserveBizService;

    // 住院申请与住院的最大间隔时间
    @Value("${xx.inhospital.reserve:7}")
    private Integer reserveDay;

    @Override
    public Future<List<DwdVisitRecordVO>> visitRecordList(DwdPatientDataQueryDTO params) {
        // 就诊信息
        List<DwdVisitRecordVO> dwdVisitRecordVOs = dwdPatientDataBizService.visitRecordList(params);
        return new AsyncResult<>(dwdVisitRecordVOs);
    }


    @Override
    public Future<List<DwdInpatientRecordVO>> inpatientRecordList(DwdPatientDataQueryDTO params) {
        // 处理返回的住院申请信息
        List<DwdInpatientRecordVO> list = dwdPatientDataBizService.inpatientRecordList(params);

        // 其他类型数据直接返回
        if (StrUtil.isNotBlank(params.getDataType()) && !DwdConstant.DataTypeEnum.InpatientReserve.getCode().equals(params.getDataType())) {
            return new AsyncResult<>(list);
        }

        // 住院申请类型处理
        if (DwdConstant.DataTypeEnum.InpatientReserve.getCode().equals(params.getDataType())) {
            // 查询住院信息
            DwdInpatientRecordQueryDTO dwdInpatientRecordQueryDTO = new DwdInpatientRecordQueryDTO();
            dwdInpatientRecordQueryDTO.setPatientId(params.getPatientId());
            dwdInpatientRecordQueryDTO.setPatientIds(params.getPatientIds());
            dwdInpatientRecordQueryDTO.setStartDate(params.getStartDate());
            dwdInpatientRecordQueryDTO.setEndDate(params.getEndDate());
            list = dwdInpatientRecordBizService.simpleList(dwdInpatientRecordQueryDTO);
        }

        if (CollectionUtil.isEmpty(list)) {
            return new AsyncResult<>(new ArrayList<>(0));
        }

        this.inpatientDeptHandler(params, list);

        return new AsyncResult<>(list);
    }

    private void inpatientDeptHandler(DwdPatientDataQueryDTO params, List<DwdInpatientRecordVO> list) {

        // 住院科室
        List<String> inpatientRecordDeptIds = new ArrayList<>(); 
        list.forEach(i -> {
            inpatientRecordDeptIds.add(i.getAdmitDepartmentCodeStandard());
        });

        // 根据科室查询对应的科室：key为住院科室id，value为申请科室id列表
        log.info("1.根据科室查询对应的科室>>>>>> {}", JSON.toJSONString(inpatientRecordDeptIds));
        Map<String, List<String>> deptIdMap = deptRelationBizService.listByDeptIds(inpatientRecordDeptIds);
        log.info("2.根据科室查询对应的科室<<<<<< {}", JSON.toJSONString(deptIdMap));

        List<String> deptIds = new ArrayList<>();
        deptIdMap.forEach((k, v) -> deptIds.addAll(v));
        // 住院预约信息
        DwdInpatientReserveQueryDTO queryDTO = new DwdInpatientReserveQueryDTO();
        queryDTO.setPatientId(params.getPatientId());
        queryDTO.setPatientIds(params.getPatientIds());
        queryDTO.setApplyDepartmentCodeStandards(deptIds);
        if (StrUtil.isNotBlank(params.getStartDate())) {
            queryDTO.setStartDate(DateUtil.offsetDay(DateUtil.parseDate(params.getStartDate()), -reserveDay).toString()); // 申请时间比住院时间多往前查七天
        }
        queryDTO.setEndDate(params.getEndDate());
        log.info("3.住院预约信息>>>>>> {}", JSON.toJSONString(queryDTO));
        List<DwdInpatientReserveVO> voList = dwdInpatientReserveBizService.list(queryDTO);
        log.info("4.住院预约信息<<<<<< {}", JSON.toJSONString(voList));
        // 整理成：key为申请科室id，value为住院申请信息列表
        Map<String, List<DwdInpatientReserveVO>> voMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(voList)) {
            voList.forEach(i -> {
                List<DwdInpatientReserveVO> l = voMap.get(i.getApplyDepartmentCodeStandard());
                if (null == l) {
                    l = new ArrayList<>();
                    voMap.put(i.getApplyDepartmentCodeStandard(), l);
                }
                l.add(i);
            });
        }
        log.info("5.住院预约信息>>>>>> {}", JSON.toJSONString(voMap));

        // 整理成：key为住院科室id，value为住院申请信息列表
        Map<String, List<DwdInpatientReserveVO>> recordVoMap = new HashMap<>(); 
        deptIdMap.forEach((k, v) -> {
            List<DwdInpatientReserveVO> reserveVOList = new ArrayList<>();
            if (CollectionUtil.isNotEmpty(v)) {
                v.forEach(i -> {
                    List<DwdInpatientReserveVO> l = voMap.get(i);
                    if (null != l) {
                        reserveVOList.addAll(l);
                    }
                });
            }
            recordVoMap.put(k, reserveVOList);
        });

        log.info("6.住院申请信息>>>>>> {}", JSON.toJSONString(recordVoMap));

        // 把住院申请信息放到住院信息中
        list.forEach(i -> {
            List<DwdInpatientReserveVO> vos = recordVoMap.get(i.getAdmitDepartmentCodeStandard());
            if (CollectionUtil.isNotEmpty(vos)) {
                List<DwdInpatientReserveVO> dwdInpatientReserveVOs = new ArrayList<>(); // 筛选出时间匹配的数据（默认7天内）

                vos.forEach(j -> {
                    if (j.getApplyDate().before(i.getInpatientDate()) && j.getApplyDate().after(DateUtil.offsetDay(i.getInpatientDate(), -reserveDay))) {
                        dwdInpatientReserveVOs.add(j);
                    }
                });

                i.setDwdInpatientReserveVOs(dwdInpatientReserveVOs);
            }
        });
    }

    @Override
    public Future<List<DwdExaminationPatientVO>> examinationPatientList(DwdPatientDataQueryDTO params) {
        // 体检信息
        List<DwdExaminationPatientVO> dwdExaminationPatientVOs = dwdPatientDataBizService.examinationPatientList(params);
        return new AsyncResult<>(dwdExaminationPatientVOs);
    }

}
