package com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.impl;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.dw.healthy.data.client.api.constant.DwdConstant;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdInpatientRecordQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdInpatientReserveQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.dto.DwdPatientDataQueryDTO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdExaminationPatientVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdInpatientRecordVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdInpatientReserveVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdPatientArchiveDataVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdPatientDataVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.bean.vo.DwdVisitRecordVO;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DeptRelationBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdInpatientRecordBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdInpatientReserveBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdPatientBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.DwdPatientDataBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.PatientDwdDataAsyncBizService;
import com.xxh.cloud.dw.healthy.data.dwd.framework.modules.biz.service.PatientDwdDataBizService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;
import cn.hutool.core.date.DateUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;

@Slf4j
@Service
public class PatientDwdDataBizServiceImpl implements PatientDwdDataBizService {

    @Autowired
    private DeptRelationBizService deptRelationBizService;
    @Autowired
    private DwdPatientDataBizService dwdPatientDataBizService;
    @Autowired
    private DwdInpatientReserveBizService dwdInpatientReserveBizService;
    @Autowired
    private DwdInpatientRecordBizService dwdInpatientRecordBizService;
    @Autowired
    private DwdPatientBizService dwdPatientBizService;
    @Autowired
    private PatientDwdDataAsyncBizService patientDwdDataAsyncBizService;

    // 住院申请与住院的最大间隔时间
    @Value("${xx.inhospital.reserve:7}")
    private Integer reserveDay;

    @Override
    public List<DwdPatientDataVO> list(DwdPatientDataQueryDTO params) {
        log.info("PatientDwdDataBizServiceImpl.list::params >>>>>> {}", JSON.toJSONString(params));

        // 患者id转化
        if (StrUtil.isNotBlank(params.getPatientId())) {
            List<String> patientIds = dwdPatientBizService.getPatientIdByMainPersonId(params.getPatientId());
            if (CollectionUtil.isEmpty(patientIds)) {
                return new ArrayList<>(0);
            }
            params.setPatientId(null);
            params.setPatientIds(patientIds);
        }
        
        // 就诊信息
        List<DwdVisitRecordVO> dwdVisitRecordVOs = dwdPatientDataBizService.visitRecordList(params);
        // 住院信息
        List<DwdInpatientRecordVO> dwdInpatientRecordVOs = this.inpatientRecordList(params);

        if (CollectionUtil.isEmpty(dwdVisitRecordVOs) && CollectionUtil.isEmpty(dwdInpatientRecordVOs)) {
            return new ArrayList<>(0);
        }

        // 合并信息
        List<DwdPatientDataVO> list = new ArrayList<>(dwdVisitRecordVOs.size() + dwdInpatientRecordVOs.size());

        int visitRecordIndex = 0;
        int inpatientRecordIndex = 0;
        while (visitRecordIndex < dwdVisitRecordVOs.size() && inpatientRecordIndex < dwdInpatientRecordVOs.size()) {
            if (dwdVisitRecordVOs.get(visitRecordIndex).getVisitDate().after(dwdInpatientRecordVOs.get(inpatientRecordIndex).getInpatientDate())) {
                DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                dwdPatientDataVO.setDwdVisitRecordVO(dwdVisitRecordVOs.get(visitRecordIndex));
                list.add(dwdPatientDataVO);
                
                visitRecordIndex++;
            } else {
                DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                dwdPatientDataVO.setDwdInpatientRecordVO(dwdInpatientRecordVOs.get(inpatientRecordIndex));
                list.add(dwdPatientDataVO);

                inpatientRecordIndex++;
            }
        }

        while (visitRecordIndex < dwdVisitRecordVOs.size()) {
            DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
            dwdPatientDataVO.setDwdVisitRecordVO(dwdVisitRecordVOs.get(visitRecordIndex));
            list.add(dwdPatientDataVO);

            visitRecordIndex++;
        }

        while (inpatientRecordIndex < dwdInpatientRecordVOs.size()) {
            DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
            dwdPatientDataVO.setDwdInpatientRecordVO(dwdInpatientRecordVOs.get(inpatientRecordIndex));
            list.add(dwdPatientDataVO);

            inpatientRecordIndex++;
        }

        return list;
    }

    @Override
    public List<DwdPatientDataVO> listV2(DwdPatientDataQueryDTO params) {
        log.info("DwdPatientDataBizServiceImpl.listV2::params >>>>>> {}", JSON.toJSONString(params));
        
        // 就诊信息
        List<DwdVisitRecordVO> dwdVisitRecordVOs = dwdPatientDataBizService.visitRecordList(params);
        // 住院信息
        List<DwdInpatientRecordVO> dwdInpatientRecordVOs = this.inpatientRecordList(params);
        // 体检信息
        List<DwdExaminationPatientVO> dwdExaminationPatientVOs = dwdPatientDataBizService.examinationPatientList(params);

        if (CollectionUtil.isEmpty(dwdVisitRecordVOs) && CollectionUtil.isEmpty(dwdInpatientRecordVOs) && CollectionUtil.isEmpty(dwdExaminationPatientVOs)) {
            return new ArrayList<>(0);
        }

        // 合并信息
        List<DwdPatientDataVO> list = new ArrayList<>(dwdVisitRecordVOs.size() + dwdInpatientRecordVOs.size());

        int visitRecordIndex = 0;
        int inpatientRecordIndex = 0;
        int examinationPatientIndex = 0;
        while (true) {
            if (visitRecordIndex >= dwdVisitRecordVOs.size() && inpatientRecordIndex >= dwdInpatientRecordVOs.size() && examinationPatientIndex >= dwdExaminationPatientVOs.size()) {
                break;
            }

            Date visitDate = visitRecordIndex < dwdVisitRecordVOs.size() ? dwdVisitRecordVOs.get(visitRecordIndex).getVisitDate() : null;
            Date inpatientDate = inpatientRecordIndex < dwdInpatientRecordVOs.size() ? dwdInpatientRecordVOs.get(inpatientRecordIndex).getInpatientDate() : null;
            Date examinationDate = examinationPatientIndex < dwdExaminationPatientVOs.size() ? dwdExaminationPatientVOs.get(examinationPatientIndex).getDataCreateDate() : null;
            
            if (visitDate == null && inpatientDate == null && examinationDate == null) {
                break;
            }

            if (null != visitDate && null != inpatientDate && null != examinationDate) {
                if (visitDate.after(inpatientDate) && visitDate.after(examinationDate)) {
                    DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                    dwdPatientDataVO.setDwdVisitRecordVO(dwdVisitRecordVOs.get(visitRecordIndex));
                    list.add(dwdPatientDataVO);
                    
                    visitRecordIndex++;
                } else if (inpatientDate.after(visitDate) && inpatientDate.after(examinationDate)) {
                    DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                    dwdPatientDataVO.setDwdInpatientRecordVO(dwdInpatientRecordVOs.get(inpatientRecordIndex));
                    list.add(dwdPatientDataVO);

                    inpatientRecordIndex++;
                } else {
                    DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                    dwdPatientDataVO.setDwdExaminationPatientVO(dwdExaminationPatientVOs.get(examinationPatientIndex));
                    list.add(dwdPatientDataVO);

                    examinationPatientIndex++;
                }
            } else if (null != inpatientDate && null != examinationDate) {
                if (inpatientDate.after(examinationDate)) {
                    DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                    dwdPatientDataVO.setDwdInpatientRecordVO(dwdInpatientRecordVOs.get(inpatientRecordIndex));
                    list.add(dwdPatientDataVO);

                    inpatientRecordIndex++;
                } else {
                    DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                    dwdPatientDataVO.setDwdExaminationPatientVO(dwdExaminationPatientVOs.get(examinationPatientIndex));
                    list.add(dwdPatientDataVO);

                    examinationPatientIndex++;
                }
            } else if (null != visitDate && null != examinationDate) {
                if (visitDate.after(examinationDate)) {
                    DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                    dwdPatientDataVO.setDwdVisitRecordVO(dwdVisitRecordVOs.get(visitRecordIndex));
                    list.add(dwdPatientDataVO);
                    
                    visitRecordIndex++;
                } else {
                    DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                    dwdPatientDataVO.setDwdExaminationPatientVO(dwdExaminationPatientVOs.get(examinationPatientIndex));
                    list.add(dwdPatientDataVO);

                    examinationPatientIndex++;
                }
            } else if (null != visitDate && null != inpatientDate) {
                if (visitDate.after(inpatientDate)) {
                    DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                    dwdPatientDataVO.setDwdVisitRecordVO(dwdVisitRecordVOs.get(visitRecordIndex));
                    list.add(dwdPatientDataVO);
                    
                    visitRecordIndex++;
                } else {
                    DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                    dwdPatientDataVO.setDwdInpatientRecordVO(dwdInpatientRecordVOs.get(inpatientRecordIndex));
                    list.add(dwdPatientDataVO);

                    inpatientRecordIndex++;
                }
            } else if (null != visitDate) {
                DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                dwdPatientDataVO.setDwdVisitRecordVO(dwdVisitRecordVOs.get(visitRecordIndex));
                list.add(dwdPatientDataVO);
                
                visitRecordIndex++;
            } else if (null != inpatientDate) {
                DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                dwdPatientDataVO.setDwdInpatientRecordVO(dwdInpatientRecordVOs.get(inpatientRecordIndex));
                list.add(dwdPatientDataVO);

                inpatientRecordIndex++;
            } else {
                DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                dwdPatientDataVO.setDwdExaminationPatientVO(dwdExaminationPatientVOs.get(examinationPatientIndex));
                list.add(dwdPatientDataVO);

                examinationPatientIndex++;
            }
        }

        return list;
    }

    // 住院数据
    private List<DwdInpatientRecordVO> inpatientRecordList(DwdPatientDataQueryDTO params) {
        // 处理返回的住院申请信息
        List<DwdInpatientRecordVO> list = dwdPatientDataBizService.inpatientRecordList(params);

        // 其他类型数据直接返回
        if (StrUtil.isNotBlank(params.getDataType()) && !DwdConstant.DataTypeEnum.InpatientReserve.getCode().equals(params.getDataType())) {
            return list;
        }

        // 住院申请类型处理
        if (DwdConstant.DataTypeEnum.InpatientReserve.getCode().equals(params.getDataType())) {
            // 查询住院信息
            DwdInpatientRecordQueryDTO dwdInpatientRecordQueryDTO = new DwdInpatientRecordQueryDTO();
            dwdInpatientRecordQueryDTO.setPatientId(params.getPatientId());
            dwdInpatientRecordQueryDTO.setPatientIds(params.getPatientIds());
            dwdInpatientRecordQueryDTO.setStartDate(params.getStartDate());
            dwdInpatientRecordQueryDTO.setEndDate(params.getEndDate());
            list = dwdInpatientRecordBizService.simpleList(dwdInpatientRecordQueryDTO);
        }

        if (CollectionUtil.isEmpty(list)) {
            return list;
        }

        // 住院科室
        List<String> inpatientRecordDeptIds = new ArrayList<>(); 
        list.forEach(i -> {
            inpatientRecordDeptIds.add(i.getAdmitDepartmentCodeStandard());
        });

        // 根据科室查询对应的科室：key为住院科室id，value为申请科室id列表
        log.info("1.根据科室查询对应的科室>>>>>> {}", JSON.toJSONString(inpatientRecordDeptIds));
        Map<String, List<String>> deptIdMap = deptRelationBizService.listByDeptIds(inpatientRecordDeptIds);
        log.info("2.根据科室查询对应的科室<<<<<< {}", JSON.toJSONString(deptIdMap));

        List<String> deptIds = new ArrayList<>();
        deptIdMap.forEach((k, v) -> deptIds.addAll(v));
        // 住院预约信息
        DwdInpatientReserveQueryDTO queryDTO = new DwdInpatientReserveQueryDTO();
        queryDTO.setPatientId(params.getPatientId());
        queryDTO.setPatientIds(params.getPatientIds());
        queryDTO.setApplyDepartmentCodeStandards(deptIds);
        if (StrUtil.isNotBlank(params.getStartDate())) {
            queryDTO.setStartDate(DateUtil.offsetDay(DateUtil.parseDate(params.getStartDate()), -reserveDay).toString()); // 申请时间比住院时间多往前查七天
        }
        queryDTO.setEndDate(params.getEndDate());
        log.info("3.住院预约信息>>>>>> {}", JSON.toJSONString(queryDTO));
        List<DwdInpatientReserveVO> voList = dwdInpatientReserveBizService.list(queryDTO);
        log.info("4.住院预约信息<<<<<< {}", JSON.toJSONString(voList));
        // 整理成：key为申请科室id，value为住院申请信息列表
        Map<String, List<DwdInpatientReserveVO>> voMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(voList)) {
            voList.forEach(i -> {
                List<DwdInpatientReserveVO> l = voMap.get(i.getApplyDepartmentCodeStandard());
                if (null == l) {
                    l = new ArrayList<>();
                    voMap.put(i.getApplyDepartmentCodeStandard(), l);
                }
                l.add(i);
            });
        }
        log.info("5.住院预约信息>>>>>> {}", JSON.toJSONString(voMap));

        // 整理成：key为住院科室id，value为住院申请信息列表
        Map<String, List<DwdInpatientReserveVO>> recordVoMap = new HashMap<>(); 
        deptIdMap.forEach((k, v) -> {
            List<DwdInpatientReserveVO> reserveVOList = new ArrayList<>();
            if (CollectionUtil.isNotEmpty(v)) {
                v.forEach(i -> {
                    List<DwdInpatientReserveVO> l = voMap.get(i);
                    if (null != l) {
                        reserveVOList.addAll(l);
                    }
                });
            }
            recordVoMap.put(k, reserveVOList);
        });

        log.info("6.住院申请信息>>>>>> {}", JSON.toJSONString(recordVoMap));

        // 把住院申请信息放到住院信息中
        list.forEach(i -> {
            List<DwdInpatientReserveVO> vos = recordVoMap.get(i.getAdmitDepartmentCodeStandard());
            if (CollectionUtil.isNotEmpty(vos)) {
                List<DwdInpatientReserveVO> dwdInpatientReserveVOs = new ArrayList<>(); // 筛选出时间匹配的数据（默认7天内）

                vos.forEach(j -> {
                    if (j.getApplyDate().before(i.getInpatientDate()) && j.getApplyDate().after(DateUtil.offsetDay(i.getInpatientDate(), -reserveDay))) {
                        dwdInpatientReserveVOs.add(j);
                    }
                });

                i.setDwdInpatientReserveVOs(dwdInpatientReserveVOs);
            }
        });

        return list;
    }

    @Override
    public List<DwdPatientDataVO> listV3(DwdPatientDataQueryDTO params) {
        log.info("DwdPatientDataBizServiceImpl.listV3::params >>>>>> {}", JSON.toJSONString(params));
        
        // 就诊信息
        Future<List<DwdVisitRecordVO>> dwdVisitRecordList = patientDwdDataAsyncBizService.visitRecordList(params);
        // 住院信息
        Future<List<DwdInpatientRecordVO>> dwdInpatientRecordList = patientDwdDataAsyncBizService.inpatientRecordList(params);
        // 体检信息
        Future<List<DwdExaminationPatientVO>> dwdExaminationPatientList = patientDwdDataAsyncBizService.examinationPatientList(params);

        try {
            // 就诊信息
            List<DwdVisitRecordVO> dwdVisitRecordVOs = dwdVisitRecordList.get();
            // 住院信息
            List<DwdInpatientRecordVO> dwdInpatientRecordVOs = dwdInpatientRecordList.get();
            // 体检信息
            List<DwdExaminationPatientVO> dwdExaminationPatientVOs = dwdExaminationPatientList.get();


            if (CollectionUtil.isEmpty(dwdVisitRecordVOs) && CollectionUtil.isEmpty(dwdInpatientRecordVOs) && CollectionUtil.isEmpty(dwdExaminationPatientVOs)) {
                return new ArrayList<>(0);
            }

            // 合并信息
            List<DwdPatientDataVO> list = new ArrayList<>(dwdVisitRecordVOs.size() + dwdInpatientRecordVOs.size());

            int visitRecordIndex = 0;
            int inpatientRecordIndex = 0;
            int examinationPatientIndex = 0;
            while (true) {
                if (visitRecordIndex >= dwdVisitRecordVOs.size() && inpatientRecordIndex >= dwdInpatientRecordVOs.size() && examinationPatientIndex >= dwdExaminationPatientVOs.size()) {
                    break;
                }

                Date visitDate = visitRecordIndex < dwdVisitRecordVOs.size() ? dwdVisitRecordVOs.get(visitRecordIndex).getVisitDate() : null;
                Date inpatientDate = inpatientRecordIndex < dwdInpatientRecordVOs.size() ? dwdInpatientRecordVOs.get(inpatientRecordIndex).getInpatientDate() : null;
                Date examinationDate = examinationPatientIndex < dwdExaminationPatientVOs.size() ? dwdExaminationPatientVOs.get(examinationPatientIndex).getDataCreateDate() : null;
                
                if (visitDate == null && inpatientDate == null && examinationDate == null) {
                    break;
                }

                if (null != visitDate && null != inpatientDate && null != examinationDate) {
                    if (visitDate.after(inpatientDate) && visitDate.after(examinationDate)) {
                        DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                        dwdPatientDataVO.setDwdVisitRecordVO(dwdVisitRecordVOs.get(visitRecordIndex));
                        list.add(dwdPatientDataVO);
                        
                        visitRecordIndex++;
                    } else if (inpatientDate.after(visitDate) && inpatientDate.after(examinationDate)) {
                        DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                        dwdPatientDataVO.setDwdInpatientRecordVO(dwdInpatientRecordVOs.get(inpatientRecordIndex));
                        list.add(dwdPatientDataVO);

                        inpatientRecordIndex++;
                    } else {
                        DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                        dwdPatientDataVO.setDwdExaminationPatientVO(dwdExaminationPatientVOs.get(examinationPatientIndex));
                        list.add(dwdPatientDataVO);

                        examinationPatientIndex++;
                    }
                } else if (null != inpatientDate && null != examinationDate) {
                    if (inpatientDate.after(examinationDate)) {
                        DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                        dwdPatientDataVO.setDwdInpatientRecordVO(dwdInpatientRecordVOs.get(inpatientRecordIndex));
                        list.add(dwdPatientDataVO);

                        inpatientRecordIndex++;
                    } else {
                        DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                        dwdPatientDataVO.setDwdExaminationPatientVO(dwdExaminationPatientVOs.get(examinationPatientIndex));
                        list.add(dwdPatientDataVO);

                        examinationPatientIndex++;
                    }
                } else if (null != visitDate && null != examinationDate) {
                    if (visitDate.after(examinationDate)) {
                        DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                        dwdPatientDataVO.setDwdVisitRecordVO(dwdVisitRecordVOs.get(visitRecordIndex));
                        list.add(dwdPatientDataVO);
                        
                        visitRecordIndex++;
                    } else {
                        DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                        dwdPatientDataVO.setDwdExaminationPatientVO(dwdExaminationPatientVOs.get(examinationPatientIndex));
                        list.add(dwdPatientDataVO);

                        examinationPatientIndex++;
                    }
                } else if (null != visitDate && null != inpatientDate) {
                    if (visitDate.after(inpatientDate)) {
                        DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                        dwdPatientDataVO.setDwdVisitRecordVO(dwdVisitRecordVOs.get(visitRecordIndex));
                        list.add(dwdPatientDataVO);
                        
                        visitRecordIndex++;
                    } else {
                        DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                        dwdPatientDataVO.setDwdInpatientRecordVO(dwdInpatientRecordVOs.get(inpatientRecordIndex));
                        list.add(dwdPatientDataVO);

                        inpatientRecordIndex++;
                    }
                } else if (null != visitDate) {
                    DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                    dwdPatientDataVO.setDwdVisitRecordVO(dwdVisitRecordVOs.get(visitRecordIndex));
                    list.add(dwdPatientDataVO);
                    
                    visitRecordIndex++;
                } else if (null != inpatientDate) {
                    DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                    dwdPatientDataVO.setDwdInpatientRecordVO(dwdInpatientRecordVOs.get(inpatientRecordIndex));
                    list.add(dwdPatientDataVO);

                    inpatientRecordIndex++;
                } else {
                    DwdPatientDataVO dwdPatientDataVO = new DwdPatientDataVO();
                    dwdPatientDataVO.setDwdExaminationPatientVO(dwdExaminationPatientVOs.get(examinationPatientIndex));
                    list.add(dwdPatientDataVO);

                    examinationPatientIndex++;
                }
            }

            return list;
        } catch (Exception e) {
            log.error("异步查询档案信息异常 >>>>>> {}", e.getMessage());
            e.printStackTrace();
        }

        return new ArrayList<>(0);
    }

    @Override
    public List<DwdPatientDataVO> patientDataMergeList(DwdPatientDataQueryDTO params) {
        List<String> patientIds = dwdPatientBizService.getPatientIdByMainPersonId(params.getMainPersonId());
        if (CollectionUtil.isEmpty(patientIds)) {
            return new ArrayList<>(0);
        }
        params.setPatientIds(patientIds);

        return this.listV2(params);
    }

    @Override
    public DwdPatientArchiveDataVO patientDataMerge(DwdPatientDataQueryDTO params) {
        List<String> patientIds = dwdPatientBizService.getPatientIdByMainPersonId(params.getMainPersonId());
        if (CollectionUtil.isEmpty(patientIds)) {
            return null;
        }
        params.setPatientIds(patientIds);

        List<DwdPatientDataVO> list = this.listV2(params);

        DwdPatientArchiveDataVO vo = new DwdPatientArchiveDataVO();
        vo.setDwdPatientDataVOList(list);
        vo.setPatientIds(patientIds);

        return vo;
    }

    @Override
    public DwdPatientArchiveDataVO patientDataMergeV3(DwdPatientDataQueryDTO params) {
        List<String> patientIds = dwdPatientBizService.getPatientIdByMainPersonId(params.getMainPersonId());
        if (CollectionUtil.isEmpty(patientIds)) {
            return null;
        }
        params.setPatientIds(patientIds);

        List<DwdPatientDataVO> list = this.listV3(params);

        DwdPatientArchiveDataVO vo = new DwdPatientArchiveDataVO();
        vo.setDwdPatientDataVOList(list);
        vo.setPatientIds(patientIds);

        return vo;
    }

    @Override
    public List<DwdPatientDataVO> patientDataMergeListV3(DwdPatientDataQueryDTO params) {
        List<String> patientIds = dwdPatientBizService.getPatientIdByMainPersonId(params.getMainPersonId());
        if (CollectionUtil.isEmpty(patientIds)) {
            return new ArrayList<>(0);
        }
        params.setPatientIds(patientIds);

        return this.listV3(params);
    }

}
